/*
 *      dxirc.cpp
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifdef WIN32
  #define WIN32_LEAN_AND_MEAN
  #include <windows.h>
  #include <shellapi.h>
#else
  #include <sys/file.h>
  #include <unistd.h>
  #include <stdlib.h>
#endif //WIN32
#include <QApplication>
#include <QDesktopServices>
#include "dxirc.h"
#include "config.h"
#include "i18n.h"
#include "preferences.h"
#include "dxutils.h"
#include "dialogs/aboutdialog.h"
#include "dialogs/connectdialog.h"
#include "tabitems/irctabitem.h"
#include "tabitems/dcctabitem.h"
#include "tabitems/tetristabitem.h"
#include "tabitems/boatstabitem.h"
#include "dialogs/serverdialog.h"
#include "dialogs/aliasdialog.h"
#include "dialogs/helpdialog.h"
#include "dialogs/scriptdialog.h"
#include "dialogs/configdialog.h"
#include "dialogs/listdialog.h"
#ifdef HAVE_LUA
extern "C" {
#include <lua.h>
#include <lualib.h>
#include <lauxlib.h>
}
#endif

int argn; //argc for restart
char** args; //argv for restart

#ifndef WIN32
void dxunlink(void)
{
#if(QT_VERSION > 0x050000)
    unlink(QString(QStandardPaths::standardLocations(QStandardPaths::TempLocation).at(0)+"/dxirc-qt-"+getenv("USER")).toLatin1().data());
#else
    unlink(QString(QDesktopServices::storageLocation(QDesktopServices::TempLocation)+"/dxirc-qt-"+getenv("USER")).toAscii().data());
#endif
}
#endif

// For handling Single Instance of Application
bool isInstanceRunningAlready(int argc, char *argv[])
{
#ifdef WIN32
    HANDLE hSingleInstance = NULL;
    hSingleInstance = CreateMutex(NULL,false,TEXT("dxirc-qt.exe"));
    DWORD dwLastError = GetLastError();
    if(dwLastError == ERROR_ALREADY_EXISTS)
    {
        MessageBox(NULL, TEXT("dxirc already running"), TEXT("dxirc"), MB_OK | MB_ICONINFORMATION);
        CloseHandle(hSingleInstance);
        return true;
    }
#else
#if(QT_VERSION > 0x050000)
    QString path = QStandardPaths::standardLocations(QStandardPaths::TempLocation).at(0)+"/dxirc-qt-"+getenv("USER");
#else
    QString path = QDesktopServices::storageLocation(QDesktopServices::TempLocation)+"/dxirc-qt-"+getenv("USER");
#endif
    struct flock fl;
    int fd;
    fd = open(path.toLatin1().data(), O_RDWR | O_CREAT, 0600);
    if(fd < 0)
    {
        perror(path.toLatin1().data());
        _exit(1);
    }
    fl.l_start = 0;
    fl.l_len = 0;
    fl.l_type = F_WRLCK;
    fl.l_whence = SEEK_SET;
    if(fcntl(fd, F_SETLK, &fl) < 0)
    {
        QApplication a(argc, argv);
        QVBoxLayout *layout = new QVBoxLayout;
        layout->addWidget(new QLabel("dxirc already running"));
        QPushButton *button = new QPushButton("&Close");
        QObject::connect(button, SIGNAL(clicked()), qApp, SLOT(quit()));
        layout->addWidget(button);
        QWidget window;
        window.setLayout(layout);
        window.show();
        a.exec();
        _exit(1);
    }
    atexit(dxunlink);
#endif
    return false;
}

dxirc *dxirc::_pThis = NULL;
#ifdef HAVE_LUA
static luaL_Reg dxircFunctions[] = {
    {"AddCommand",      dxirc::onLuaAddCommand},
    {"AddEvent",        dxirc::onLuaAddEvent},
    {"RemoveName",      dxirc::onLuaRemoveName},
    {"Command",         dxirc::onLuaCommand},
    {"Print",           dxirc::onLuaPrint},
    {"GetServers",      dxirc::onLuaGetServers},
    {"GetTab",          dxirc::onLuaGetTab},
    {"GetCurrentTab",   dxirc::onLuaGetCurrentTab},
    {"GetVersion",      dxirc::onLuaGetVersion},
    {"GetTabInfo",      dxirc::onLuaGetTabInfo},
    {"SetTab",          dxirc::onLuaSetTab},
    {"CreateTab",       dxirc::onLuaCreateTab},
    {"GetTabCount",     dxirc::onLuaGetTabCount},
    {"Clear",           dxirc::onLuaClear},
    {"GetChannelNames", dxirc::onLuaGetChannelNames},
    {NULL,              NULL}
};
#endif //HAVE_LUA

//Constructor
dxirc::dxirc(QWidget *parent):
    QMainWindow(parent)
{
    m_lastID = 1; //0 is for FtTabItem
    m_lastToken = 0;
    m_logs = NULL;
    m_versionSocket = NULL;

    resize(preferences.m_w, preferences.m_h);
    move(preferences.m_x, preferences.m_y);
    if(preferences.m_maximized) showMaximized();
    setWindowIcon(QIcon(":/icons/small_dxirc.png"));
    setWindowTitle("dxirc");
    m_tabbook = new TabWidget(this);
    setCentralWidget(m_tabbook);
    m_statusBar = statusBar();
    if(!preferences.m_statusShown) m_statusBar->hide();
    IrcEngine *engine = new IrcEngine("", "");
    m_ircengines.append(engine);
    m_statusTimer = new QTimer(this);
    m_awayTimer = new QTimer(this);

    createActions();
    createMenus();
    createTrayIcon();

    m_trayIcon->show();

    connect(m_trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(slotIconActivated(QSystemTrayIcon::ActivationReason)));
    connect(m_tabbook, SIGNAL(currentChanged(int)), this, SLOT(slotTabChanged(int)));
    connect(m_statusTimer, SIGNAL(timeout()), this, SLOT(slotStatusTime()));
    connect(m_awayTimer, SIGNAL(timeout()), this, SLOT(slotAwayTime()));
    connect(m_ircengines[0], SIGNAL(ircNewChannel(QString)), this, SLOT(slotIrcNewChannel(QString)));
    connect(m_ircengines[0], SIGNAL(ircQuery(QString)), this, SLOT(slotIrcQuery(QString)));
    connect(m_ircengines[0], SIGNAL(ircDccChat(QString,QString,int)), this, SLOT(slotIrcDccChat(QString,QString,int)));
    connect(m_ircengines[0], SIGNAL(ircInvite(QString,QString)), this, SLOT(slotIrcInvite(QString,QString)));
    connect(m_ircengines[0], SIGNAL(ircEndMotd()), this, SLOT(slotIrcEndMotd()));
    connect(m_ircengines[0], SIGNAL(ircPrivmsg(QString,QString,QString)), this, SLOT(slotIrcPrivmsgAndAction(QString,QString,QString)));
    connect(m_ircengines[0], SIGNAL(ircAction(QString,QString,QString)), this, SLOT(slotIrcPrivmsgAndAction(QString,QString,QString)));
    connect(m_ircengines[0], SIGNAL(ircPart(QString,QString,QString)), this, SLOT(slotIrcPart(QString,QString)));
    connect(m_ircengines[0], SIGNAL(ircKick(QString,QString,QString,QString)), this, SLOT(slotIrcKick(QString,QString,QString)));
    connect(m_ircengines[0], SIGNAL(ircRaw(QString)), this, SLOT(slotIrcRaw(QString)));
    connect(m_ircengines[0], SIGNAL(ircDisconnect(QString)), this, SLOT(slotIrcDisconnect(QString)));
    connect(m_ircengines[0], SIGNAL(ircConnect(QString)), this, SLOT(slotIrcConnectAndReconnect(QString)));
    connect(m_ircengines[0], SIGNAL(ircDisconnect(QString)), this, SLOT(slotIrcConnectAndReconnect(QString)));
    connect(m_ircengines[0], SIGNAL(ircQuit(QString,QString)), this, SLOT(slotIrcQuit(QString)));
    connect(m_ircengines[0], SIGNAL(ircDccIn(QString,QString,QString,QString,QString)), this, SLOT(slotIrcDccIn(QString,QString,QString,QString,QString)));
    connect(m_ircengines[0], SIGNAL(ircDccMyToken(QString,int,int)), this, SLOT(slotIrcDccMyToken(QString,int,int)));
    connect(m_ircengines[0], SIGNAL(ircDccToken(QString,QString,QString,int)), this, SLOT(slotIrcDccToken(QString,QString,QString,int)));
    connect(m_ircengines[0], SIGNAL(ircDccAccept(QString,QString,QString,QString)), this, SLOT(slotIrcDccAccept(QString,QString,QString,QString)));
    connect(m_ircengines[0], SIGNAL(ircDccPaccept(int,QString)), this, SLOT(slotIrcDccPaccept(int,QString)));
    connect(m_ircengines[0], SIGNAL(ircDccResume(QString,QString,QString,QString)), this, SLOT(slotIrcDccResume(QString,QString,QString,QString)));
    connect(m_ircengines[0], SIGNAL(ircDccPresume(int,QString)), this, SLOT(slotIrcDccPresume(int,QString)));
    connect(m_ircengines[0], SIGNAL(ircBoatsInvite(QString)), this, SLOT(slotIrcBoatsInvite(QString)));
    connect(m_ircengines[0], SIGNAL(ircBoatsAccept(QString)), this, SLOT(slotIrcBoatsAccept(QString)));
    connect(m_ircengines[0], SIGNAL(irc323()), this, SLOT(slotIrc323()));

    createIrcTab("(server)", SERVER, engine, QIcon(dxutils::getIconForType(SERVER)));
    m_ft = new FtTabItem();
    connect(m_ft, SIGNAL(closeFtTabItem()), this, SLOT(slotCloseFTTab()));
    connect(m_ft, SIGNAL(cancelFt(int)), this, SLOT(slotCancelFT(int)));

    foreach(ServerInfo info, preferences.m_serverList)
    {
        if(info.autoConnect)
            connectServer(info.hostname, info.port, info.passwd, info.nick, info.realname, info.channels, info.commands, info.useSsl);
    }

    _pThis = this;
    autoloadScripts();
    m_awayTimer->start(180000);
}

//Destructor
dxirc::~dxirc()
{
    preferences.m_x = x();
    preferences.m_y = y();
    preferences.m_w = width();
    preferences.m_h = height();
    preferences.m_maximized = isMaximized();
    preferences.save();
    if(m_versionSocket) delete m_versionSocket;
    _pThis = NULL;
}

dxirc* dxirc::instance()
{
    return _pThis;
}

//handle click on close button
void dxirc::closeEvent(QCloseEvent *ev)
{
    if(preferences.m_closeToTray)
        hide();
    else
        qApp->quit();
    ev->ignore();
}

void dxirc::createActions()
{
    QAction *unread = new QAction(this);
    unread->setShortcut(preferences.getBinding("unreadTab"));
    connect(unread, SIGNAL(triggered()), this, SLOT(slotUnreadTab()));
    addAction(unread);
    QAction *next = new QAction(this);
    next->setShortcut(preferences.getBinding("switchTab"));
    connect(next, SIGNAL(triggered()), this, SLOT(slotNextTab()));
    addAction(next);
    QAction *focus = new QAction(this);
    focus->setShortcut(preferences.getBinding("forceFocus"));
    connect(focus, SIGNAL(triggered()), this, SLOT(slotForceFocus()));
    addAction(focus);
    QAction *moveUp = new QAction(this);
    moveUp->setData(true);
    moveUp->setShortcut(preferences.getBinding("moveTabUp"));
    connect(moveUp, SIGNAL(triggered()), this, SLOT(slotMoveTab()));
    addAction(moveUp);
    QAction *moveDown = new QAction(this);
    moveDown->setData(false);
    moveDown->setShortcut(preferences.getBinding("moveTabDown"));
    connect(moveDown, SIGNAL(triggered()), this, SLOT(slotMoveTab()));
    addAction(moveDown);
    QAction *firstTab = new QAction(this);
    firstTab->setData(0);
    firstTab->setShortcut(preferences.getBinding("firstTab"));
    connect(firstTab, SIGNAL(triggered()), this, SLOT(slotSelectTab()));
    addAction(firstTab);
    QAction *secondTab = new QAction(this);
    secondTab->setData(1);
    secondTab->setShortcut(preferences.getBinding("secondTab"));
    connect(secondTab, SIGNAL(triggered()), this, SLOT(slotSelectTab()));
    addAction(secondTab);
    QAction *thirdTab = new QAction(this);
    thirdTab->setData(2);
    thirdTab->setShortcut(preferences.getBinding("thirdTab"));
    connect(thirdTab, SIGNAL(triggered()), this, SLOT(slotSelectTab()));
    addAction(thirdTab);
    QAction *fourthTab = new QAction(this);
    fourthTab->setData(3);
    fourthTab->setShortcut(preferences.getBinding("fourthTab"));
    connect(fourthTab, SIGNAL(triggered()), this, SLOT(slotSelectTab()));
    addAction(fourthTab);
    QAction *fifthTab = new QAction(this);
    fifthTab->setData(4);
    fifthTab->setShortcut(preferences.getBinding("fifthTab"));
    connect(fifthTab, SIGNAL(triggered()), this, SLOT(slotSelectTab()));
    addAction(fifthTab);
    QAction *sixthTab = new QAction(this);
    sixthTab->setData(5);
    sixthTab->setShortcut(preferences.getBinding("sixthTab"));
    connect(sixthTab, SIGNAL(triggered()), this, SLOT(slotSelectTab()));
    addAction(sixthTab);
    QAction *seventhTab = new QAction(this);
    seventhTab->setData(6);
    seventhTab->setShortcut(preferences.getBinding("seventhTab"));
    connect(seventhTab, SIGNAL(triggered()), this, SLOT(slotSelectTab()));
    addAction(seventhTab);
    QAction *eighthTab = new QAction(this);
    eighthTab->setData(7);
    eighthTab->setShortcut(preferences.getBinding("eighthTab"));
    connect(eighthTab, SIGNAL(triggered()), this, SLOT(slotSelectTab()));
    addAction(eighthTab);
    QAction *ninethTab = new QAction(this);
    ninethTab->setData(8);
    ninethTab->setShortcut(preferences.getBinding("ninethTab"));
    connect(ninethTab, SIGNAL(triggered()), this, SLOT(slotSelectTab()));
    addAction(ninethTab);
    QAction *list = new QAction(this);
    list->setShortcut(preferences.getBinding("listCommand"));
    connect(list, SIGNAL(triggered()), this, SLOT(slotCmdList()));
    addAction(list);
}

void dxirc::slotUnreadTab()
{
    int index = m_tabbook->currentIndex();
    for(int i=index; i<m_tabbook->count(); i++)
    {
        if(m_tabbook->tabTextColor(i) == preferences.m_unreadColor || m_tabbook->tabTextColor(i) == preferences.m_highlightColor)
        {
            m_tabbook->setCurrentIndex(i);
            return;
        }
    }
    for(int i=index; i>-1; i--)
    {
        if(m_tabbook->tabTextColor(i) == preferences.m_unreadColor || m_tabbook->tabTextColor(i) == preferences.m_highlightColor)
        {
            m_tabbook->setCurrentIndex(i);
            return;
        }
    }
}

void dxirc::slotNextTab()
{
    if(m_tabbook->count())
    {
        int index = m_tabbook->currentIndex();
        if(index < m_tabbook->count()-1)
        {
            m_tabbook->setCurrentIndex(index+1);
        }
        else
        {
            m_tabbook->setCurrentIndex(0);
        }
    }
}

void dxirc::slotForceFocus()
{
    int index = m_tabbook->currentIndex();
    dxTabItem *dx = qobject_cast<dxTabItem*>(m_tabbook->widget(index));
    if(dx)
    {
        dx->setCommandFocus();
    }
}

void dxirc::slotMoveTab()
{
    const QAction *action = qobject_cast<const QAction*>(sender());
    bool up = action->data().toBool();
    if(m_tabbook->count()>1)
    {
        int id = getCurrentTabId();
        int index = m_tabbook->currentIndex();
        QWidget *tab = m_tabbook->currentWidget();
        QIcon icon = m_tabbook->tabIcon(index);
        QColor color = m_tabbook->tabTextColor(index);
        QString name = m_tabbook->tabText(index);
        if(up)
        {
            if(index) index--;
            else index = m_tabbook->count()-1;
        }
        else
        {
            if(index == m_tabbook->count()-1) index = 0;
            else index++;
        }
        m_tabbook->insertTab(index, tab, icon, name);
        m_tabbook->setTabTextColor(index, color);
        setCurrentTabById(id);
    }
}

void dxirc::slotSelectTab()
{
    const QAction *action = qobject_cast<const QAction*>(sender());
    int index = action->data().toInt();
    m_tabbook->setCurrentIndex(index);
}

void dxirc::slotRestartApp()
{
    preferences.m_x = x();
    preferences.m_y = y();
    preferences.m_w = width();
    preferences.m_h = height();
    preferences.m_maximized = isMaximized();
    for(int i = 0; i<m_tabbook->count(); i++)
    {
        if(!QString::compare(m_tabbook->widget(i)->metaObject()->className(),"IrcTabItem")
                && static_cast<IrcTabItem*>(m_tabbook->widget(i))->getType()==CHANNEL)
        {
            preferences.m_langs.remove(m_tabbook->tabText(i));
            preferences.m_langs.insert(m_tabbook->tabText(i), static_cast<IrcTabItem*>(m_tabbook->widget(i))->getSpellLang());
        }
    }
    preferences.save();
    while(m_ircengines.size())
    {
        m_ircengines[0]->disconnect(this);
        if(m_ircengines[0]->getConnected()) m_ircengines[0]->disconnectIrc();
        m_ircengines.pop_front();
    }
#ifdef HAVE_LUA
    for(int i=0; i<m_scriptEvents.size(); i++)
    {
        if(QString::compare("quit", m_scriptEvents[i].name, Qt::CaseInsensitive) == 0)
        {
            for(int j=0; j<m_scripts.size(); j++)
            {
                if(QString::compare(m_scriptEvents[i].script, m_scripts[j].name, Qt::CaseInsensitive) == 0)
                {
                    lua_getglobal(m_scripts[j].L, m_scriptEvents[i].funcname.toUtf8().constData());
                    if (lua_type(m_scripts[j].L, -1) != LUA_TFUNCTION) lua_pop(m_scripts[j].L, 1);
                    else
                    {
                        if (lua_pcall(m_scripts[j].L, 0, 0, 0))
                        {
                            appendIrcStyledText(_("Lua plugin: error calling %1 %2").arg(m_scriptEvents[i].funcname).arg(QString::fromUtf8(lua_tostring(m_scripts[j].L, -1))), 4, false);
                            lua_pop(m_scripts[j].L, 1);
                        }
                    }
                }
            }
        }
    }
    while(m_scripts.size())
    {
        if(m_scripts[0].L != NULL) lua_close(m_scripts[0].L);
        m_scripts.pop_front();
    }
    while(m_scriptEvents.size())
    {
        m_scriptEvents.pop_front();
    }
#endif //HAVE_LUA
#ifdef Q_OS_WIN
    qApp->quit();
    QString str ;
    for(int i=0; i < argn; i++)
    {
        str += args[i];
        str += (i != argn - 1? " " : "");
    }
    ShellExecuteA(NULL, "open", args[0], argn>1?str.toUtf8().constData():NULL, NULL, SW_SHOWNORMAL);
#else
    if(fork() == 0)
        execvp(args[0], args);
    else
        _exit(0);
#endif //Q_OS_WIN
}

void dxirc::slotCmdList()
{
    if(m_tabbook->count())
    {
        int index = m_tabbook->currentIndex();
        if(m_tabbook->widget(index)
                && (!QString::compare(m_tabbook->widget(index)->metaObject()->className(),"TetrisTabItem")
                || !QString::compare(m_tabbook->widget(index)->metaObject()->className(),"FtTabItem")
                || !QString::compare(m_tabbook->widget(index)->metaObject()->className(),"DccTabItem")))
            return;
        if(m_tabbook->widget(index) && !QString::compare(m_tabbook->widget(index)->metaObject()->className(),"IrcTabItem"))
        {
            IrcTabItem *currenttab = static_cast<IrcTabItem*>(m_tabbook->widget(index));
            IrcEngine *currentserver = currenttab->getEngine();
            if(currentserver == NULL) return;
            if(currentserver->getConnected())
            {
                bool ok;
                QString param = QInputDialog::getText(this, _("Question"), _("Parameters of /LIST command")+QString("\n")+currentserver->getNetworkName(),
                                                      QLineEdit::Normal, QString(), &ok);
                if(ok) currentserver->sendList(param);
            }
            return;
        }
        if(m_tabbook->widget(index) && !QString::compare(m_tabbook->widget(index)->metaObject()->className(),"BoatsTabItem"))
        {
            BoatsTabItem *currenttab = static_cast<BoatsTabItem*>(m_tabbook->widget(index));
            IrcEngine *currentserver = currenttab->getEngine();
            if(currentserver == NULL) return;
            if(currentserver->getConnected())
            {
                bool ok;
                QString param = QInputDialog::getText(this, _("Question"), _("Parameters of /LIST command")+QString("\n")+currentserver->getNetworkName(),
                                                      QLineEdit::Normal, QString(), &ok);
                if(ok) currentserver->sendList(param);
            }
            return;
        }
    }
}

void dxirc::createMenus()
{
    QMenu *serverMenu = menuBar()->addMenu(_("&Server"));
    QAction *serverAction = new QAction(QIcon(dxutils::getDataPath()+"/icons/server.png"), _("&Server list"), this);
    serverAction->setShortcut(preferences.getBinding("serverList"));
    connect(serverAction, SIGNAL(triggered()), this, SLOT(slotCmdServers()));
    serverMenu->addAction(serverAction);
    QAction *connectAction = new QAction(QIcon(dxutils::getDataPath()+"/icons/connect.png"), _("Quick &connect"), this);
    connectAction->setShortcut(preferences.getBinding("quickConnect"));
    connect(connectAction, SIGNAL(triggered()), this, SLOT(slotCmdConnect()));
    serverMenu->addAction(connectAction);
    m_reconnect = new QAction(_("&Reconnect all"), this);
    m_reconnect->setEnabled(false);
    connect(m_reconnect, SIGNAL(triggered()), this, SLOT(slotCmdReconnect()));
    serverMenu->addAction(m_reconnect);
    m_disconnect = new QAction(QIcon(dxutils::getDataPath()+"/icons/disconnect.png"), _("&Disconnect"), this);
    m_disconnect->setShortcut(preferences.getBinding("disconnect"));
    m_disconnect->setEnabled(false);
    connect(m_disconnect, SIGNAL(triggered()), this, SLOT(slotCmdDisconnect()));
    serverMenu->addAction(m_disconnect);
    serverMenu->addSeparator();
    QAction *dccAction = new QAction(QIcon(dxutils::getDataPath()+"/icons/file.png"), _("DCC &transfers"), this);
    dccAction->setShortcut(preferences.getBinding("dccTransfers"));
    connect(dccAction, SIGNAL(triggered()), this, SLOT(slotCmdTransfers()));
    serverMenu->addAction(dccAction);
#ifdef HAVE_LUA
    serverMenu->addSeparator();
    QAction *scriptAction = new QAction(QIcon(dxutils::getDataPath()+"/icons/script.png"), _("S&cripts"), this);
    scriptAction->setShortcut(preferences.getBinding("scripts"));
    connect(scriptAction, SIGNAL(triggered()), this, SLOT(slotCmdScripts()));
    serverMenu->addAction(scriptAction);
#endif
    serverMenu->addSeparator();
    m_logviewer = new QAction(QIcon(dxutils::getDataPath()+"/icons/logs.png"), _("&Log viewer"), this);
    m_logviewer->setShortcut(preferences.getBinding("logViewer"));
    if(!preferences.m_logging) m_logviewer->setEnabled(false);
    connect(m_logviewer, SIGNAL(triggered()), this, SLOT(slotCmdLogs()));
    serverMenu->addAction(m_logviewer);
    QAction *quitAction = new QAction(QIcon(":icons/quit"), _("&Quit"), this);
#ifndef Q_OS_DARWIN
    serverMenu->addSeparator();
#endif
    quitAction->setShortcut(preferences.getBinding("quit"));
    quitAction->setMenuRole(QAction::QuitRole);
    serverMenu->addAction(quitAction);
    connect(quitAction, SIGNAL(triggered()), this, SLOT(slotCmdQuit()));

    QMenu *editMenu = menuBar()->addMenu(_("&Edit"));
    m_closeTab = new QAction(QIcon(dxutils::getDataPath()+"/icons/cross.png"), _("Close current tab"), this);
    m_closeTab->setShortcut(preferences.getBinding("closeCurrentTab"));
    m_closeTab->setEnabled(false);
    connect(m_closeTab, SIGNAL(triggered()), this, SLOT(slotCmdCloseTab()));
    editMenu->addAction(m_closeTab);
    editMenu->addSeparator();
    m_clearTab = new QAction(QIcon(dxutils::getDataPath()+"/icons/clear.png"), _("Clear window"), this);
    m_clearTab->setShortcut(preferences.getBinding("clearWindow"));
    connect(m_clearTab, SIGNAL(triggered()), this, SLOT(slotCmdClearTab()));
    editMenu->addAction(m_clearTab);
    m_clearTabs = new QAction(_("Clear all windows"), this);
    m_clearTabs->setShortcut(preferences.getBinding("clearAllWindows"));
    connect(m_clearTabs, SIGNAL(triggered()), this, SLOT(slotCmdClearTabs()));
    editMenu->addAction(m_clearTabs);
    editMenu->addSeparator();
    m_users = new QAction(_("Users list"), this);
    m_users->setShortcut(preferences.getBinding("usersList"));
    m_users->setCheckable(true);
    m_users->setChecked(preferences.m_usersShown);
    connect(m_users, SIGNAL(triggered()), this, SLOT(slotCmdUsersList()));
    editMenu->addAction(m_users);
#ifdef HAVE_ENCHANT
    m_spellCombo = new QAction(_("Spellchecking language list"), this);
    m_spellCombo->setShortcut(preferences.getBinding("spellList"));
    m_spellCombo->setCheckable(true);
    if(!(preferences.m_useSpell && dxUtils.getAvailableSpellLangsNum()))
    {
        preferences.m_showSpellCombo = false;
        m_spellCombo->setEnabled(false);
    }
    m_spellCombo->setChecked(preferences.m_showSpellCombo);
    connect(m_spellCombo, SIGNAL(triggered()), this, SLOT(slotCmdSpell()));
    editMenu->addAction(m_spellCombo);
#endif
    m_status = new QAction(_("Status bar"), this);
    m_status->setCheckable(true);
    m_status->setChecked(preferences.m_statusShown);
    connect(m_status, SIGNAL(triggered()), this, SLOT(slotCmdStatus()));
    editMenu->addAction(m_status);
    QAction *aliasAction = new QAction(_("&Aliases"), this);
    connect(aliasAction, SIGNAL(triggered()), this, SLOT(slotCmdAlias()));
    editMenu->addAction(aliasAction);
    QAction *prefsAction = new QAction(QIcon(dxutils::getDataPath()+"/icons/wrench.png"), _("&Preferences"), this);
    prefsAction->setMenuRole(QAction::PreferencesRole);
    editMenu->addAction(prefsAction);
    connect(prefsAction, SIGNAL(triggered()), this, SLOT(slotCmdPreferences()));

    QMenu *helpMenu = menuBar()->addMenu(_("&Help"));
    connect(helpMenu->addAction(_("&Check latest version")), SIGNAL(triggered()), this, SLOT(slotCmdCheckVersion()));
    QAction *aboutAction = new QAction(QIcon(":/icons/small_dxirc.png"), _("&About..."), this);
    helpMenu->addAction(aboutAction);
    aboutAction->setMenuRole(QAction::AboutRole);
    connect(aboutAction, SIGNAL(triggered()), this, SLOT(slotCmdAbout()));
    QAction *helpAction = new QAction(QIcon(dxutils::getDataPath()+"/icons/help.png"), _("&Help"), this);
    helpAction->setShortcut(preferences.getBinding("help"));
    connect(helpAction, SIGNAL(triggered()), this, SLOT(slotCmdHelp()));
    helpMenu->addAction(helpAction);
}

void dxirc::createTrayIcon()
{
    QMenu *trayMenu = new QMenu(this);
    connect(trayMenu->addAction(_("&Quit")), SIGNAL(triggered()), this, SLOT(slotCmdQuit()));
    m_trayIcon = new QSystemTrayIcon(this);
    m_trayIcon->setContextMenu(trayMenu);
    m_trayIcon->setToolTip("dxirc");
    setTrayIcon();
}

void dxirc::setTrayIcon(bool newmsg, bool newfile)
{
#ifdef Q_OS_DARWIN
    if(newfile)
    {
        m_trayIcon->setIcon(QIcon(dxutils::getDataPath()+"/icons/bnewfile.png"));
        return;
    }
    if(newmsg)
    {
        m_trayIcon->setIcon(QIcon(dxutils::getDataPath()+"/icons/bnewm.png"));
        return;
    }
    m_trayIcon->setIcon(QIcon(":/icons/black_dxirc.png"));
#else
    if(newfile)
    {
        m_trayIcon->setIcon(QIcon(dxutils::getDataPath()+"/icons/newfile.png"));
        return;
    }
    if(newmsg)
    {
        m_trayIcon->setIcon(QIcon(dxutils::getDataPath()+"/icons/newm.png"));
        return;
    }
    m_trayIcon->setIcon(QIcon(":/icons/small_dxirc.png"));
#endif
}

int dxirc::onLuaAddCommand(lua_State *lua)
{
#ifdef HAVE_LUA
    QString name, funcname, helptext, script;
    if(lua_isstring(lua, 1)) name = QString::fromUtf8(lua_tostring(lua,1));
    if(lua_isstring(lua, 2)) funcname = QString::fromUtf8(lua_tostring(lua,2));
    if(lua_isstring(lua, 3)) helptext = QString::fromUtf8(lua_tostring(lua,3));
    if(name.isEmpty() || funcname.isEmpty() || helptext.isEmpty()) return 0;
    if(dxUtils.isCommand(name))
    {
        _pThis->appendIrcStyledText(_("Command %1 already exists").arg(name), 4, false);
        return 0;
    }
    if(_pThis->m_scripts.size())
    {
        for(int i=0; i<_pThis->m_scripts.size(); i++)
        {
            if(lua == _pThis->m_scripts[i].L) script = _pThis->m_scripts[i].name;
        }
    }
    if(script.isEmpty()) return 0;
    LuaScriptCommand command;
    command.name = name;
    command.funcname = funcname;
    command.helptext = helptext;
    command.script = script;
    dxUtils.addScriptCommand(command);
    return  1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaAddEvent(lua_State *lua)
{
#ifdef HAVE_LUA
    QString name, funcname, script;
    if(lua_isstring(lua, 1)) name = QString::fromUtf8(lua_tostring(lua,1));
    if(lua_isstring(lua, 2)) funcname = QString::fromUtf8(lua_tostring(lua,2));
    if(name.isEmpty() || funcname.isEmpty()) return 0;
    if(_pThis->m_scripts.size())
    {
        for(int i=0; i<_pThis->m_scripts.size(); i++)
        {
            if(lua == _pThis->m_scripts[i].L) script = _pThis->m_scripts[i].name;
        }
    }
    if(script.isEmpty()) return 0;
    if(_pThis->m_scriptEvents.size())
    {
        for(int i=0; i<_pThis->m_scriptEvents.size(); i++)
        {
            if(QString::compare(name, _pThis->m_scriptEvents[i].name, Qt::CaseInsensitive)==0
                    && QString::compare(funcname, _pThis->m_scriptEvents[i].funcname, Qt::CaseInsensitive)==0
                    && QString::compare(script, _pThis->m_scriptEvents[i].script, Qt::CaseInsensitive)==0)
            {
                _pThis->appendIrcStyledText(_("Function %1 for event %2 already exists").arg(funcname).arg(name), 4, false);
                return 0;
            }
        }
    }
    LuaScriptEvent event;
    event.name = name;
    event.funcname = funcname;
    event.script = script;
    _pThis->m_scriptEvents.append(event);
    return  1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaRemoveName(lua_State *lua)
{
#ifdef HAVE_LUA
    QString command, script;
    if(lua_isstring(lua, 1)) command = QString::fromUtf8(lua_tostring(lua, 1));
    if(command.isEmpty()) return 0;
    if(dxUtils.removeScriptCommand(command)) return 1;
    if(_pThis->m_scripts.size())
    {
        for(int i=0; i<_pThis->m_scripts.size(); i++)
        {
            if(lua == _pThis->m_scripts[i].L) script = _pThis->m_scripts[i].name;
        }
    }
    if(script.isEmpty()) return 0;
    if(_pThis->m_scriptEvents.size())
    {
        for(int i=0; i<_pThis->m_scriptEvents.size(); i++)
        {
            if(QString::compare(command, _pThis->m_scriptEvents[i].name, Qt::CaseInsensitive) == 0
                    && QString::compare(script, _pThis->m_scriptEvents[i].script, Qt::CaseInsensitive) == 0)
            {
                _pThis->m_scriptEvents.removeAt(i);
                _pThis->appendIrcStyledText(_("Command/event %1 in script %2 was removed").arg(command).arg(script), 3, false);
            }
        }
    }
    return  1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaCommand(lua_State *lua)
{
#ifdef HAVE_LUA
    QString command;
    if(lua_isstring(lua, 1)) command = QString::fromUtf8(lua_tostring(lua, 1));
    if(command.isEmpty()) return 0;
    int id;
    if(lua_isnumber(lua, 2) && _pThis->isValidTabId(lua_tointeger(lua, 2))) id = lua_tointeger(lua, 2);
    else id = _pThis->getCurrentTabId();
    if(_pThis->m_tabbook->count())
    {
        for(int i = 0; i<_pThis->m_tabbook->count(); i++)
        {
            if(QString::compare(_pThis->m_tabbook->widget(i)->metaObject()->className(),"TetrisTabItem")
                && QString::compare(_pThis->m_tabbook->widget(i)->metaObject()->className(),"FtTabItem"))
            {
                dxTabItem *tab = static_cast<dxTabItem*>(_pThis->m_tabbook->widget(i));
                if(id==tab->getID())
                {
                    tab->processLine(command);
                    return 1;
                }
            }
        }
    }
    return  1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaPrint(lua_State *lua)
{
#ifdef HAVE_LUA
    QString text;
    if(lua_isstring(lua, 1)) text = QString::fromUtf8(lua_tostring(lua, 1));
    if(text.isEmpty()) return 0;
    int id, style;
    if(lua_isnumber(lua, 2) && _pThis->isIddxTabItem(lua_tointeger(lua, 2))) id = lua_tointeger(lua, 2);
    else id = _pThis->getCurrentTabId();
    if(lua_isnumber(lua, 3))
    {
        style = lua_tointeger(lua, 3);
        if(style<0 || style>8) style = 0;
    }
    else style = 0;
    if(_pThis->m_tabbook->count())
    {
        for(int i = 0; i<_pThis->m_tabbook->count(); i++)
        {
            if(QString::compare(_pThis->m_tabbook->widget(i)->metaObject()->className(),"TetrisTabItem")
                && QString::compare(_pThis->m_tabbook->widget(i)->metaObject()->className(),"FtTabItem"))
            {
                dxTabItem *tab = static_cast<dxTabItem*>(_pThis->m_tabbook->widget(i));
                if(id==tab->getID())
                {
                    tab->appendStyledText(text, style, true, true, true);
                    tab->makeLastRowVisible();
                    return 1;
                }
            }
        }
    }
    return  1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaGetServers(lua_State *lua)
{
#ifdef HAVE_LUA
    if(_pThis->m_ircengines.size())
    {
        lua_newtable(lua);
        for(int i=0; i<_pThis->m_ircengines.size(); i++)
        {
            lua_pushnumber(lua, i+1);
            lua_newtable(lua);
            lua_pushstring(lua, "server");
            lua_pushstring(lua, _pThis->m_ircengines[i]->getServerName().toUtf8().constData());
            lua_settable(lua, -3);
            lua_pushstring(lua, "port");
            lua_pushnumber(lua, _pThis->m_ircengines[i]->getServerPort());
            lua_settable(lua, -3);
            lua_pushstring(lua, "nick");
            lua_pushstring(lua, _pThis->m_ircengines[i]->getNickName().toUtf8().constData());
            lua_settable(lua, -3);
            lua_settable(lua, -3);
        }
    }
    else
    {
        lua_newtable(lua);
        lua_pushnumber(lua, 1);
        lua_newtable(lua);
        lua_pushstring(lua, "server");
        lua_pushnil(lua);
        lua_settable(lua, -3);
        lua_pushstring(lua, "port");
        lua_pushnil(lua);
        lua_settable(lua, -3);
        lua_pushstring(lua, "nick");
        lua_pushnil(lua);
        lua_settable(lua, -3);
        lua_settable(lua, -3);
    }
    return  1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaGetTab(lua_State *lua)
{
#ifdef HAVE_LUA
    QString tab, server;
    if(lua_isstring(lua, 1)) tab = QString::fromUtf8(lua_tostring(lua, 1));
    if(lua_isstring(lua, 2)) server = QString::fromUtf8(lua_tostring(lua,2));
    if(_pThis->m_tabbook->count())
    {
        for(int i = 0; i<_pThis->m_tabbook->count(); i++)
        {
            if(QString::compare(_pThis->m_tabbook->widget(i)->metaObject()->className(),"TetrisTabItem")
                && QString::compare(_pThis->m_tabbook->widget(i)->metaObject()->className(),"FtTabItem"))
            {
                if(QString::compare(tab, _pThis->m_tabbook->tabText(i), Qt::CaseInsensitive) == 0
                    && QString::compare(server, static_cast<dxTabItem*>(_pThis->m_tabbook->widget(i))->getServerName()) == 0)
                {
                    lua_pushnumber(lua, static_cast<dxTabItem*>(_pThis->m_tabbook->widget(i))->getID());
                    return 1;
                }
            }
        }
        lua_pushnumber(lua, _pThis->getCurrentTabId());
        return 1;
    }
    else lua_pushnumber(lua, -1);
    return  1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaGetCurrentTab(lua_State *lua)
{
#ifdef HAVE_LUA
    lua_pushnumber(lua, _pThis->getCurrentTabId());
    return 1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaGetVersion(lua_State *lua)
{
#ifdef HAVE_LUA
    lua_pushstring(lua, VERSION);
    return 1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaGetTabInfo(lua_State *lua)
{
#ifdef HAVE_LUA
    int id;
    if(lua_isnumber(lua, 1))  id = lua_tointeger(lua, 1);
    else id = -1;
    if(_pThis->m_tabbook->count() && _pThis->isIddxTabItem(id))
    {
        dxTabItem *tab = NULL;
        for(int i = 0; i<_pThis->m_tabbook->count(); i++)
        {
            if(QString::compare(_pThis->m_tabbook->widget(i)->metaObject()->className(),"TetrisTabItem")
                && QString::compare(_pThis->m_tabbook->widget(i)->metaObject()->className(),"FtTabItem"))
            {
                tab = static_cast<dxTabItem*>(_pThis->m_tabbook->widget(i));
                if(id==tab->getID())
                {
                    break;
                }
            }
        }
        lua_newtable(lua);
        lua_pushstring(lua, "name");
        lua_pushstring(lua, tab->getTabName().toUtf8().constData());
        lua_settable(lua, -3);
        switch(tab->getType()) {
            case SERVER:
            {
                lua_pushstring(lua, "type");
                lua_pushstring(lua, "server");
                lua_settable(lua, -3);
            }break;
            case CHANNEL:
            {
                lua_pushstring(lua, "type");
                lua_pushstring(lua, "channel");
                lua_settable(lua, -3);
            }break;
            case QUERY:
            {
                lua_pushstring(lua, "type");
                lua_pushstring(lua, "query");
                lua_settable(lua, -3);
            }break;
            case OTHER:
            {
                lua_pushstring(lua, "type");
                lua_pushstring(lua, "other");
                lua_settable(lua, -3);
            }break;
            case DCC:
            {
                lua_pushstring(lua, "type");
                lua_pushstring(lua, "dcc");
                lua_settable(lua, -3);
            }break;
            case BOATS:
            {
                lua_pushstring(lua, "type");
                lua_pushstring(lua, "boats");
                lua_settable(lua, -3);
            }break;
        }
        if(!QString::compare(tab->metaObject()->className(),"DccTabItem"))
        {
            lua_pushstring(lua, "type");
            lua_pushstring(lua, "dccchat");
            lua_settable(lua, -3);
        }
        lua_pushstring(lua, "servername");
        lua_pushstring(lua, tab->getServerName().toUtf8().constData());
        lua_settable(lua, -3);
        lua_pushstring(lua, "port");
        lua_pushinteger(lua, tab->getServerPort());
        lua_settable(lua, -3);
        lua_pushstring(lua, "nick");
        lua_pushstring(lua, tab->getNickName().toUtf8().constData());
        lua_settable(lua, -3);
    }
    else
    {
        lua_newtable(lua);
        lua_pushstring(lua, "name");
        lua_pushnil(lua);
        lua_settable(lua, -3);
        lua_pushstring(lua, "type");
        lua_pushnil(lua);
        lua_settable(lua, -3);
        lua_pushstring(lua, "servername");
        lua_pushnil(lua);
        lua_settable(lua, -3);
        lua_pushstring(lua, "port");
        lua_pushnil(lua);
        lua_settable(lua, -3);
        lua_pushstring(lua, "nick");
        lua_pushnil(lua);
        lua_settable(lua, -3);
    }
    return  1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaSetTab(lua_State *lua)
{
#ifdef HAVE_LUA
    _pThis->setCurrentTabById(lua_tointeger(lua, 1));
    return  1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaCreateTab(lua_State *lua)
{
#ifdef HAVE_LUA
    QString name;
    if(lua_isstring(lua, 1)) name = QString::fromUtf8(lua_tostring(lua, 1));
    else
    {
        lua_pushnil(lua);
        return 0;
    }
    if(_pThis->m_tabbook->count())
    {
        if(_pThis->tabExist(name))
        {
            lua_pushnil(lua);
            return 0;
        }
    }
    lua_pushnumber(lua, _pThis->createIrcTab(name, OTHER, NULL, QIcon(dxutils::getDataPath()+"/icons/page.png")));
    return 1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaGetTabCount(lua_State *lua)
{
#ifdef HAVE_LUA
    lua_pushnumber(lua, _pThis->m_tabbook->count());
    return 1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaClear(lua_State *lua)
{
#ifdef HAVE_LUA
    int id = -1;
    if(lua_isnumber(lua, 1) && _pThis->isIddxTabItem(lua_tointeger(lua, 1)))  id = lua_tointeger(lua, 1);
    else return 0;
    dxutils::debugLine(QString("onLuaClear id:%1").arg(id));
    if(_pThis->m_tabbook->count())
    {
        for(int i = 0; i < _pThis->m_tabbook->count(); i++)
        {
            if(QString::compare(_pThis->m_tabbook->widget(i)->metaObject()->className(),"TetrisTabItem")
                    && QString::compare(_pThis->m_tabbook->widget(i)->metaObject()->className(),"FtTabItem")
                    && static_cast<dxTabItem*>(_pThis->m_tabbook->widget(i))->getID() == id)
            {
                static_cast<dxTabItem*>(_pThis->m_tabbook->widget(i))->clearChat();
                return 1;
            }
        }
    }
    return  1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::onLuaGetChannelNames(lua_State *lua)
{
#ifdef HAVE_LUA
    QString tab, server;
    if(lua_isstring(lua, 1)) tab = QString::fromUtf8(lua_tostring(lua, 1));
    if(lua_isstring(lua, 2)) server = QString::fromUtf8(lua_tostring(lua,2));
    if(_pThis->m_tabbook->count())
    {
        for(int i = 0; i<_pThis->m_tabbook->count(); i++)
        {
            if(QString::compare(_pThis->m_tabbook->widget(i)->metaObject()->className(),"TetrisTabItem")
                && QString::compare(_pThis->m_tabbook->widget(i)->metaObject()->className(),"FtTabItem"))
            {
                if(QString::compare(tab, static_cast<dxTabItem*>(_pThis->m_tabbook->widget(i))->getTabName(), Qt::CaseInsensitive) == 0
                    && QString::compare(server, static_cast<dxTabItem*>(_pThis->m_tabbook->widget(i))->getServerName(), Qt::CaseInsensitive) == 0
                    && static_cast<dxTabItem*>(_pThis->m_tabbook->widget(i))->getType() == CHANNEL)
                {
                    lua_newtable(lua);
                    QStringList names = static_cast<IrcTabItem*>(_pThis->m_tabbook->widget(i))->getNicks();
                    for(int j=0; j<names.count(); j++)
                    {
                        lua_pushnumber(lua, j+1);
                        lua_pushstring(lua, names[j].toUtf8().constData());
                        lua_settable(lua, -3);
                    }
                    return 1;
                }
            }
        }
        lua_pushnil(lua);
        return 1;
    }
    lua_pushnil(lua);
    return  1;
#else
    return 0;
#endif //HAVE_LUA
}

void dxirc::slotCmdAbout()
{
    AboutDialog *about = new AboutDialog(this);
    about->setModal(true);
    about->show();
}

void dxirc::slotCmdQuit()
{
    for(int i = 0; i<m_tabbook->count(); i++)
    {
        if(!QString::compare(m_tabbook->widget(i)->metaObject()->className(),"IrcTabItem")
                && static_cast<IrcTabItem*>(m_tabbook->widget(i))->getType()==CHANNEL)
        {
            preferences.m_langs.remove(m_tabbook->tabText(i));
            preferences.m_langs.insert(m_tabbook->tabText(i), static_cast<IrcTabItem*>(m_tabbook->widget(i))->getSpellLang());
        }
    }
    while(m_ircengines.size())
    {
        m_ircengines[0]->disconnect(this);
        if(m_ircengines[0]->getConnected()) m_ircengines[0]->disconnectIrc();
        m_ircengines.pop_front();
    }
#ifdef HAVE_LUA
    for(int i=0; i<m_scriptEvents.size(); i++)
    {
        if(QString::compare("quit", m_scriptEvents[i].name, Qt::CaseInsensitive) == 0)
        {
            for(int j=0; j<m_scripts.size(); j++)
            {
                if(QString::compare(m_scriptEvents[i].script, m_scripts[j].name, Qt::CaseInsensitive) == 0)
                {
                    lua_getglobal(m_scripts[j].L, m_scriptEvents[i].funcname.toUtf8().constData());
                    if (lua_type(m_scripts[j].L, -1) != LUA_TFUNCTION) lua_pop(m_scripts[j].L, 1);
                    else
                    {
                        if (lua_pcall(m_scripts[j].L, 0, 0, 0))
                        {
                            appendIrcStyledText(_("Lua plugin: error calling %1 %2").arg(m_scriptEvents[i].funcname).arg(QString::fromUtf8(lua_tostring(m_scripts[j].L, -1))), 4, false);
                            lua_pop(m_scripts[j].L, 1);
                        }
                    }
                }
            }
        }
    }
    while(m_scripts.size())
    {
        if(m_scripts[0].L != NULL) lua_close(m_scripts[0].L);
        m_scripts.pop_front();
    }
    while(m_scriptEvents.size())
    {
        m_scriptEvents.pop_front();
    }
#endif //HAVE_LUA
    qApp->quit();
}

//handle m_trayIcon click
void dxirc::slotIconActivated(QSystemTrayIcon::ActivationReason reason)
{
    if(reason == QSystemTrayIcon::Trigger)
    {
        if(this->isVisible())
        {
            preferences.m_maximized = this->isMaximized();
            this->hide();
        }
        else
        {
            if(preferences.m_maximized) this->showMaximized();
            else this->show();
            this->raise();
            this->activateWindow();
        }
        setTrayIcon();
    }
}

void dxirc::slotCmdServers()
{
    ServerDialog *dialog = new ServerDialog(this);
    connect(dialog, SIGNAL(connectServer(ServerInfo)), this, SLOT(slotConnectServer(ServerInfo)));
    dialog->setModal(true);
    dialog->show();
}

void dxirc::slotCmdConnect()
{
    ConnectDialog *dialog = new ConnectDialog(this);
    dialog->setModal(true);
    if(dialog->exec() == QDialog::Accepted)
    {
        ServerInfo srv = dialog->serverInfo();
        connectServer(srv.hostname, srv.port, srv.passwd, srv.nick, srv.realname, srv.channels, srv.commands, srv.useSsl);
    }
}

void dxirc::slotCmdReconnect()
{
    for(int i=0; i<m_ircengines.size(); i++)
        m_ircengines[i]->reconnect();
}

void dxirc::slotCmdDisconnect()
{
    if(m_tabbook->count())
    {
        int index = m_tabbook->currentIndex();
        if(m_tabbook->widget(index)
                && (!QString::compare(m_tabbook->widget(index)->metaObject()->className(),"TetrisTabItem")
                || !QString::compare(m_tabbook->widget(index)->metaObject()->className(),"FtTabItem")))
            return;
        if(m_tabbook->widget(index) && !QString::compare(m_tabbook->widget(index)->metaObject()->className(),"IrcTabItem"))
        {
            IrcTabItem *currenttab = static_cast<IrcTabItem*>(m_tabbook->widget(index));
            IrcEngine *currentserver = currenttab->getEngine();
            if(currentserver == NULL) return;
            if(currentserver->getConnected())
            {
                if(QMessageBox::question(this, _("Confirm disconnect"),
                    _("Disconnect server: %1\nPort: %2\nNick: %3").arg(currentserver->getNetworkName()).arg(currentserver->getServerPort()).arg(currentserver->getNickName()),
                    QMessageBox::Yes | QMessageBox::No) == QMessageBox::Yes)
                {
                    currentserver->disconnectIrc();
                    IrcTabItem *tab = NULL;
                    BoatsTabItem *btab = NULL;
                    for(int i = m_tabbook->count()-1; i > -1; i--)
                    {
                        tab = qobject_cast<IrcTabItem*>(m_tabbook->widget(i));
                        if(tab && tab->isSameServer(currentserver))
                        {
                            if(tab->getType() == CHANNEL)
                            {
                                preferences.m_langs.remove(m_tabbook->tabText(i));
                                preferences.m_langs.insert(m_tabbook->tabText(i), static_cast<IrcTabItem*>(m_tabbook->widget(i))->getSpellLang());
                            }
                            m_tabbook->removeTab(i);
                            tab->deleteLater();
                        }
                        btab = qobject_cast<BoatsTabItem*>(m_tabbook->widget(i));
                        if(btab && btab->isSameServer(currentserver))
                        {
                            m_tabbook->removeTab(i);
                            btab->deleteLater();
                        }
                    }
                    if(m_tabbook->count())
                    {
                        sortTabs();
                        m_tabbook->setCurrentIndex(m_tabbook->count()-1);
                    }
                }
            }
            else
            {
                currentserver->disconnectIrc();
                IrcTabItem *tab = NULL;
                BoatsTabItem *btab = NULL;
                for(int i = m_tabbook->count()-1; i > -1; i--)
                {
                    tab = qobject_cast<IrcTabItem*>(m_tabbook->widget(i));
                    if(tab && tab->isSameServer(currentserver))
                    {
                        if(tab->getType() == CHANNEL)
                        {
                            preferences.m_langs.remove(m_tabbook->tabText(i));
                            preferences.m_langs.insert(m_tabbook->tabText(i), static_cast<IrcTabItem*>(m_tabbook->widget(i))->getSpellLang());
                        }
                        m_tabbook->removeTab(i);
                        tab->deleteLater();
                    }
                    btab = qobject_cast<BoatsTabItem*>(m_tabbook->widget(i));
                    if(btab && btab->isSameServer(currentserver))
                    {
                        m_tabbook->removeTab(i);
                        btab->deleteLater();
                    }
                }
                if(m_tabbook->count())
                {
                    sortTabs();
                    m_tabbook->setCurrentIndex(m_tabbook->count()-1);
                }
            }
            return;
        }
        if(m_tabbook->widget(index) && !QString::compare(m_tabbook->widget(index)->metaObject()->className(),"DccTabItem"))
        {
            DccTabItem *currenttab = static_cast<DccTabItem*>(m_tabbook->widget(index));
            if(currenttab->getConnected())
            {
                if(QMessageBox::question(this, _("Confirm disconnect"),
                    _("Disconnect dcc connection: %1\nPort: %2\nWith: %3").arg(currenttab->getServerName()).arg(currenttab->getServerPort()).arg(currenttab->getTabName()),
                    QMessageBox::Yes | QMessageBox::No) == QMessageBox::Yes)
                {
                    currenttab->disconnectDcc();
                    m_tabbook->removeTab(index);
                    currenttab->deleteLater();
                    if(m_tabbook->count())
                    {
                        sortTabs();
                        m_tabbook->setCurrentIndex(m_tabbook->count()-1);
                    }
                }
            }
            else
            {
                m_tabbook->removeTab(index);
                currenttab->deleteLater();
                if(m_tabbook->count())
                {
                    sortTabs();
                    m_tabbook->setCurrentIndex(m_tabbook->count()-1);
                }
            }
            return;
        }
        if(m_tabbook->widget(index) && !QString::compare(m_tabbook->widget(index)->metaObject()->className(),"BoatsTabItem"))
        {
            BoatsTabItem *currenttab = static_cast<BoatsTabItem*>(m_tabbook->widget(index));
            IrcEngine *currentserver = currenttab->getEngine();
            if(currentserver == NULL) return;
            if(currentserver->getConnected())
            {
                if(QMessageBox::question(this, _("Confirm disconnect"),
                    _("Disconnect server: %1\nPort: %2\nNick: %3").arg(currentserver->getNetworkName()).arg(currentserver->getServerPort()).arg(currentserver->getNickName()),
                    QMessageBox::Yes | QMessageBox::No) == QMessageBox::Yes)
                {
                    currentserver->disconnectIrc();
                    IrcTabItem *tab = NULL;
                    BoatsTabItem *btab = NULL;
                    for(int i = m_tabbook->count()-1; i > -1; i--)
                    {
                        tab = qobject_cast<IrcTabItem*>(m_tabbook->widget(i));
                        if(tab && tab->isSameServer(currentserver))
                        {
                            if(tab->getType() == CHANNEL)
                            {
                                preferences.m_langs.remove(m_tabbook->tabText(i));
                                preferences.m_langs.insert(m_tabbook->tabText(i), static_cast<IrcTabItem*>(m_tabbook->widget(i))->getSpellLang());
                            }
                            m_tabbook->removeTab(i);
                            tab->deleteLater();
                        }
                        btab = qobject_cast<BoatsTabItem*>(m_tabbook->widget(i));
                        if(btab && btab->isSameServer(currentserver))
                        {
                            m_tabbook->removeTab(i);
                            btab->deleteLater();
                        }
                    }
                    if(m_tabbook->count())
                    {
                        sortTabs();
                        m_tabbook->setCurrentIndex(m_tabbook->count()-1);
                    }
                }
            }
            else
            {
                currentserver->disconnectIrc();
                IrcTabItem *tab = NULL;
                BoatsTabItem *btab = NULL;
                for(int i = m_tabbook->count()-1; i > -1; i--)
                {
                    tab = qobject_cast<IrcTabItem*>(m_tabbook->widget(i));
                    if(tab && tab->isSameServer(currentserver))
                    {
                        if(tab->getType() == CHANNEL)
                        {
                            preferences.m_langs.remove(m_tabbook->tabText(i));
                            preferences.m_langs.insert(m_tabbook->tabText(i), static_cast<IrcTabItem*>(m_tabbook->widget(i))->getSpellLang());
                        }
                        m_tabbook->removeTab(i);
                        tab->deleteLater();
                    }
                    btab = qobject_cast<BoatsTabItem*>(m_tabbook->widget(i));
                    if(btab && btab->isSameServer(currentserver))
                    {
                        m_tabbook->removeTab(i);
                        btab->deleteLater();
                    }
                }
                if(m_tabbook->count())
                {
                    sortTabs();
                    m_tabbook->setCurrentIndex(m_tabbook->count()-1);
                }
            }
            return;
        }
    }
    updateMenus();
}

void dxirc::slotCmdScripts()
{
    ScriptDialog *dialog = new ScriptDialog(this);
    dialog->setModal(true);
    dialog->show();
}

void dxirc::slotCmdLogs()
{
    if(!m_logs)
    {
        m_logs = new LogDialog(this);
    }
    m_logs->show();
    m_logs->raise();
    m_logs->activateWindow();
}

void dxirc::slotCmdCloseTab()
{
    int index = m_tabbook->currentIndex();
    IrcTabItem *irc = qobject_cast<IrcTabItem*>(m_tabbook->widget(index));
    if(irc)
    {
        if(irc->getType() == OTHER)
        {
            m_tabbook->removeTab(index);
            irc->deleteLater();
            sortTabs();
            updateMenus();
            return;
        }
        IrcEngine *server = irc->getEngine();
        if(server == NULL) return;
        if(irc->getType() == QUERY)
        {
            if(server->getConnected() && isLastTab(server))
            {
                irc->setType(SERVER, server->getNetworkName());
                m_tabbook->setTabIcon(index, QIcon(dxutils::getIconForType(SERVER)));
            }
            else
            {
                if(!server->getConnected()) server->disconnectIrc();
                m_tabbook->removeTab(index);
                irc->deleteLater();
            }
            sortTabs();
            updateMenus();
            return;
        }
        if(irc->getType() == CHANNEL)
        {
            if(server->getConnected()) server->sendPart(m_tabbook->tabText(index));
            if(server->getConnected() && isLastTab(server))
            {
                irc->setType(SERVER, server->getNetworkName());
                m_tabbook->setTabIcon(index, QIcon(dxutils::getIconForType(SERVER)));
            }
            else
            {
                if(!server->getConnected()) server->disconnectIrc();
                m_tabbook->removeTab(index);
                irc->deleteLater();
            }
            sortTabs();
            updateMenus();
            return;
        }
        if(irc->getType() == SERVER)
        {
            server->disconnectIrc();
            for(int i = m_tabbook->count()-1; i > -1; i--)
            {
                irc = qobject_cast<IrcTabItem*>(m_tabbook->widget(i));
                if(irc && irc->isSameServer(server))
                {
                    m_tabbook->removeTab(i);
                    irc->deleteLater();
                }
            }
            sortTabs();
            updateMenus();
            return;
        }
    }
    DccTabItem *dcc = qobject_cast<DccTabItem*>(m_tabbook->widget(index));
    if(dcc)
    {
        dcc->disconnect();
        m_tabbook->removeTab(index);
        dcc->deleteLater();
        sortTabs();
        updateMenus();
        return;
    }
    BoatsTabItem *boats = qobject_cast<BoatsTabItem*>(m_tabbook->widget(index));
    if(boats)
    {
        m_tabbook->removeTab(index);
        boats->leftGame();
        boats->deleteLater();
        sortTabs();
        updateMenus();
        return;
    }
    TetrisTabItem *tetris = qobject_cast<TetrisTabItem*>(m_tabbook->widget(index));
    if(tetris)
    {
        m_tabbook->removeTab(index);
        tetris->deleteLater();
        updateMenus();
        return;
    }
    if(m_ft == m_tabbook->widget(index))
    {
        slotCloseFTTab();
        return;
    }
}

void dxirc::slotCmdClearTab()
{
    dxTabItem *tab = qobject_cast<dxTabItem*>(m_tabbook->currentWidget());
    if(tab)
    {
        tab->clearChat();
    }
}

void dxirc::slotCmdClearTabs()
{
    dxTabItem *dx = NULL;
    for(int i=0; i<m_tabbook->count(); i++)
    {
        dx = qobject_cast<dxTabItem*>(m_tabbook->widget(i));
        if(dx)
        {
            dx->clearChat();
            if(dx->getType() == CHANNEL)
            {
                m_tabbook->setTabIcon(i, QIcon(dxutils::getIconForType(CHANNEL)));
            }
            else if(dx->getType() == QUERY)
            {
                m_tabbook->setTabIcon(i, QIcon(dxutils::getIconForType(QUERY)));
            }
            else if(dx->getType() == DCC)
            {
                m_tabbook->setTabIcon(i, QIcon(dxutils::getIconForType(DCC)));
            }
        }
        m_tabbook->setTabTextColor(i, palette().windowText().color());
    }
    setTrayIcon();
}

void dxirc::slotCmdUsersList()
{
    preferences.m_usersShown = !preferences.m_usersShown;
    IrcTabItem *irc = NULL;
    for(int i=0; i<m_tabbook->count(); i++)
    {
        irc = qobject_cast<IrcTabItem*>(m_tabbook->widget(i));
        if(irc)
        {
            if(preferences.m_usersShown) irc->showUsers();
            else irc->hideUsers();
        }
    }
}

void dxirc::slotCmdSpell()
{
    preferences.m_showSpellCombo = !preferences.m_showSpellCombo;
    dxTabItem *dx = NULL;
    for(int i=0; i<m_tabbook->count(); i++)
    {
        dx = qobject_cast<dxTabItem*>(m_tabbook->widget(i));
        if(dx) dx->showSpellComboUpdated();
    }
}

//show/hide statusbar
void dxirc::slotCmdStatus()
{
    preferences.m_statusShown = !preferences.m_statusShown;
    if(preferences.m_statusShown) m_statusBar->show();
    else m_statusBar->hide();
}

void dxirc::slotCmdAlias()
{
    AliasDialog *dialog = new AliasDialog(this);
    dialog->setModal(true);
    dialog->show();
}

void dxirc::slotCmdPreferences()
{
    QColor unread = preferences.m_unreadColor;
    QColor high = preferences.m_highlightColor;
    bool oldOwnServerWindow = preferences.m_ownServerWindow;
    ConfigDialog *dialog = new ConfigDialog(this);
    connect(dialog, SIGNAL(restartApp()), this, SLOT(slotRestartApp()));
    dialog->setModal(true);
    if(dialog->exec() == QDialog::Accepted)
    {
        dxTabItem *dx = NULL;
        for(int i=0; i<m_tabbook->count(); i++)
        {
            dx = qobject_cast<dxTabItem*>(m_tabbook->widget(i));
            if(dx) dx->preferencesUpdated();
            if(m_tabbook->tabTextColor(i)==unread) m_tabbook->setTabTextColor(i, preferences.m_unreadColor);
            if(m_tabbook->tabTextColor(i)==high) m_tabbook->setTabTextColor(i, preferences.m_highlightColor);
        }
        updateTabPosition();
        if(oldOwnServerWindow!=preferences.m_ownServerWindow) updateServerWindow();
        updateMenus();
        if(m_logs) m_logs->setLogPath(preferences.m_logPath);
    }
}

void dxirc::slotCmdTransfers()
{
    m_tabbook->addTab(m_ft, QIcon(dxutils::getDataPath()+"/icons/file.png"), _("DCC transfers"));
    m_tabbook->setTabTextColor(m_tabbook->count()-1, palette().windowText().color());
    m_tabbook->setCurrentWidget(m_ft);
    updateTabPosition();
    updateMenus();
}

void dxirc::slotCmdCheckVersion()
{
    if(!m_versionSocket)
    {
        m_versionSocket = new dxSocket(false);
        connect(m_versionSocket, SIGNAL(socketConnected()), this, SLOT(slotVersionConnected()));
        connect(m_versionSocket, SIGNAL(socketCanRead()), this, SLOT(slotVersionCanRead()));
        connect(m_versionSocket, SIGNAL(socketError(QString)), this, SLOT(slotVersionError(QString)));
    }
    if(m_versionSocket->isConnecting())
    {
        QMessageBox::information(this, _("Information"), _("Checking version in progress"), QMessageBox::Ok);
        return;
    }
    if(m_versionSocket->isConnected())
    {
        QMessageBox::information(this, _("Information"), _("Checking version in progress"), QMessageBox::Ok);
        return;
    }
    updateStatus(_("Checking version in progress"));
    m_versionSocket->setHost("dxirc.org", 80);
    m_versionSocket->connectTo();
}

void dxirc::slotVersionConnected()
{
    if(m_versionSocket)
        m_versionSocket->write("GET http://www.dxirc.org/version HTTP/1.0\nUser-Agent: dxirc version\n\n", 72);
}

void dxirc::slotVersionCanRead()
{
    slotStatusTime();
    if(m_versionSocket)
    {
        int read = m_versionSocket->getBytesAvailable();
        char *buffer = new char[read];
        read = m_versionSocket->read(buffer, read);
        QString serverVersion = buffer;
        serverVersion = serverVersion.section('\n', serverVersion.count('\n')-(buffer[read-1]=='\n'?1:0)).section('\n',0,0);
        QString currentVersion = VERSION;
        int current = currentVersion.section('.',0,0).toInt()*1000+currentVersion.section('.',1).section('.',0,0).toInt()*10+currentVersion.section('.',2).toInt();
        int server = serverVersion.section('.',0,0).toInt()*1000+serverVersion.section('.',1).section('.',0,0).toInt()*10+serverVersion.section('.',2).toInt();
        if(current==server)
            QMessageBox::information(this, _("Information"), _("You have last version %1").arg(VERSION), QMessageBox::Ok);
        else if(current<server)
            QMessageBox::information(this, _("Information"), _("New version %1 available").arg(serverVersion), QMessageBox::Ok);
        else
            QMessageBox::information(this, _("Information"), _("You have svn version %1").arg(VERSION), QMessageBox::Ok);
        delete []buffer;
        m_versionSocket->disconnectFrom();
    }
}

void dxirc::slotVersionError(QString error)
{
    dxutils::debugLine(error);
    slotStatusTime();
    if(m_versionSocket->isConnected()) QMessageBox::critical(this, _("Error"), _("Error occurred during version check"), QMessageBox::Ok);
}

void dxirc::slotCmdHelp()
{
    HelpDialog *dialog = new HelpDialog(this);
    dialog->setModal(true);
    dialog->show();
}

void dxirc::updateStatus(QString text)
{
    m_statusBar->showMessage(text, 5000);
    m_trayIcon->setToolTip("dxirc\n"+text);
    m_statusTimer->start(5000);
}

void dxirc::slotStatusTime()
{
    m_statusBar->clearMessage();
    m_statusTimer->stop();
    m_trayIcon->setToolTip("dxirc");
}

void dxirc::slotAwayTime()
{
    IrcTabItem *item = NULL;
    for(int i=0; i<m_tabbook->count(); i++)
    {
        item = qobject_cast<IrcTabItem*>(m_tabbook->widget(i));
        if(item) item->checkAway();
    }
}

void dxirc::slotConnectServer(ServerInfo info)
{
    connectServer(info.hostname, info.port, info.passwd, info.nick, info.realname, info.channels, info.commands, info.useSsl);
}

void dxirc::slotIrcDccServer(IrcEngine *server, QString with)
{
    createDccTab(server->getNickName(), with, server->getLocalIP(), preferences.m_dccPortD, preferences.m_dccPortH, true, server);
}

void dxirc::slotIrcDccOut(IrcEngine* server, QString nick, QString file)
{
    DccFile dcc;
    dcc.path = file;
    dcc.speed = 0;
    dcc.currentPosition = 0;
    dcc.size = QFileInfo(file).size();
    dcc.type = DCC_OUT;
    dcc.started = false;
    dcc.canceled = false;
    dcc.finished = false;
    dcc.finishedPosition = 0;
    dcc.ip = server->getLocalIP();
    dcc.port = 0;
    dcc.token = -1;
    dcc.nick = nick;
    for(int i=0; i<m_dccfilesList.size(); i++)
    {
        if(dcc.path == m_dccfilesList[i].path && dcc.type == m_dccfilesList[i].type)
        {
            m_dccfilesList.removeAt(i);
            break;
        }
    }
    m_dccfilesList.append(dcc);
    DccEngine *engine = new DccEngine(dcc, server);
    connect(engine, SIGNAL(dccError(QString)), this, SLOT(slotDccError(QString)));
    connect(engine, SIGNAL(dccPosition(DccFile)), this, SLOT(slotDccPosition(DccFile)));
    m_dccengines.append(engine);
    engine->startConnection();
    slotCmdTransfers();
}

void dxirc::slotIrcDccPout(IrcEngine* server, QString nick, QString file)
{
    DccFile dcc;
    dcc.path = file;
    dcc.speed = 0;
    dcc.currentPosition = 0;
    dcc.size = QFileInfo(file).size();
    dcc.type = DCC_POUT;
    dcc.started = false;
    dcc.canceled = false;
    dcc.finished = false;
    dcc.finishedPosition = 0;
    if(m_lastToken==32767) m_lastToken=0;
    dcc.token = ++m_lastToken;
    dcc.nick = nick;
    for(int i=0; i<m_dccfilesList.size(); i++)
    {
        if(dcc.path == m_dccfilesList[i].path && dcc.type == m_dccfilesList[i].type)
        {
            m_dccfilesList.removeAt(i);
            break;
        }
    }
    m_dccfilesList.append(dcc);
    server->sendCtcp(nick, "DCC SEND "+dxutils::removeSpaces(dcc.path.section('/',-1))+" "+server->getLocalIPBinary()+" 0 "+QString::number(dcc.size)+" "+QString::number(dcc.token));
}

void dxirc::slotIrcDccChat(QString with, QString address, int port)
{
    IrcEngine *server = static_cast<IrcEngine*>(sender());
    if(preferences.m_autoDccChat)
        createDccTab(server->getNickName(), with, address, port, port, false, server);
    if(!preferences.m_autoDccChat
            && QMessageBox::question(this, _("Question"),
            _("%1 offers DCC Chat on %2(%3) port %4.\n Do you want connect?").arg(with).arg(address).arg(server->getHostname(address)).arg(port),
            QMessageBox::Yes | QMessageBox::No) == QMessageBox::Yes)
        createDccTab(server->getNickName(), with, address, port, port, false, server);
}

void dxirc::slotIrcInvite(QString nick, QString channel)
{
    if(preferences.m_ignoreCommandsList.contains("invite"))
        return;
    IrcEngine *server = static_cast<IrcEngine*>(sender());
    if(QMessageBox::question(this, _("Question"),
        _("%1 invites you on %2 on %3 \n Do you want connect?").arg(nick).arg(channel).arg(server->getNetworkName()),
        QMessageBox::Yes | QMessageBox::No) == QMessageBox::Yes)
        server->sendJoin(channel);
}

void dxirc::slotIrcEndMotd()
{
    updateMenus();
}

void dxirc::slotIrcPrivmsgAndAction(QString nick, QString to, QString msg)
{
    slotIrcPrivmsgAndAction(static_cast<IrcEngine*>(sender()), nick, to, msg);
}

void dxirc::slotIrcPrivmsgAndAction(IrcEngine *server, QString nick, QString to, QString msg)
{
    if(!isVisible() && preferences.m_notifyTray)
    {
        setTrayIcon(true, false);
    }
#ifdef HAVE_LUA
    if(!m_scripts.size() || !m_scriptEvents.size()) return;

    for(int i=0; i<m_scriptEvents.size(); i++)
    {
        if(QString::compare("privmsg", m_scriptEvents[i].name, Qt::CaseInsensitive) == 0)
        {
            for(int j=0; j<m_scripts.size(); j++)
            {
                if(QString::compare(m_scriptEvents[i].script, m_scripts[j].name, Qt::CaseInsensitive) == 0)
                {
                    lua_getglobal(m_scripts[j].L, m_scriptEvents[i].funcname.toUtf8().constData());
                    if (lua_type(m_scripts[j].L, -1) != LUA_TFUNCTION) lua_pop(m_scripts[j].L, 1);
                    else
                    {
                        lua_pushstring(m_scripts[j].L, nick.toUtf8().constData());
                        lua_pushstring(m_scripts[j].L, msg.toUtf8().constData());
                        lua_pushinteger(m_scripts[j].L, getTabId(server, to == server->getNickName() ? nick : to));
                        if (lua_pcall(m_scripts[j].L, 3, 0, 0))
                        {
                            appendIrcStyledText(_("Lua plugin: error calling %1 %2").arg(m_scriptEvents[i].funcname).arg(QString::fromUtf8(lua_tostring(m_scripts[j].L, -1))), 4, false);
                            lua_pop(m_scripts[j].L, 1);
                        }
                    }
                }
            }
        }
    }
#endif //HAVE_LUA
}

void dxirc::slotIrcPart(QString nick, QString channel)
{
    IrcEngine *server = static_cast<IrcEngine*>(sender());
#ifdef HAVE_LUA
    for(int i=0; i<m_scriptEvents.size(); i++)
    {
        if(QString::compare("part", m_scriptEvents[i].name, Qt::CaseInsensitive) == 0)
        {
            for(int j=0; j<m_scripts.size(); j++)
            {
                if(QString::compare(m_scriptEvents[i].script, m_scripts[j].name, Qt::CaseInsensitive) == 0)
                {
                    lua_getglobal(m_scripts[j].L, m_scriptEvents[i].funcname.toUtf8().constData());
                    if (lua_type(m_scripts[j].L, -1) != LUA_TFUNCTION) lua_pop(m_scripts[j].L, 1);
                    else
                    {
                        lua_pushstring(m_scripts[j].L, nick.toUtf8().constData());
                        lua_pushinteger(m_scripts[j].L, getTabId(server, channel));
                        if (lua_pcall(m_scripts[j].L, 2, 0, 0))
                        {
                            appendIrcStyledText(_("Lua plugin: error calling %1 %2").arg(m_scriptEvents[i].funcname).arg(QString::fromUtf8(lua_tostring(m_scripts[j].L, -1))), 4, false);
                            lua_pop(m_scripts[j].L, 1);
                        }
                    }
                }
            }
        }
    }
#endif //HAVE_LUA
    if(isFriend(nick, channel, server->getNetworkName()))
    {
        if(preferences.m_sounds && preferences.m_soundDisconnect)
            dxutils::playFile(preferences.m_pathDisconnect);
        if(preferences.m_notify && preferences.m_notifyDisconnect)
            dxUtils.showNotify(_("%1 has parted %2").arg(nick).arg(channel));
    }
    if(tabExist(server, channel))
    {
        if(nick == server->getNickName())
        {
            int index = getTabIndex(server, channel);
            IrcTabItem *irc = static_cast<IrcTabItem*>(getTabItemById(getTabId(server, channel)));
            if(index == -1 || irc == NULL) return;
            preferences.m_langs.remove(m_tabbook->tabText(index));
            preferences.m_langs.insert(m_tabbook->tabText(index), irc->getSpellLang());
            if(server->getConnected() && isLastTab(server))
            {
                irc->setType(SERVER, server->getNetworkName());
                m_tabbook->setTabIcon(index, QIcon(dxutils::getIconForType(SERVER)));
            }
            else
            {
                m_tabbook->removeTab(index);
                irc->deleteLater();
            }
            sortTabs();
            updateMenus();
        }
    }
}

void dxirc::slotIrcKick(QString nick, QString to, QString channel)
{
    Q_UNUSED(nick)
    IrcEngine *server = static_cast<IrcEngine*>(sender());
    if(to == server->getNickName())
    {
        int index = getTabIndex(server, channel);
        IrcTabItem *irc = static_cast<IrcTabItem*>(getTabItemById(getTabId(server, channel)));
        if(index == -1 || irc == NULL) return;
        if(server->getConnected() && isLastTab(server))
        {
            irc->setType(SERVER, server->getNetworkName());
            m_tabbook->setTabIcon(index, QIcon(dxutils::getIconForType(SERVER)));
        }
        else
        {
            m_tabbook->removeTab(index);
            irc->deleteLater();
        }
        sortTabs();
        updateMenus();
    }
}

void dxirc::slotIrcDisconnect(QString msg)
{
    IrcEngine *server = static_cast<IrcEngine*>(sender());
    IrcTabItem *irc = NULL;
    for(int i = m_tabbook->count()-1; i > -1; i--)
    {
        irc = qobject_cast<IrcTabItem*>(m_tabbook->widget(i));
        if(irc && server && irc->isSameServer(server))
        {
            if(irc->getType() == CHANNEL)
            {
                preferences.m_langs.remove(m_tabbook->tabText(i));
                preferences.m_langs.insert(m_tabbook->tabText(i), irc->getSpellLang());
            }
            m_tabbook->removeTab(i);
            irc->deleteLater();
        }
    }
    if(m_tabbook->count())
    {
        sortTabs();
        m_tabbook->setCurrentIndex(0);
    }
    updateMenus();
    updateStatus(msg);
}

void dxirc::slotIrcConnectAndReconnect(QString msg)
{
    updateStatus(msg);
}

void dxirc::slotIrcJoin(QString nick, QString channel)
{
    IrcEngine *server = static_cast<IrcEngine*>(sender());
    if(isFriend(nick, channel, server->getNetworkName()))
    {
        if(preferences.m_sounds && preferences.m_soundConnect)
            dxutils::playFile(preferences.m_pathConnect);
        if(preferences.m_notify && preferences.m_notifyConnect)
            dxUtils.showNotify(_("%1 has joined to %2").arg(nick).arg(channel));
    }
#ifdef HAVE_LUA
    if(server->isUserIgnored(nick, channel)) return;
    if(!m_scripts.size() || !m_scriptEvents.size()) return;
    for(int i=0; i<m_scriptEvents.size(); i++)
    {
        if(QString::compare("join", m_scriptEvents[i].name, Qt::CaseInsensitive) == 0)
        {
            for(int j=0; j<m_scripts.size(); j++)
            {
                if(QString::compare(m_scriptEvents[i].script, m_scripts[j].name, Qt::CaseInsensitive) == 0)
                {
                    lua_getglobal(m_scripts[j].L, m_scriptEvents[i].funcname.toUtf8().constData());
                    if (lua_type(m_scripts[j].L, -1) != LUA_TFUNCTION) lua_pop(m_scripts[j].L, 1);
                    else
                    {
                        lua_pushstring(m_scripts[j].L, nick.toUtf8().constData());
                        lua_pushinteger(m_scripts[j].L, getTabId(server, channel));
                        if (lua_pcall(m_scripts[j].L, 2, 0, 0))
                        {
                            appendIrcStyledText(_("Lua plugin: error calling %1 %2").arg(m_scriptEvents[i].funcname).arg(QString::fromUtf8(lua_tostring(m_scripts[j].L, -1))), 4, false);
                            lua_pop(m_scripts[j].L, 1);
                        }
                    }
                }
            }
        }
    }
#endif //HAVE_LUA
}

void dxirc::slotIrcQuit(QString nick)
{
    IrcEngine *server = static_cast<IrcEngine*>(sender());
    if(isFriend(nick, "all", server->getNetworkName()))
    {
        if(preferences.m_sounds && preferences.m_soundDisconnect)
            dxutils::playFile(preferences.m_pathDisconnect);
        if(preferences.m_notify && preferences.m_notifyDisconnect)
            dxUtils.showNotify(_("%1 has quit").arg(nick));
    }
}

void dxirc::slotIrcDccIn(QString nick, QString ip, QString port, QString filename, QString size)
{
    IrcEngine *server = static_cast<IrcEngine*>(sender());
    if(preferences.m_autoDccFile)
    {
        QFileInfo info(filename);
        DccFile dcc;
        dcc.path = getUniqueName(preferences.m_dccPath, info.baseName(), info.completeSuffix());
        dcc.speed = 0;
        dcc.currentPosition = 0;
        dcc.size = size.toLongLong();
        dcc.type = DCC_IN;
        dcc.started = false;
        dcc.canceled = false;
        dcc.finished = false;
        dcc.finishedPosition = 0;
        dcc.token = -1;
        dcc.ip = ip;
        dcc.port = port.toInt();
        dcc.nick = nick;
        for(int i=0; i<m_dccfilesList.size(); i++)
        {
            if(dcc.path == m_dccfilesList[i].path && (m_dccfilesList[i].type==DCC_IN || m_dccfilesList[i].type==DCC_PIN))
            {
                m_dccfilesList.removeAt(i);
                break;
            }
        }
        m_dccfilesList.append(dcc);
        info = QFileInfo(dcc.path);
        if(isForResume(info.fileName()))
        {
            server->sendCtcp(nick, "DCC RESUME "+info.fileName()+" "+QString::number(dcc.port)+" "+QString::number(QFileInfo(dcc.path+".part").size()));
        }
        else
        {
            DccEngine *engine = new DccEngine(dcc, server);
            connect(engine, SIGNAL(dccError(QString)), this, SLOT(slotDccError(QString)));
            connect(engine, SIGNAL(dccPosition(DccFile)), this, SLOT(slotDccPosition(DccFile)));
            m_dccengines.append(engine);
            engine->startConnection();
            appendIrcStyledText(_("Receiving \"%1\" from %2").arg(filename).arg(nick),8,false);
        }
    }
    else
    {
        if(isForResume(filename)
            &&QMessageBox::question(this, _("Question"), _("%1 offers file %2 with size %3 over DCC .\nFile is already partially downloaded.\nDo you want continue in download?").arg(nick).arg(filename).arg(dxutils::getFileSize(size)), QMessageBox::Yes|QMessageBox::No) == QMessageBox::Yes)
        {
            DccFile dcc;
            dcc.path = preferences.m_dccPath+"/"+filename;
            dcc.speed = 0;
            dcc.currentPosition = 0;
            dcc.size = size.toLongLong();
            dcc.type = DCC_IN;
            dcc.started = false;
            dcc.canceled = false;
            dcc.finished = false;
            dcc.finishedPosition = 0;
            dcc.token = -1;
            dcc.ip = ip;
            dcc.port = port.toInt();
            dcc.nick = nick;
            for(int i=0; i<m_dccfilesList.size(); i++)
            {
                if(dcc.path == m_dccfilesList[i].path && (m_dccfilesList[i].type==DCC_IN || m_dccfilesList[i].type==DCC_PIN))
                {
                    m_dccfilesList.removeAt(i);
                    break;
                }
            }
            m_dccfilesList.append(dcc);
            QFileInfo info(dcc.path+".part");
            server->sendCtcp(nick, "DCC RESUME "+filename+" "+port+" "+QString::number(info.size()));
            return;
        }
        if(QMessageBox::question(this, _("Question"),
                _("%1 offers file %2 with size %3 over DCC .\nDo you want connect?").arg(nick).arg(filename).arg(dxutils::getFileSize(size)), QMessageBox::Yes|QMessageBox::No) == QMessageBox::Yes)
        {
            QFileDialog::Options options = QFileDialog::DontResolveSymlinks;
            QString path = QFileDialog::getSaveFileName(this,
                                        _("Save file"),
                                        preferences.m_dccPath+"/"+filename,
                                        "",
                                        NULL,
                                        options);
            if(!path.isEmpty())
            {
                QFileInfo info(path);
                DccFile dcc;
                dcc.path = getUniqueName(info.absolutePath(), info.baseName(), info.completeSuffix()); // dirty (owerwrite user opinion) but absolutely needed :)
                dcc.speed = 0;
                dcc.currentPosition = 0;
                dcc.size = size.toLongLong();
                dcc.type = DCC_IN;
                dcc.started = false;
                dcc.canceled = false;
                dcc.finished = false;
                dcc.finishedPosition = 0;
                dcc.token = -1;
                dcc.ip = ip;
                dcc.port = port.toInt();
                dcc.nick = nick;
                for(int i=0; i<m_dccfilesList.size(); i++)
                {
                    if(dcc.path == m_dccfilesList[i].path && (m_dccfilesList[i].type==DCC_IN || m_dccfilesList[i].type==DCC_PIN))
                    {
                        m_dccfilesList.removeAt(i);
                        break;
                    }
                }
                //old .part file have to be deleted
                QFile file(dcc.path+".part");
                if(file.exists())
                    file.remove();
                m_dccfilesList.append(dcc);
                DccEngine *engine = new DccEngine(dcc, server);
                connect(engine, SIGNAL(dccError(QString)), this, SLOT(slotDccError(QString)));
                connect(engine, SIGNAL(dccPosition(DccFile)), this, SLOT(slotDccPosition(DccFile)));
                m_dccengines.append(engine);
                engine->startConnection();
                appendIrcStyledText(_("Receiving \"%1\" from %2").arg(filename).arg(nick),8,false);
                slotCmdTransfers();
            }
        }
    }
}

void dxirc::slotIrcDccMyToken(QString ip, int port, int token)
{
    IrcEngine *server = static_cast<IrcEngine*>(sender());
    int index = -1;
    for(int i=0; i<m_dccfilesList.size(); i++)
    {
        if(m_dccfilesList[i].token == token && m_dccfilesList[i].type == DCC_POUT)
        {
            index = i;
            break;
        }
    }
    if(index == -1)
        return;
    m_dccfilesList[index].ip = ip;
    m_dccfilesList[index].port = port;
    DccEngine *engine = new DccEngine(m_dccfilesList[index], server);
    connect(engine, SIGNAL(dccError(QString)), this, SLOT(slotDccError(QString)));
    connect(engine, SIGNAL(dccPosition(DccFile)), this, SLOT(slotDccPosition(DccFile)));
    m_dccengines.append(engine);
    engine->startConnection();
    slotCmdTransfers();
}

void dxirc::slotIrcDccToken(QString nick, QString filename, QString size, int token)
{
    IrcEngine *server = static_cast<IrcEngine*>(sender());
    if(preferences.m_autoDccFile)
    {
        QFileInfo info(filename);
        DccFile dcc;
        dcc.path = getUniqueName(preferences.m_dccPath, info.baseName(), info.completeSuffix());
        dcc.speed = 0;
        dcc.currentPosition = 0;
        dcc.size = size.toLongLong();
        dcc.type = DCC_PIN;
        dcc.started = false;
        dcc.canceled = false;
        dcc.finished = false;
        dcc.finishedPosition = 0;
        dcc.token = token;
        dcc.nick = nick;
        dcc.ip = server->getLocalIP();
        dcc.port = 0;
        for(int i=0; i<m_dccfilesList.size(); i++)
        {
            if(dcc.path == m_dccfilesList[i].path && (m_dccfilesList[i].type==DCC_IN || m_dccfilesList[i].type==DCC_PIN))
            {
                m_dccfilesList.removeAt(i);
                break;
            }
        }
        m_dccfilesList.append(dcc);
        info = QFileInfo(dcc.path);
        if(isForResume(info.fileName()))
        {
            server->sendCtcp(nick, "DCC RESUME "+info.fileName()+" 0 "+QString::number(QFileInfo(dcc.path+".part").size())+" "+QString::number(dcc.token));
        }
        else
        {
            DccEngine *engine = new DccEngine(dcc, server);
            connect(engine, SIGNAL(dccError(QString)), this, SLOT(slotDccError(QString)));
            connect(engine, SIGNAL(dccPosition(DccFile)), this, SLOT(slotDccPosition(DccFile)));
            m_dccengines.append(engine);
            engine->startConnection();
            appendIrcStyledText(_("Receiving \"%1\" from %2").arg(filename).arg(nick),8,false);
        }
    }
    else
    {
        if(isForResume(filename)
                && QMessageBox::question(this, _("Question"),
                _("%1 offers file %2 with size %3 over DCC passive.\nFile is already partially downloaded.\nDo you want continue in download?").arg(nick).arg(filename).arg(dxutils::getFileSize(size)),
                QMessageBox::Yes|QMessageBox::No) == QMessageBox::Yes)
        {
            DccFile dcc;
            dcc.path = preferences.m_dccPath+"/"+filename;
            dcc.speed = 0;
            dcc.currentPosition = 0;
            dcc.size = size.toLongLong();
            dcc.type = DCC_PIN;
            dcc.started = false;
            dcc.canceled = false;
            dcc.finished = false;
            dcc.finishedPosition = 0;
            dcc.token = token;
            dcc.nick = nick;
            for(int i=0; i<m_dccfilesList.size(); i++)
            {
                if(dcc.path == m_dccfilesList[i].path && (m_dccfilesList[i].type==DCC_IN || m_dccfilesList[i].type==DCC_PIN))
                {
                    m_dccfilesList.removeAt(i);
                    break;
                }
            }
            m_dccfilesList.append(dcc);
            server->sendCtcp(nick, "DCC RESUME "+filename+" 0 "+QString::number(QFileInfo(dcc.path+".part").size())+" "+QString::number(dcc.token));
            return;
        }
        if(QMessageBox::question(this, _("Question"),
            _("%1 offers file %2 with size %3 over DCC passive.\n Do you want accept?").arg(nick).arg(filename).arg(dxutils::getFileSize(size)),
            QMessageBox::Yes|QMessageBox::No) == QMessageBox::Yes)
        {
            QFileDialog::Options options = QFileDialog::DontResolveSymlinks;
            QString path = QFileDialog::getSaveFileName(this,
                                        _("Save file"),
                                        preferences.m_dccPath+"/"+filename,
                                        "",
                                        NULL,
                                        options);
            if(!path.isEmpty())
            {
                QFileInfo info(path);
                DccFile dcc;
                dcc.path = getUniqueName(info.absolutePath(), info.baseName(), info.completeSuffix()); // dirty (owerwrite user opinion) but absolutely needed :)
                dcc.speed = 0;
                dcc.currentPosition = 0;
                dcc.size = size.toLongLong();
                dcc.type = DCC_PIN;
                dcc.started = false;
                dcc.canceled = false;
                dcc.finished = false;
                dcc.finishedPosition = 0;
                dcc.token = token;
                dcc.nick = nick;
                dcc.ip = server->getLocalIP();
                dcc.port = 0;
                for(int i=0; i<m_dccfilesList.size(); i++)
                {
                    if(dcc.path == m_dccfilesList[i].path && (m_dccfilesList[i].type==DCC_IN || m_dccfilesList[i].type==DCC_PIN))
                    {
                        m_dccfilesList.removeAt(i);
                        break;
                    }
                }
                //old .part file have to be deleted
                QFile file(dcc.path+".part");
                if(file.exists())
                    file.remove();
                m_dccfilesList.append(dcc);
                DccEngine *engine = new DccEngine(dcc, server);
                connect(engine, SIGNAL(dccError(QString)), this, SLOT(slotDccError(QString)));
                connect(engine, SIGNAL(dccPosition(DccFile)), this, SLOT(slotDccPosition(DccFile)));
                m_dccengines.append(engine);
                engine->startConnection();
                appendIrcStyledText(_("Receiving \"%1\" from %2").arg(filename).arg(nick),8,false);
                slotCmdTransfers();
            }
        }
    }
}

void dxirc::slotIrcDccResume(QString nick, QString filename, QString port, QString position)
{
    int engineIndex = -1;
    for(int i=0; i<m_dccengines.size(); i++)
    {
        if(m_dccengines[i]->isForResume(nick, filename, port.toInt()))
        {
            engineIndex = i;
            break;
        }
    }
    if(engineIndex == -1) return;
    int dccIndex = -1;
    for(int i=0; i<m_dccfilesList.size(); i++)
    {
        if(m_dccfilesList[i].path == m_dccengines[engineIndex]->getDccFile().path
                && m_dccfilesList[i].size == m_dccengines[engineIndex]->getDccFile().size
                && m_dccfilesList[i].nick == m_dccengines[engineIndex]->getDccFile().nick)
        {
            dccIndex = i;
            break;
        }
    }
    if(dccIndex == -1) return;
    quint32 pos = position.toULong();
    if(pos >= m_dccfilesList[dccIndex].size) return;
    IrcEngine *server = static_cast<IrcEngine*>(sender());
    m_dccengines[engineIndex]->setDccPosition(pos);
    m_dccfilesList[dccIndex].ip = m_dccengines[engineIndex]->getDccFile().ip;
    m_dccfilesList[dccIndex].port = m_dccengines[engineIndex]->getDccFile().port;
    m_dccfilesList[dccIndex].currentPosition = pos;
    m_dccfilesList[dccIndex].speed = 0;
    m_dccfilesList[dccIndex].finishedPosition = pos;
    m_dccfilesList[dccIndex].canceled = false;
    server->sendCtcp(nick, "DCC ACCEPT "+dxutils::removeSpaces(m_dccengines[engineIndex]->getDccFile().path.section('/',-1))+" "+QString::number(m_dccengines[engineIndex]->getDccFile().port)+" "+position);
}

void dxirc::slotIrcDccPresume(int token, QString position)
{
    int dccIndex = -1;
    for(int i=0; i<m_dccfilesList.size(); i++)
    {
        if(m_dccfilesList[i].token == token)
        {
            dccIndex = i;
            break;
        }
    }
    if(dccIndex == -1) return;
    quint32 pos = position.toULong();
    if(pos >= m_dccfilesList[dccIndex].size) return;
    IrcEngine *server = static_cast<IrcEngine*>(sender());
    m_dccfilesList[dccIndex].currentPosition = pos;
    m_dccfilesList[dccIndex].speed = 0;
    m_dccfilesList[dccIndex].finishedPosition = pos;
    m_dccfilesList[dccIndex].canceled = false;
    server->sendCtcp(m_dccfilesList[dccIndex].nick, "DCC ACCEPT "+dxutils::removeSpaces(m_dccfilesList[dccIndex].path.section('/',-1))+" 0 "+position+" "+QString::number(token));
}

void dxirc::slotIrcDccAccept(QString nick, QString filename, QString port, QString position)
{
    int dccIndex = -1;
    for(int i=0; i<m_dccfilesList.size(); i++)
    {
        if(m_dccfilesList[i].nick == nick && QFileInfo(m_dccfilesList[i].path).fileName() == filename
                && m_dccfilesList[i].port == port.toInt())
        {
            dccIndex = i;
            break;
        }
    }
    if(dccIndex == -1) return;
    quint32 pos = position.toULong();
    if(pos >= m_dccfilesList[dccIndex].size) return;
    IrcEngine *server = static_cast<IrcEngine*>(sender());
    m_dccfilesList[dccIndex].currentPosition = pos;
    m_dccfilesList[dccIndex].speed = 0;
    m_dccfilesList[dccIndex].finishedPosition = pos;
    m_dccfilesList[dccIndex].canceled = false;
    DccEngine *engine = new DccEngine(m_dccfilesList[dccIndex], server);
    connect(engine, SIGNAL(dccError(QString)), this, SLOT(slotDccError(QString)));
    connect(engine, SIGNAL(dccPosition(DccFile)), this, SLOT(slotDccPosition(DccFile)));
    m_dccengines.append(engine);
    engine->startConnection();
    appendIrcStyledText(_("Receiving \"%1\" from %2").arg(filename).arg(nick),8,false);
    if(!preferences.m_autoDccFile) slotCmdTransfers();
}

void dxirc::slotIrcDccPaccept(int token, QString position)
{
    int dccIndex = -1;
    for(int i=0; i<m_dccfilesList.size(); i++)
    {
        if(m_dccfilesList[i].token == token)
        {
            dccIndex = i;
            break;
        }
    }
    if(dccIndex == -1) return;
    quint32 pos = position.toULong();
    if(pos >= m_dccfilesList[dccIndex].size) return;
    IrcEngine *server = static_cast<IrcEngine*>(sender());
    m_dccfilesList[dccIndex].currentPosition = pos;
    m_dccfilesList[dccIndex].speed = 0;
    m_dccfilesList[dccIndex].finishedPosition = pos;
    m_dccfilesList[dccIndex].canceled = false;
    m_dccfilesList[dccIndex].ip = server->getLocalIP();
    m_dccfilesList[dccIndex].port = 0;
    DccEngine *engine = new DccEngine(m_dccfilesList[dccIndex], server);
    connect(engine, SIGNAL(dccError(QString)), this, SLOT(slotDccError(QString)));
    connect(engine, SIGNAL(dccPosition(DccFile)), this, SLOT(slotDccPosition(DccFile)));
    m_dccengines.append(engine);
    engine->startConnection();
    appendIrcStyledText(_("Receiving \"%1\" from %2").arg(QFileInfo(m_dccfilesList[dccIndex].path).fileName()).arg(m_dccfilesList[dccIndex].nick),8,false);
    if(!preferences.m_autoDccFile) slotCmdTransfers();
}

void dxirc::slotIrcRaw(QString line)
{
#ifdef HAVE_LUA
    if(!m_scripts.size() || !m_scriptEvents.size()) return;
    IrcEngine *server = static_cast<IrcEngine*>(sender());
    for(int i=0; i<m_scriptEvents.size(); i++)
    {
        if(QString::compare("raw", m_scriptEvents[i].name, Qt::CaseInsensitive) == 0)
        {
            for(int j=0; j<m_scripts.size(); j++)
            {
                if(QString::compare(m_scriptEvents[i].script, m_scripts[j].name, Qt::CaseInsensitive) == 0)
                {
                    lua_getglobal(m_scripts[j].L, m_scriptEvents[i].funcname.toUtf8().constData());
                    if(lua_type(m_scripts[j].L, -1) != LUA_TFUNCTION) lua_pop(m_scripts[j].L, 1);
                    else
                    {
                        lua_pushstring(m_scripts[j].L, line.toUtf8().constData());
                        lua_newtable(m_scripts[j].L);
                        lua_pushstring(m_scripts[j].L, "server");
                        lua_pushstring(m_scripts[j].L, server->getServerName().toUtf8().constData());
                        lua_settable(m_scripts[j].L, -3);
                        lua_pushstring(m_scripts[j].L, "port");
                        lua_pushnumber(m_scripts[j].L, server->getServerPort());
                        lua_settable(m_scripts[j].L, -3);
                        lua_pushstring(m_scripts[j].L, "nick");
                        lua_pushstring(m_scripts[j].L, server->getNickName().toUtf8().constData());
                        lua_settable(m_scripts[j].L, -3);
                        if(lua_pcall(m_scripts[j].L, 2, 0, 0))
                        {
                            appendIrcStyledText(_("Lua plugin: error calling %1 %2").arg(m_scriptEvents[i].funcname).arg(QString::fromUtf8(lua_tostring(m_scripts[j].L, -1))), 4, false);
                            lua_pop(m_scripts[j].L, 1);
                        }
                    }
                }
            }
        }
    }
#endif //HAVE_LUA
}

void dxirc::slotIrcBoatsInvite(QString nick)
{
    IrcEngine *server = static_cast<IrcEngine*>(sender());
    if(QMessageBox::question(this, _("Question"),
        _("%1 invite you into Boats game.\n Do you want play?").arg(nick),
        QMessageBox::Yes|QMessageBox::No) == QMessageBox::Yes)
    {
        createBoatsTab(nick, server, false);
        server->sendCtcp(nick, "BOATS ACCEPT");
    }
    else
    {
        server->sendCtcp(nick, "BOATS DECLINE");
    }
}

void dxirc::slotIrcBoatsAccept(QString nick)
{
    createBoatsTab(nick, static_cast<IrcEngine*>(sender()), true);
}

void dxirc::slotIrc323()
{
    IrcEngine *server = static_cast<IrcEngine*>(sender());
    ListDialog *dialog = new ListDialog(server, server->getChannelsList(), this);
    dialog->setModal(true);
    dialog->show();
}

void dxirc::connectServer(QString hostname, int port, QString pass, QString nick, QString rname, QString channels, QString commands, bool ssl)
{
    dxUtils.debugLine("ConnectServer");
    if(m_ircengines.size() == 1 && !m_ircengines[0]->getConnected() && !m_ircengines[0]->getConnecting())
    {
        m_ircengines[0]->setServerName(hostname);
        m_ircengines[0]->setServerPort(port);
        m_ircengines[0]->setServerPassword(pass);
        nick.length() ? m_ircengines[0]->setNickName(nick) : m_ircengines[0]->setNickName("_xxx_");
        nick.length() ? m_ircengines[0]->setUserName(nick) : m_ircengines[0]->setUserName("_xxx_");
        rname.length() ? m_ircengines[0]->setRealName(rname) : m_ircengines[0]->setRealName(nick.length() ? nick : "_xxx_");
        if(channels.length()>1) m_ircengines[0]->setStartChannels(channels);
        if(commands.length()) m_ircengines[0]->setStartCommands(commands);
#ifdef QT_NO_OPENSSL
        ssl = false;
#endif //QT_NO_OPENSSL
        m_ircengines[0]->setUseSsl(ssl);
        if(!m_tabbook->count())
        {
            createIrcTab(hostname, SERVER, m_ircengines[0], QIcon(dxutils::getIconForType(SERVER)));
        }
        else
        {
            IrcTabItem *tab = NULL;
            bool needNewTab = true;
            for(int i = 0; i<m_tabbook->count(); i++)
            {
                tab = qobject_cast<IrcTabItem*>(m_tabbook->widget(i));
                if(tab && tab->getType()==SERVER)
                {
                    m_tabbook->setTabText(i, hostname);
                    needNewTab = false;
                    break;
                }
            }
            if(needNewTab) createIrcTab(hostname, SERVER, m_ircengines[0], QIcon(dxutils::getIconForType(SERVER)));
        }
        m_ircengines[0]->clearAttempts();
        m_ircengines[0]->startConnection();
    }
    else if(!serverExist(hostname, port, nick))
    {
        IrcEngine *server = new IrcEngine(channels.length()>1 ? channels : "", commands.length() ? commands : "");
        m_ircengines.prepend(server);
        connect(m_ircengines[0], SIGNAL(ircNewChannel(QString)), this, SLOT(slotIrcNewChannel(QString)));
        connect(m_ircengines[0], SIGNAL(ircQuery(QString)), this, SLOT(slotIrcQuery(QString)));
        connect(m_ircengines[0], SIGNAL(ircDccChat(QString,QString,int)), this, SLOT(slotIrcDccChat(QString,QString,int)));
        connect(m_ircengines[0], SIGNAL(ircInvite(QString,QString)), this, SLOT(slotIrcInvite(QString,QString)));
        connect(m_ircengines[0], SIGNAL(ircEndMotd()), this, SLOT(slotIrcEndMotd()));
        connect(m_ircengines[0], SIGNAL(ircPrivmsg(QString,QString,QString)), this, SLOT(slotIrcPrivmsgAndAction(QString,QString,QString)));
        connect(m_ircengines[0], SIGNAL(ircAction(QString,QString,QString)), this, SLOT(slotIrcPrivmsgAndAction(QString,QString,QString)));
        connect(m_ircengines[0], SIGNAL(ircPart(QString,QString,QString)), this, SLOT(slotIrcPart(QString,QString)));
        connect(m_ircengines[0], SIGNAL(ircKick(QString,QString,QString,QString)), this, SLOT(slotIrcKick(QString,QString,QString)));
        connect(m_ircengines[0], SIGNAL(ircRaw(QString)), this, SLOT(slotIrcRaw(QString)));
        connect(m_ircengines[0], SIGNAL(ircDisconnect(QString)), this, SLOT(slotIrcDisconnect(QString)));
        connect(m_ircengines[0], SIGNAL(ircConnect(QString)), this, SLOT(slotIrcConnectAndReconnect(QString)));
        connect(m_ircengines[0], SIGNAL(ircDisconnect(QString)), this, SLOT(slotIrcConnectAndReconnect(QString)));
        connect(m_ircengines[0], SIGNAL(ircQuit(QString,QString)), this, SLOT(slotIrcQuit(QString)));
        connect(m_ircengines[0], SIGNAL(ircDccIn(QString,QString,QString,QString,QString)), this, SLOT(slotIrcDccIn(QString,QString,QString,QString,QString)));
        connect(m_ircengines[0], SIGNAL(ircDccMyToken(QString,int,int)), this, SLOT(slotIrcDccMyToken(QString,int,int)));
        connect(m_ircengines[0], SIGNAL(ircDccToken(QString,QString,QString,int)), this, SLOT(slotIrcDccToken(QString,QString,QString,int)));
        connect(m_ircengines[0], SIGNAL(ircDccAccept(QString,QString,QString,QString)), this, SLOT(slotIrcDccAccept(QString,QString,QString,QString)));
        connect(m_ircengines[0], SIGNAL(ircDccPaccept(int,QString)), this, SLOT(slotIrcDccPaccept(int,QString)));
        connect(m_ircengines[0], SIGNAL(ircDccResume(QString,QString,QString,QString)), this, SLOT(slotIrcDccResume(QString,QString,QString,QString)));
        connect(m_ircengines[0], SIGNAL(ircDccPresume(int,QString)), this, SLOT(slotIrcDccPresume(int,QString)));
        connect(m_ircengines[0], SIGNAL(ircBoatsInvite(QString)), this, SLOT(slotIrcBoatsInvite(QString)));
        connect(m_ircengines[0], SIGNAL(ircBoatsAccept(QString)), this, SLOT(slotIrcBoatsAccept(QString)));
        connect(m_ircengines[0], SIGNAL(irc323()), this, SLOT(slotIrc323()));
        m_ircengines[0]->setServerName(hostname);
        m_ircengines[0]->setServerPort(port);
        m_ircengines[0]->setServerPassword(pass);
        nick.length() ? m_ircengines[0]->setNickName(nick) : m_ircengines[0]->setNickName("_xxx_");
        nick.length() ? m_ircengines[0]->setUserName(nick) : m_ircengines[0]->setUserName("_xxx_");
        rname.length() ? m_ircengines[0]->setRealName(rname) : m_ircengines[0]->setRealName(nick.length() ? nick : "_xxx_");
#ifdef QT_NO_OPENSSL
        ssl = false;
#endif //QT_NO_OPENSSL
        m_ircengines[0]->setUseSsl(ssl);
        createIrcTab(hostname, SERVER, m_ircengines[0], QIcon(dxutils::getIconForType(SERVER)));
        m_ircengines[0]->clearAttempts();
        m_ircengines[0]->startConnection();
    }
    updateMenus();
}

void dxirc::updateTabPosition()
{
    switch(preferences.m_tabPosition) {
    case 0: //bottom
        m_tabbook->setTabPosition(QTabWidget::South);
        break;
    case 1: //left
        m_tabbook->setTabPosition(QTabWidget::West);
        break;
    case 2: //top
        m_tabbook->setTabPosition(QTabWidget::North);
        break;
    case 3: //right
        m_tabbook->setTabPosition(QTabWidget::East);
        break;
    default:
        m_tabbook->setTabPosition(QTabWidget::South);
    }
}

void dxirc::updateMenus()
{
    if(m_tabbook->count())
    {
        m_closeTab->setEnabled(true);
        m_clearTab->setEnabled(true);
        m_clearTabs->setEnabled(true);
    }
    else
    {
        m_closeTab->setEnabled(false);
        m_clearTab->setEnabled(false);
        m_clearTabs->setEnabled(false);
    }
    bool someConnected = false;
    for(int i = 0; i < m_ircengines.size(); i++)
    {
        if(m_ircengines[i]->getConnected())
        {
            someConnected = true;
            break;
        }
    }
    if(someConnected)
    {
        m_disconnect->setEnabled(true);
        m_reconnect->setEnabled(true);
    }
    else
    {
        m_disconnect->setEnabled(false);
        m_reconnect->setEnabled(false);
    }
#ifdef HAVE_ENCHANT
    if(preferences.m_useSpell && dxUtils.getAvailableSpellLangsNum())
        m_spellCombo->setEnabled(true);
    else
        m_spellCombo->setEnabled(false);
#endif
    if(preferences.m_logging)
        m_logviewer->setEnabled(true);
    else
        m_logviewer->setEnabled(false);
}

bool caseLessThan(const void *s1, const void *s2)
{
    dxTabItem *a = (dxTabItem*)s1;
    dxTabItem *b = (dxTabItem*)s2;
    return (a->getType()==SERVER?a->getRealServerName().toLower():a->getRealServerName().toLower()+a->getTabName().toLower())
            < (b->getType()==SERVER?b->getRealServerName().toLower():b->getRealServerName().toLower()+b->getTabName().toLower());
}

void dxirc::updateServerWindow()
{
    if(preferences.m_ownServerWindow)
    {
        foreach(IrcEngine *server, m_ircengines)
        {
            createIrcTab(server->getNetworkName(), SERVER, server, QIcon(dxutils::getIconForType(SERVER)));
        }
    }
    else
    {
        dxTabItem *dx = NULL;
        for(int i=m_tabbook->count()-1; i>-1; i--)
        {
            dx = qobject_cast<dxTabItem*>(m_tabbook->widget(i));
            if(dx)
            {
                if(dx->getType() == SERVER)
                {
                    m_tabbook->removeTab(i);
                    dx->deleteLater();
                }
            }
        }
        sortTabs();
        updateMenus();
    }
}

void dxirc::sortTabs()
{
    if(preferences.m_disableSortTabs)
        return;
    if(m_tabbook->count())
    {
        int id = getCurrentTabId();
        QList<dxTabItem*> tabs;
        dxTabItem *item = NULL;
        for(int i=0; i<m_tabbook->count(); i++)
        {
            item = qobject_cast<dxTabItem*>(m_tabbook->widget(i));
            if(item)
            {
                tabs.append(item);
            }
        }
        qSort(tabs.begin(), tabs.end(), caseLessThan);
        for(int i=0; i<tabs.size(); i++)
        {
            m_tabbook->insertTab(i, tabs[i], QIcon(dxutils::getIconForType(tabs[i]->getType())), tabs[i]->getTabName());
        }
        setCurrentTabById(id);
    }
}

quint32 dxirc::createIrcTab(const QString& tabtext, TYPE typ, IrcEngine* engine, QIcon icon)
{
    IrcTabItem *tab = new IrcTabItem(tabtext, engine, m_tabbook, typ, m_lastID);
    sendNewTab(engine, tabtext, m_lastID, false, false, typ);
    m_tabbook->addTab(tab, icon, tabtext);
    m_tabbook->setTabTextColor(m_tabbook->count()-1, palette().windowText().color());
    updateTabPosition();
    m_lastID++;
    connect(tab, SIGNAL(highlight(TYPE)), this, SLOT(slotHighlight(TYPE)));
    connect(tab, SIGNAL(newmsg(TYPE)), this, SLOT(slotNewmsg(TYPE)));
    connect(tab, SIGNAL(luaRequest(LuaRequest)), this, SLOT(slotLuaRequest(LuaRequest)));
    connect(tab, SIGNAL(newtetris()), this, SLOT(slotNewtetris()));
    connect(tab, SIGNAL(connectDialog()), this, SLOT(slotCmdConnect()));
    connect(tab, SIGNAL(connectServer(ServerInfo)), this, SLOT(slotConnectServer(ServerInfo)));
    connect(tab, SIGNAL(ircDccServer(IrcEngine*,QString)), this, SLOT(slotIrcDccServer(IrcEngine*,QString)));
    connect(tab, SIGNAL(ircDccOut(IrcEngine*,QString,QString)), this, SLOT(slotIrcDccOut(IrcEngine*,QString,QString)));
    connect(tab, SIGNAL(ircDccPout(IrcEngine*,QString,QString)), this, SLOT(slotIrcDccPout(IrcEngine*,QString,QString)));
    connect(tab, SIGNAL(ircPrivmsg(IrcEngine*,QString,QString,QString)), this, SLOT(slotIrcPrivmsgAndAction(IrcEngine*,QString,QString,QString)));
    connect(tab, SIGNAL(ircAction(IrcEngine*,QString,QString,QString)), this, SLOT(slotIrcPrivmsgAndAction(IrcEngine*,QString,QString,QString)));
    connect(tab, SIGNAL(ircQuery(IrcEngine*,QString)), this, SLOT(slotIrcQuery(IrcEngine*,QString)));
    connect(tab, SIGNAL(ownMsg(QString)), this, SLOT(slotOwnMsg(QString)));
    connect(tab, SIGNAL(command(QString)), this, SLOT(slotCommand(QString)));
    connect(tab, SIGNAL(addIgnoreCommand(QString)), this, SLOT(slotAddIgnoreCommand(QString)));
    connect(tab, SIGNAL(removeIgnoreCommand(QString)), this, SLOT(slotRemoveIgnoreCommand(QString)));
    connect(tab, SIGNAL(addIgnoreUser(QString)), this, SLOT(slotAddIgnoreUser(QString)));
    connect(tab, SIGNAL(removeIgnoreUser(QString)), this, SLOT(slotRemoveIgnoreUser(QString)));
    connect(tab, SIGNAL(ircQuit()), this, SLOT(slotCmdQuit()));
    connect(tab, SIGNAL(newlog(QString)), this, SLOT(slotNewlog(QString)));
    connect(tab, SIGNAL(newname(QString)), this, SLOT(slotNewname(QString)));
    sortTabs();
    updateMenus();
    return m_lastID-1;
}

quint32 dxirc::createDccTab(const QString &mynick, const QString &nick, const QString &address, int portD, int portH, bool listen, IrcEngine *engine)
{
    DccTabItem *tab = new DccTabItem(m_tabbook, mynick, nick, address, portD, portH, listen, m_lastID, engine);
    sendNewTab(NULL, nick, m_lastID, false, true, DCC);
    m_tabbook->addTab(tab, QIcon(dxutils::getIconForType(DCC)), nick);
    m_tabbook->setTabTextColor(m_tabbook->count()-1, palette().windowText().color());
    updateTabPosition();
    m_lastID++;
    connect(tab, SIGNAL(highlight(TYPE)), this, SLOT(slotHighlight(TYPE)));
    connect(tab, SIGNAL(newmsg(TYPE)), this, SLOT(slotNewmsg(TYPE)));
    connect(tab, SIGNAL(newtetris()), this, SLOT(slotNewtetris()));
    connect(tab, SIGNAL(ownMsg(QString)), this, SLOT(slotOwnMsg(QString)));
    connect(tab, SIGNAL(command(QString)), this, SLOT(slotCommand(QString)));
    connect(tab, SIGNAL(luaRequest(LuaRequest)), this, SLOT(slotLuaRequest(LuaRequest)));
    connect(tab, SIGNAL(newlog(QString)), this, SLOT(slotNewlog(QString)));
    connect(tab, SIGNAL(newname(QString)), this, SLOT(slotNewname(QString)));
    sortTabs();
    updateMenus();
    return m_lastID-1;
}

quint32 dxirc::createBoatsTab(const QString &tabtext, IrcEngine *engine, bool first)
{
    BoatsTabItem *tab = new BoatsTabItem(tabtext, first, m_tabbook, m_lastID, engine);
    sendNewTab(engine, tabtext, m_lastID, false, false, BOATS);
    m_tabbook->addTab(tab, QIcon(dxutils::getIconForType(DCC)), tabtext);
    m_tabbook->setTabTextColor(m_tabbook->count()-1, palette().windowText().color());
    updateTabPosition();
    m_lastID++;
    connect(tab, SIGNAL(newname(QString)), this, SLOT(slotNewname(QString)));
    sortTabs();
    updateMenus();
    return m_lastID-1;
}

void dxirc::slotNewtetris()
{
    TetrisTabItem *tab = new TetrisTabItem(m_tabbook, m_lastID);
    sendNewTab(NULL, "tetris", m_lastID, true, false, OTHER);
    m_tabbook->addTab(tab, "tetris");
    m_tabbook->setTabTextColor(m_tabbook->count()-1, palette().windowText().color());
    updateTabPosition();
    m_lastID++;
    updateMenus();
    m_tabbook->setCurrentIndex(m_tabbook->count()-1);
}

//send event to script for new created tab
void dxirc::sendNewTab(IrcEngine *server, const QString &name, int id, bool isTetris, bool isDccTab, TYPE type)
{
#ifdef HAVE_LUA
    if(!m_scripts.size() || !m_scriptEvents.size())
        return;
    for(int i=0; i<m_scriptEvents.size(); i++)
    {
        if(QString::compare("newtab", m_scriptEvents[i].name, Qt::CaseInsensitive) == 0)
        {
            for(int j=0; j<m_scripts.size(); j++)
            {
                if(QString::compare(m_scriptEvents[i].script, m_scripts[j].name, Qt::CaseInsensitive) == 0)
                {
                    lua_getglobal(m_scripts[j].L, m_scriptEvents[i].funcname.toUtf8().constData());
                    if (lua_type(m_scripts[j].L, -1) != LUA_TFUNCTION) lua_pop(m_scripts[j].L, 1);
                    else
                    {
                        lua_newtable(m_scripts[j].L);
                        lua_pushstring(m_scripts[j].L, "id");
                        lua_pushinteger(m_scripts[j].L, id);
                        lua_settable(m_scripts[j].L, -3);
                        lua_pushstring(m_scripts[j].L, "name");
                        lua_pushstring(m_scripts[j].L, name.toUtf8().constData());
                        lua_settable(m_scripts[j].L, -3);
                        if(isTetris)
                        {
                            lua_pushstring(m_scripts[j].L, "type");
                            lua_pushstring(m_scripts[j].L, "tetris");
                            lua_settable(m_scripts[j].L, -3);
                        }
                        else if(isDccTab)
                        {
                            lua_pushstring(m_scripts[j].L, "type");
                            lua_pushstring(m_scripts[j].L, "dccchat");
                            lua_settable(m_scripts[j].L, -3);
                        }
                        else
                        {
                            switch(type) {
                                case SERVER:
                                {
                                    lua_pushstring(m_scripts[j].L, "type");
                                    lua_pushstring(m_scripts[j].L, "server");
                                    lua_settable(m_scripts[j].L, -3);
                                }break;
                                case CHANNEL:
                                {
                                    lua_pushstring(m_scripts[j].L, "type");
                                    lua_pushstring(m_scripts[j].L, "channel");
                                    lua_settable(m_scripts[j].L, -3);
                                }break;
                                case QUERY:
                                {
                                    lua_pushstring(m_scripts[j].L, "type");
                                    lua_pushstring(m_scripts[j].L, "query");
                                    lua_settable(m_scripts[j].L, -3);
                                }break;
                                case OTHER:
                                {
                                    lua_pushstring(m_scripts[j].L, "type");
                                    lua_pushstring(m_scripts[j].L, "other");
                                    lua_settable(m_scripts[j].L, -3);
                                }break;
                                case DCC:
                                {
                                    lua_pushstring(m_scripts[j].L, "type");
                                    lua_pushstring(m_scripts[j].L, "dcc");
                                    lua_settable(m_scripts[j].L, -3);
                                }break;
                                case BOATS:
                                {
                                    lua_pushstring(m_scripts[j].L, "type");
                                    lua_pushstring(m_scripts[j].L, "boats");
                                    lua_settable(m_scripts[j].L, -3);
                                }break;
                            }
                        }
                        lua_pushstring(m_scripts[j].L, "servername");
                        lua_pushstring(m_scripts[j].L, server ? server->getServerName().toUtf8().constData() : "");
                        lua_settable(m_scripts[j].L, -3);
                        lua_pushstring(m_scripts[j].L, "port");
                        lua_pushinteger(m_scripts[j].L, server ? server->getServerPort() : 0);
                        lua_settable(m_scripts[j].L, -3);
                        lua_pushstring(m_scripts[j].L, "nick");
                        lua_pushstring(m_scripts[j].L, server ? server->getNickName().toUtf8().constData() : "");
                        lua_settable(m_scripts[j].L, -3);
                        if (lua_pcall(m_scripts[j].L, 1, 0, 0))
                        {
                            appendIrcStyledText(_("Lua plugin: error calling %1 %2").arg(m_scriptEvents[i].funcname).arg(QString::fromUtf8(lua_tostring(m_scripts[j].L, -1))), 4, false);
                            lua_pop(m_scripts[j].L, 1);
                        }
                    }
                }
            }
        }
    }
#endif //HAVE_LUA
}

bool dxirc::serverExist(QString server, int port, QString nick)
{
    for(int i = 0; i < m_ircengines.size(); i++)
    {
        if(m_ircengines[i]->getServerName().toLower() == server.toLower()
                && m_ircengines[i]->getServerPort() == port
                && m_ircengines[i]->getNickName().toLower() == nick.toLower()
                && m_ircengines[i]->getConnected()) return true;
    }
    return false;
}

void dxirc::slotIrcNewChannel(QString channel)
{
    IrcEngine *server = static_cast<IrcEngine *>(sender());
    int serverTabIndex = getServerTab(server);
    if(tabExist(server, channel))
    {
        return;
    }
    if(serverTabIndex != -1 && !preferences.m_ownServerWindow)
    {
        static_cast<IrcTabItem*>(m_tabbook->widget(serverTabIndex))->setType(CHANNEL, channel);
        m_tabbook->setTabIcon(serverTabIndex, QIcon(dxutils::getIconForType(CHANNEL)));
        sortTabs();
    }
    else
    {
        createIrcTab(channel, CHANNEL, server, QIcon(dxutils::getIconForType(CHANNEL)));
    }
    sortTabs();
    updateMenus();
}

void dxirc::slotIrcQuery(QString with)
{
    slotIrcQuery(static_cast<IrcEngine *>(sender()), with);
}

void dxirc::slotIrcQuery(IrcEngine *server, QString with)
{
    if(tabExist(server, with))
        return;
    int serverTabIndex = getServerTab(server);
    if(serverTabIndex != -1 && !preferences.m_ownServerWindow)
    {
        static_cast<IrcTabItem*>(m_tabbook->widget(serverTabIndex))->setType(QUERY, with);
        m_tabbook->setCurrentIndex(serverTabIndex);
        m_tabbook->setTabIcon(serverTabIndex, QIcon(dxutils::getIconForType(QUERY)));
        sortTabs();
    }
    else
    {
        setCurrentTabById(createIrcTab(with, QUERY, server, QIcon(dxutils::getIconForType(QUERY))));
    }
    updateMenus();
}

//Handle for entered text in IrcTab for ownmsg in lua scripting
void dxirc::slotOwnMsg(QString msg)
{
#ifdef HAVE_LUA
    dxTabItem *tab = static_cast<dxTabItem*>(sender());
    if(!m_scripts.size() || !m_scriptEvents.size())
    {
        tab->setHasOwnMsg(false);
        return;
    }
    tab->setHasOwnMsg(hasOwnMsg());
    for(int i=0; i<m_scriptEvents.size(); i++)
    {
        if(QString::compare("ownmsg", m_scriptEvents[i].name, Qt::CaseInsensitive) == 0)
        {
            for(int j=0; j<m_scripts.size(); j++)
            {
                if(QString::compare(m_scriptEvents[i].script, m_scripts[j].name, Qt::CaseInsensitive) == 0)
                {
                    lua_getglobal(m_scripts[j].L, m_scriptEvents[i].funcname.toUtf8().constData());
                    if (lua_type(m_scripts[j].L, -1) != LUA_TFUNCTION) lua_pop(m_scripts[j].L, 1);
                    else
                    {
                        lua_pushstring(m_scripts[j].L, msg.toUtf8().constData());
                        lua_pushinteger(m_scripts[j].L, tab->getID());
                        if (lua_pcall(m_scripts[j].L, 2, 0, 0))
                        {
                            appendIrcStyledText(_("Lua plugin: error calling %1 %2").arg(m_scriptEvents[i].funcname).arg(QString::fromUtf8(lua_tostring(m_scripts[j].L, -1))), 4, false);
                            lua_pop(m_scripts[j].L, 1);
                        }
                    }
                }
            }
        }
    }
#endif //HAVE_LUA
}

//Handle for entered command in IrcTab for all in lua scripting
void dxirc::slotCommand(QString command)
{
#ifdef HAVE_LUA
    dxTabItem *tab = static_cast<dxTabItem*>(sender());
    if(!m_scripts.size() || !m_scriptEvents.size())
    {
        tab->setHasAllCommand(false);
        return;
    }
    tab->setHasAllCommand(hasAllCommand());
    for(int i=0; i<m_scriptEvents.size(); i++)
    {
        if(QString::compare("all", m_scriptEvents[i].name, Qt::CaseInsensitive) == 0)
        {
            for(int j=0; j<m_scripts.size(); j++)
            {
                if(QString::compare(m_scriptEvents[i].script, m_scripts[j].name, Qt::CaseInsensitive) == 0)
                {
                    lua_getglobal(m_scripts[j].L, m_scriptEvents[i].funcname.toUtf8().constData());
                    if (lua_type(m_scripts[j].L, -1) != LUA_TFUNCTION) lua_pop(m_scripts[j].L, 1);
                    else
                    {
                        if(command[0].toLatin1() == '/')
                        {
                            lua_pushstring(m_scripts[j].L, command.section(' ',0,0).section('/',1).toUtf8().constData());
                            lua_pushstring(m_scripts[j].L, command.section(' ',1).toUtf8().constData());
                        }
                        else
                        {
                            lua_pushnil(m_scripts[j].L);
                            lua_pushstring(m_scripts[j].L, command.toUtf8().constData());
                        }
                        lua_pushinteger(m_scripts[j].L, tab->getID());
                        if (lua_pcall(m_scripts[j].L, 3, 0, 0))
                        {
                            appendIrcStyledText(_("Lua plugin: error calling %1 %2").arg(m_scriptEvents[i].funcname).arg(QString::fromUtf8(lua_tostring(m_scripts[j].L, -1))), 4, false);
                            lua_pop(m_scripts[j].L, 1);
                        }
                    }
                }
            }
        }
    }
#endif //HAVE_LUA
}

void dxirc::slotNewlog(QString logpath)
{
    dxTabItem *tab = static_cast<dxTabItem*>(sender());
    int id = getTabId(m_tabbook->tabText(m_tabbook->indexOf(tab)));
    if(id==-1) id = createIrcTab(m_tabbook->tabText(m_tabbook->indexOf(tab)), OTHER, NULL, QIcon(dxutils::getIconForType(OTHER)));
    IrcTabItem *logtab = static_cast<IrcTabItem*>(getTabItemById(id));
    logtab->showLog(logpath);
    setCurrentTabById(id);
}

void dxirc::slotNewname(QString name)
{
    int index = m_tabbook->indexOf(static_cast<QWidget*>(sender()));
    if(index==-1) return;
    m_tabbook->setTabText(index, name);
}

void dxirc::slotDccError(QString error)
{
    appendIrcStyledText(error, 4, false);
}

void dxirc::slotDccPosition(DccFile dcc)
{
    int index = -1;
    for(int i=0; i<m_dccfilesList.size(); i++)
    {
        if(m_dccfilesList[i].path == dcc.path)
        {
            index = i;
            break;
        }
    }
    if(index == -1) return;
    m_dccfilesList[index].speed = dcc.speed;
    m_dccfilesList[index].currentPosition = dcc.currentPosition;
    m_dccfilesList[index].finishedPosition = dcc.finishedPosition;
    m_dccfilesList[index].canceled = dcc.canceled;
    m_dccfilesList[index].ip = dcc.ip;
    m_dccfilesList[index].port = dcc.port;
    m_dccfilesList[index].finished = dcc.finished;
    m_dccfilesList[index].started = dcc.started;
    if(m_dccfilesList[index].type == DCC_IN  || m_dccfilesList[index].type == DCC_PIN)
    {
        if(m_dccfilesList[index].finished)
        {
            if(m_dccfilesList[index].canceled)
            {
                appendIrcStyledText(_("Download of \"%1\" from %2 canceled").arg(QFileInfo(m_dccfilesList[index].path).fileName()).arg(m_dccfilesList[index].nick), 4, false);
            }
            else
            {
                appendIrcStyledText(_("Download of \"%1\" from %2 finished").arg(QFileInfo(m_dccfilesList[index].path).fileName()).arg(m_dccfilesList[index].nick), 8, false);
                if(!isVisible()) setTrayIcon(false, true);
            }
        }
        else
        {
            if(m_dccfilesList[index].canceled)
            {
                appendIrcStyledText(_("Download of \"%1\" from %2 canceled").arg(QFileInfo(m_dccfilesList[index].path).fileName()).arg(m_dccfilesList[index].nick), 4, false);
            }
        }
    }
    else
    {
        if(m_dccfilesList[index].finished)
        {
            if(m_dccfilesList[index].canceled)
            {
                appendIrcStyledText(_("Upload of \"%1\" to %2 canceled").arg(QFileInfo(m_dccfilesList[index].path).fileName()).arg(m_dccfilesList[index].nick), 4, false);
            }
            else
            {
                appendIrcStyledText(_("Upload of \"%1\" to %2 finished").arg(QFileInfo(m_dccfilesList[index].path).fileName()).arg(m_dccfilesList[index].nick), 8, false);
                if(!isVisible()) setTrayIcon(false, true);
            }
        }
        else
        {
            if(m_dccfilesList[index].canceled)
            {
                appendIrcStyledText(_("Upload of \"%1\" to %2 canceled").arg(QFileInfo(m_dccfilesList[index].path).fileName()).arg(m_dccfilesList[index].nick), 4, false);
            }
        }
    }
}

int dxirc::getServerTab(IrcEngine *server)
{
    IrcTabItem *item = NULL;
    for(int i=0; i<m_tabbook->count(); i++)
    {
        item = qobject_cast<IrcTabItem*>(m_tabbook->widget(i));
        if(item && item->getType() == SERVER && item->isSameServer(server)) return i;
    }
    return -1;
}

bool dxirc::isFriend(QString nick, QString on, QString network)
{
    bool bnick = false;
    bool bchannel = false;
    bool bnetwork = false;
    for(int i=0; i<preferences.m_friendsList.size(); i++)
    {
        QString fnick = preferences.m_friendsList[i].nick;
        if(QRegExp(QString("\\<"+fnick+"\\>").toLower().replace("*","\\w*")).exactMatch(nick.toLower())) bnick = true;
        if(bnick)
        {
            if(preferences.m_friendsList[i].channel == "all") bchannel = true;
            if(preferences.m_friendsList[i].channel.contains(','))
            {
                for(int j=1; j<preferences.m_friendsList[i].channel.count(',')+2; j++)
                {
                    if(QRegExp(QString(dxutils::getParam(preferences.m_friendsList[i].channel, j, false, ',').toLower()+"\\>").replace("*","\\w*")).exactMatch(on.toLower()))
                    {
                        bchannel = true;
                        break;
                    }
                }
            }
            else
            {
                if(QRegExp(QString(preferences.m_friendsList[i].channel+"\\>").toLower().replace("*","\\w*")).exactMatch(on.toLower())) bchannel = true;
            }
            if(preferences.m_friendsList[i].network == "all") bnetwork = true;
            if(QRegExp(QString("\\<"+preferences.m_friendsList[i].network+"\\>").toLower().replace("*","\\w*")).exactMatch(network.toLower())) bnetwork = true;
            break;
        }
    }
    return bnick && bchannel && bnetwork;
}

/*return unique filename
usefull mainly for autoDccFile */
QString dxirc::getUniqueName(const QString &path, const QString &name, const QString &extension)
{
    if(extension.isEmpty())
    {
        if(!QFile::exists(path+"/"+name)) return path+"/"+name;
        else return path+"/"+name+"-"+QDateTime::currentDateTime().toString("yyyyMMddhhmmss");
    }
    else
    {
        if(!QFile::exists(path+"/"+name+"."+extension)) return path+"/"+name+"."+extension;
        else return path+"/"+name+"-"+QDateTime::currentDateTime().toString("yyyyMMddhhmmss")+"."+extension;
    }
}

bool dxirc::isLastTab(IrcEngine* server)
{
    int numTabs = 0;
    IrcTabItem *item = NULL;
    for(int i=0; i<m_tabbook->count(); i++)
    {
        item = qobject_cast<IrcTabItem*>(m_tabbook->widget(i));
        if(item && item->isSameServer(server)) numTabs++;
    }
    if(numTabs > 1) return false;
    else return true;
}

bool dxirc::hasTetrisTab()
{
    if(m_tabbook->count())
    {
        for(int i = 0; i < m_tabbook->count(); i++)
        {
            if(!QString::compare(m_tabbook->widget(i)->metaObject()->className(),"TetrisTabItem")) return true;
        }
    }
    return false;
}

bool dxirc::tabExist(IrcEngine *server, QString name)
{
    IrcTabItem *item = NULL;
    for(int i = 0; i < m_tabbook->count(); i++)
    {
        item = qobject_cast<IrcTabItem*>(m_tabbook->widget(i));
        if(item && server && item->isSameServer(server) && QString::compare(m_tabbook->tabText(i), name) == 0) return true;
    }
    return false;
}

int dxirc::getTabIndex(IrcEngine* server, QString name)
{
    IrcTabItem *item = NULL;
    for(int i = 0; i < m_tabbook->count(); i++)
    {
        item = qobject_cast<IrcTabItem*>(m_tabbook->widget(i));
        if(item && server && item->isSameServer(server) && QString::compare(m_tabbook->tabText(i), name) == 0) return i;
    }
    return -1;
}

//usefull only for tetristab, othertab
bool dxirc::tabExist(QString name)
{
    IrcTabItem *item = NULL;
    for(int i = 0; i < m_tabbook->count(); i++)
    {
        if(!QString::compare(m_tabbook->widget(i)->metaObject()->className(),"TetrisTabItem")
                && !QString::compare(m_tabbook->tabText(i), name))
        {
            return true;
        }
        item = qobject_cast<IrcTabItem*>(m_tabbook->widget(i));
        if(item && item->getType() == OTHER && QString::compare(m_tabbook->tabText(i), name) == 0) return true;
    }
    return false;
}

int dxirc::getCurrentTabId()
{
    int index = m_tabbook->currentIndex();
    if(!m_tabbook->widget(index)) return -1;
    if(m_ft == m_tabbook->widget(index)) return 0;
    if(!QString::compare(m_tabbook->widget(index)->metaObject()->className(),"TetrisTabItem"))
        return static_cast<TetrisTabItem*>(m_tabbook->widget(index))->getID();
    dxTabItem *item = qobject_cast<dxTabItem*>(m_tabbook->widget(index));
    if(item)
        return item->getID();
    return -1;
}

void dxirc::setCurrentTabById(int id)
{
    dxTabItem *item = NULL;
    for(int i = 0; i < m_tabbook->count(); i++)
    {
        if(id ==0 && m_ft==m_tabbook->widget(i))
        {
            m_tabbook->setCurrentIndex(i);
            return;
        }
        item = qobject_cast<dxTabItem*>(m_tabbook->widget(i));
        if(item && item->getID() == id)
        {
            m_tabbook->setCurrentIndex(i);
            return;
        }
        if(!QString::compare(m_tabbook->widget(i)->metaObject()->className(),"TetrisTabItem"))
        {
            TetrisTabItem *tetris = static_cast<TetrisTabItem*>(m_tabbook->widget(i));
            if(tetris->getID() == id)
            {
                m_tabbook->setCurrentIndex(i);
                return;
            }
        }
    }
}

int dxirc::getTabId(IrcEngine *server, QString name)
{
    IrcTabItem *item = NULL;
    for(int i = 0; i < m_tabbook->count(); i++)
    {
        item = qobject_cast<IrcTabItem*>(m_tabbook->widget(i));
        if(item && server && item->isSameServer(server) && QString::compare(m_tabbook->tabText(i), name) == 0)
        {
            return item->getID();
        }
    }
    return -1;
}

//usefull only for tetristab, othertab
int dxirc::getTabId(QString name)
{
    IrcTabItem *item = NULL;
    for(int i = 0; i < m_tabbook->count(); i++)
    {
        item = qobject_cast<IrcTabItem*>(m_tabbook->widget(i));
        if(item && !QString::compare(m_tabbook->tabText(i), name) && item->getType() == OTHER)
        {
            return item->getID();
        }
        if(!QString::compare(m_tabbook->widget(i)->metaObject()->className(),"TetrisTabItem")
                && !QString::compare(m_tabbook->tabText(i), name))
        {
            return static_cast<TetrisTabItem*>(m_tabbook->widget(i))->getID();
        }
    }
    return -1;
}

QWidget* dxirc::getTabItemById(int id)
{
    if(id == 0) return m_ft;
    dxTabItem *item = NULL;
    for(int i = 0; i < m_tabbook->count(); i++)
    {
        item = qobject_cast<dxTabItem*>(m_tabbook->widget(i));
        if(item && item->getID() == id)
        {
            return item;
        }
        if(!QString::compare(m_tabbook->widget(i)->metaObject()->className(),"TetrisTabItem"))
        {
            TetrisTabItem *tetris = static_cast<TetrisTabItem*>(m_tabbook->widget(i));
            if(tetris->getID() == id)
            {
                return item;
            }
        }
    }
    return NULL;
}

void dxirc::slotHighlight(TYPE type)
{
    int index = m_tabbook->indexOf(static_cast<QWidget*>(sender()));
    if(index==-1) return;
    if(!isVisible())
    {
        setTrayIcon(true, false);
        if(preferences.m_sounds && preferences.m_soundMessage) dxutils::playFile(preferences.m_pathMessage);
    }
    if(index==m_tabbook->currentIndex()) return;
    dxTabItem *tab = qobject_cast<dxTabItem*>(sender());
    if(tab)
    {
        if(type == CHANNEL)
        {
            updateStatus(_("New highlighted message on %1").arg(m_tabbook->tabText(index)));
            if(preferences.m_notify && preferences.m_notifyMessage) dxUtils.showNotify(_("New highlighted message on %1").arg(m_tabbook->tabText(index)));
            m_tabbook->setTabIcon(index, QIcon(dxutils::getDataPath()+"/icons/chnewm.png"));
        }
        if(type == QUERY)
        {
            updateStatus(_("New message on chat with %1").arg(m_tabbook->tabText(index)));
            if(preferences.m_notify && preferences.m_notifyMessage) dxUtils.showNotify(_("New message on chat with %1").arg(m_tabbook->tabText(index)));
            m_tabbook->setTabIcon(index, QIcon(dxutils::getDataPath()+"/icons/unewm.png"));
        }
        if(type == DCC)
        {
            updateStatus(_("New message on chat with %1").arg(m_tabbook->tabText(index)));
            if(preferences.m_notify && preferences.m_notifyMessage) dxUtils.showNotify(_("New message on chat with %1").arg(m_tabbook->tabText(index)));
            m_tabbook->setTabIcon(index, QIcon(dxutils::getDataPath()+"/icons/dccnewm.png"));
        }
        m_tabbook->setTabTextColor(index, preferences.m_highlightColor);
    }
}

void dxirc::slotNewmsg(TYPE type)
{
    int index = m_tabbook->indexOf(static_cast<QWidget*>(sender()));
    if(index==-1) return;
    if((type == QUERY || type == DCC) && !isVisible())
    {
        setTrayIcon(true, false);
        if(preferences.m_sounds && preferences.m_soundMessage) dxutils::playFile(preferences.m_pathMessage);
    }
    if(index==m_tabbook->currentIndex()) return;
    dxTabItem *tab = qobject_cast<dxTabItem*>(sender());
    if(tab)
    {
        if(type == CHANNEL)
        {
            m_tabbook->setTabIcon(index, QIcon(dxutils::getDataPath()+"/icons/chnewm.png"));
        }
        if(type == QUERY)
        {
            updateStatus(_("New message on chat with %1").arg(m_tabbook->tabText(index)));
            if(preferences.m_notify && preferences.m_notifyMessage) dxUtils.showNotify(_("New message on chat with %1").arg(m_tabbook->tabText(index)));
            m_tabbook->setTabIcon(index, QIcon(dxutils::getDataPath()+"/icons/unewm.png"));
        }
        if(type == DCC)
        {
            updateStatus(_("New message on chat with %1").arg(m_tabbook->tabText(index)));
            if(preferences.m_notify && preferences.m_notifyMessage) dxUtils.showNotify(_("New message on chat with %1").arg(m_tabbook->tabText(index)));
            m_tabbook->setTabIcon(index, QIcon(dxutils::getDataPath()+"/icons/dccnewm.png"));
        }
        if(m_tabbook->tabTextColor(index)!=preferences.m_highlightColor) m_tabbook->setTabTextColor(index, preferences.m_unreadColor);
    }
}

void dxirc::slotLuaRequest(LuaRequest lua)
{
#ifdef HAVE_LUA
    if(lua.type == LUA_LOAD)
    {
        loadLuaScript(lua.text);
    }
    if(lua.type == LUA_UNLOAD)
    {
        unloadLuaScript(lua.text);
    }
    if(lua.type == LUA_LIST)
    {
        if(!m_scripts.size())
        {
            appendIrcStyledText(_("Scripts aren't loaded"), 4, false);
            return;
        }
        else
        {
            appendIrcStyledText(_("Loaded scrips:"), 7, false);
            for(int i=0; i<m_scripts.size(); i++)
            {
                appendIrcText(_("Name: %1").arg(m_scripts[i].name), false);
                appendIrcText(_("Description: %1").arg(m_scripts[i].description), false);
                appendIrcText(_("Version: %1").arg(m_scripts[i].version), false);
                appendIrcText(_("Path: %1").arg(m_scripts[i].path), false);
                appendIrcText(dxUtils.availableScriptCommands(m_scripts[i].name), false);
                if(i+1<m_scripts.size()) appendIrcText("", false);
            }
        }
        return;
    }
    if(lua.type == LUA_COMMAND)
    {
        dxTabItem *tab = static_cast<dxTabItem*>(sender());
        QString command = lua.text.section(' ',0,0);
        QString text = lua.text.section(' ',1);
        for(int i=0; i<m_scripts.size(); i++)
        {
            if(QString::compare(dxUtils.getScriptName(command), m_scripts[i].name, Qt::CaseInsensitive) == 0)
            {
                lua_getglobal(m_scripts[i].L, dxUtils.getFuncname(command).toUtf8().constData());
                if(lua_isfunction(m_scripts[i].L, -1))
                {
                    lua_pushstring(m_scripts[i].L, text.toUtf8().constData());
                    lua_pushnumber(m_scripts[i].L, tab->getID());
                    if(lua_pcall(m_scripts[i].L, 2, 0, 0)) appendIrcStyledText(_("Error: %1").arg(QString::fromUtf8(lua_tostring(m_scripts[i].L, -1))), 4, false);
                }
                else lua_pop(m_scripts[i].L, 1);
                return;
            }
        }
    }
#else
    appendIrcStyledText(_("dxirc is compiled without support for Lua scripting"), 4, false);
#endif //HAVE_LUA
}

void dxirc::slotTabChanged(int index)
{
    if(index < 0 || index >= m_tabbook->count()) return;
    dxutils::debugLine(m_tabbook->widget(index)->metaObject()->className());
    dxTabItem *tab = qobject_cast<dxTabItem*>(m_tabbook->widget(index));
    if(tab)
    {
        if(tab->getType() == SERVER)
            updateStatus(tab->getRealServerName()+"-"+tab->getNickName());
        else if(tab->getType() == CHANNEL)
        {
            updateStatus((m_tabbook->tabText(index)[0].toLatin1()=='&'?m_tabbook->tabText(index).mid(1):m_tabbook->tabText(index))+"-"+tab->getRealServerName()+"-"+tab->getNickName());
            m_tabbook->setTabIcon(index, QIcon(dxutils::getIconForType(CHANNEL)));
        }
        else if(tab->getType() == QUERY)
        {
            updateStatus((m_tabbook->tabText(index)[0].toLatin1()=='&'?m_tabbook->tabText(index).mid(1):m_tabbook->tabText(index))+"-"+tab->getRealServerName()+"-"+tab->getNickName());
            m_tabbook->setTabIcon(index, QIcon(dxutils::getIconForType(QUERY)));
        }
        else if(tab->getType() == DCC)
        {
            updateStatus((m_tabbook->tabText(index)[0].toLatin1()=='&'?m_tabbook->tabText(index).mid(1):m_tabbook->tabText(index))+"-"+tab->getServerName()+"-"+tab->getNickName());
            m_tabbook->setTabIcon(index, QIcon(dxutils::getIconForType(DCC)));
        }
        else
            updateStatus(m_tabbook->tabText(index));
        tab->setCommandFocus();
        tab->makeLastRowVisible();
    }
    if(m_ft == m_tabbook->widget(index) || !QString::compare(m_tabbook->widget(index)->metaObject()->className(),"TetrisTabItem"))
    {
        updateStatus("");
    }
    m_tabbook->setTabTextColor(index, palette().windowText().color());
    if(isVisible()) setTrayIcon();
}

void dxirc::slotCloseFTTab()
{
    m_tabbook->removeTab(m_tabbook->currentIndex());
    updateMenus();
}

void dxirc::slotCancelFT(int index)
{
    if(index >= m_dccfilesList.size())
        return;
    for(int i=0; i<m_dccengines.size(); i++)
    {
        if(m_dccengines[i]->hasDccFile(m_dccfilesList[index]))
        {
            m_dccengines[i]->disconnect();
            return;
        }
    }
    //dccfile hasn't server
    m_dccfilesList[index].canceled = true;
    m_dccfilesList[index].token = -1;
}

void dxirc::slotAddIgnoreUser(QString text)
{
    IrcTabItem *tab = qobject_cast<IrcTabItem*>(sender());
    QString user = text.section(' ',0,0);
    QString channel = text.section(' ',1,1);
    QString network = dxutils::getParam(text, 3, true);
    if(preferences.m_ignoreUsersList.size())
    {
        bool updated = false;
        for(int i=0; i<preferences.m_ignoreUsersList.size(); i++)
        {
            if(QString::compare(user, preferences.m_ignoreUsersList[i].nick, Qt::CaseSensitive)==0)
            {
                updated = true;
                channel.isEmpty() ? preferences.m_ignoreUsersList[i].channel = "all" : preferences.m_ignoreUsersList[i].channel = channel;
                network.isEmpty() ? preferences.m_ignoreUsersList[i].network = "all" : preferences.m_ignoreUsersList[i].network = network;
                tab->appendStyledText(_("'%1' was updated in ignored users").arg(user), 3, false, false, false);
                break;
            }
        }
        if(!updated)
        {
            UserInfo iuser;
            iuser.nick = user;
            channel.isEmpty() ? iuser.channel = "all" : iuser.channel = channel;
            network.isEmpty() ? iuser.network = "all" : iuser.network = network;
            preferences.m_ignoreUsersList.append(iuser);
            tab->appendStyledText(_("'%1' was added to ignored users").arg(user), 3, false, false, false);
        }
    }
    else
    {
        UserInfo iuser;
        iuser.nick = user;
        channel.isEmpty() ? iuser.channel = "all" : iuser.channel = channel;
        network.isEmpty() ? iuser.network = "all" : iuser.network = network;
        preferences.m_ignoreUsersList.append(iuser);
        tab->appendStyledText(_("'%1' was added to ignored users").arg(user), 3, false, false, false);
    }
}

void dxirc::slotRemoveIgnoreUser(QString text)
{
    IrcTabItem *tab = qobject_cast<IrcTabItem*>(sender());
    bool updated = false;
    if(preferences.m_ignoreUsersList.size())
    {
        for(int i=preferences.m_ignoreUsersList.size()-1; i>-1; i--)
        {
            if(QString::compare(text, preferences.m_ignoreUsersList[i].nick, Qt::CaseSensitive)==0)
            {
                updated = true;
                preferences.m_ignoreUsersList.removeAt(i);
                tab->appendStyledText(_("'%1' was removed from ignored users").arg(text), 3, false, false, false);
                break;
            }
        }
        if(!updated)
        {
            tab->appendStyledText(_("'%1' wasn't removed from ignored users").arg(text), 3, false, false, false);
        }
    }
}

void dxirc::slotAddIgnoreCommand(QString command)
{
    command = command.toLower();
    IrcTabItem *tab = static_cast<IrcTabItem*>(sender());;
    QString available[11] = { "away", "ban", "ctcp", "invite", "join", "me", "nick", "notice", "mode", "part", "quit"};
    bool canAdd = false;
    for(int i=0; i<11; i++)
    {
        if(QString::compare(command,available[i])==0)
        {
            canAdd = true;
            break;
        }
    }
    if(!canAdd)
    {
        tab->appendStyledText(_("'%1' can't be added to ignored commands").arg(command), 4, false, false, false);
        return;
    }
    else
    {
        foreach(QString item, preferences.m_ignoreCommandsList)
        {
            if(item.compare(command)==0)
            {
                tab->appendStyledText(_("'%1' is already added in ignored commands").arg(command), 4, false, false, false);
                return;
            }
        }
        tab->appendStyledText(_("'%1' was added to ignored commands").arg(command), 3, false, false, false);
        preferences.m_ignoreCommandsList.append(command);
    }
}

void dxirc::slotRemoveIgnoreCommand(QString command)
{
    command = command.toLower();
    IrcTabItem *tab = static_cast<IrcTabItem*>(sender());
    QStringList tempList;
    bool inCommands = false;
    foreach(QString item, preferences.m_ignoreCommandsList)
    {
        if(item.compare(command)==0)
        {
            tab->appendStyledText(_("'%1' was removed from ignored commands").arg(command), 3, false, false, false);
            inCommands = true;
        }
        else
            tempList.append(item);
    }
    if(!inCommands) tab->appendStyledText(_("'%1' isn't in ignored commands").arg(command), 4, false, false, false);
    preferences.m_ignoreCommandsList = tempList;
}

void dxirc::appendIrcText(QString text, bool logLine)
{
    if(m_tabbook->count())
    {
        int index = m_tabbook->currentIndex();
        if(!m_tabbook->widget(index)) return;
        if(!QString::compare(m_tabbook->widget(index)->metaObject()->className(),"TetrisTabItem")
            || !QString::compare(m_tabbook->widget(index)->metaObject()->className(),"FtTabItem"))
        {
            for(int i=0; i<m_tabbook->count(); i++)
            {
                if(QString::compare(m_tabbook->widget(i)->metaObject()->className(),"TetrisTabItem")
                    && QString::compare(m_tabbook->widget(i)->metaObject()->className(),"FtTabItem"))
                {
                    index = i;
                    break;
                }
            }
        }
        dxTabItem *currenttab = qobject_cast<dxTabItem*>(m_tabbook->widget(index));
        if(currenttab == 0) return;
        currenttab->appendText(text, true, logLine);
        currenttab->makeLastRowVisible();
    }
}

void dxirc::appendIrcStyledText(QString text, int style, bool logLine)
{
    if(m_tabbook->count())
    {
        int index = m_tabbook->currentIndex();
        if(!m_tabbook->widget(index)) return;
        if(!QString::compare(m_tabbook->widget(index)->metaObject()->className(),"TetrisTabItem")
            || !QString::compare(m_tabbook->widget(index)->metaObject()->className(),"FtTabItem"))
        {
            for(int i=0; i<m_tabbook->count(); i++)
            {
                if(QString::compare(m_tabbook->widget(i)->metaObject()->className(),"TetrisTabItem")
                    && QString::compare(m_tabbook->widget(i)->metaObject()->className(),"FtTabItem"))
                {
                    index = i;
                    break;
                }
            }
        }
        dxTabItem *currenttab = qobject_cast<dxTabItem*>(m_tabbook->widget(index));
        if(currenttab == 0) return;
        currenttab->appendStyledText(text, style, true, false, logLine);
        currenttab->makeLastRowVisible();
    }
}

bool dxirc::isValidTabId(int id)
{
    if(id<0) return false;
    if(id == 0) return true; //FtTabItem
    dxTabItem *item = NULL;
    for(int i = 0; i < m_tabbook->count(); i++)
    {
        if(!QString::compare(m_tabbook->widget(i)->metaObject()->className(),"TetrisTabItem"))
        {
            if(static_cast<TetrisTabItem*>(m_tabbook->widget(i))->getID()==id) return true;
        }
        item = qobject_cast<dxTabItem*>(m_tabbook->widget(i));
        if(item && item->getID()==id) return true;
    }
    return false;
}

bool dxirc::isIddxTabItem(int id)
{
    if(id<0) return false;
    for(int i = 0; i < m_tabbook->count(); i++)
    {
        if(QString::compare(m_tabbook->widget(i)->metaObject()->className(),"TetrisTabItem")
            && QString::compare(m_tabbook->widget(i)->metaObject()->className(),"FtTabItem"))
        {
            if(static_cast<dxTabItem*>(m_tabbook->widget(i))->getID()==id) return true;
        }
    }
    return false;
}

int dxirc::loadLuaScript(QString path, bool showMessage)
{
#ifdef HAVE_LUA
    if(m_scripts.size())
    {
       for(int i=0; i<m_scripts.size(); i++)
        {
            if(QString::compare(path, m_scripts[i].path, Qt::CaseInsensitive)==0)
            {
                appendIrcStyledText(_("Script %1 is already loaded").arg(path), 4, false);
                return 0;
            }
        }
    }
    if(hasLuaAll(path))
    {
        if(showMessage)
        {
            if(QMessageBox::question(this, _("Question"), _("Script %1 contains dxirc.AddAll\nThis can BREAK dxirc funcionality.\nLoad it anyway?").arg(path), QMessageBox::Yes|QMessageBox::No) == QMessageBox::No) return 0;
        }
        else appendIrcStyledText(_("Script %1 contains dxirc.AddAll. This can BREAK dxirc funcionality.").arg(path), 4, false);
    }
    lua_State *L = luaL_newstate();
    if(L == NULL)
    {
        appendIrcStyledText(_("Unable to initialize Lua."), 4, false);
        return 0;
    }
    luaL_openlibs(L);
#if LUA_VERSION_NUM > 501
    lua_newtable(L);
    lua_pushvalue(L, -1);
    lua_setglobal(L, "dxirc");
    luaL_setfuncs(L, dxircFunctions, 0);
#else
    luaL_register(L, "dxirc", dxircFunctions);
#endif
    if(luaL_dofile(L, path.toUtf8().constData()))
    {
        appendIrcStyledText(_("Unable to load/run the file %1").arg(QString::fromUtf8(lua_tostring(L, -1))), 4, false);
        return 0;
    }
    lua_getglobal(L, "dxirc_Register");
    if(lua_pcall(L, 0, 3, 0))
    {
        appendIrcStyledText(_("Lua plugin: error registering script %1").arg(QString::fromUtf8(lua_tostring(L, -1))), 4, false);
        return 0;
    }
    QString name = QString::fromUtf8(lua_tostring(L, -3));
    QString version = QString::fromUtf8(lua_tostring(L, -2));
    QString description = QString::fromUtf8(lua_tostring(L, -1));
    lua_pop(L, 4);
    if(m_scripts.size())
    {
       for(int i=0; i<m_scripts.size(); i++)
        {
            if(QString::compare(name, m_scripts[i].name, Qt::CaseInsensitive)==0)
            {
                appendIrcStyledText(_("Script with name %1 is already loaded").arg(name.toLower()), 4, false);
                return 0;
            }
        }
    }
    LuaScript script;
    script.L = L;
    script.name = name;
    script.version = version;
    script.description = description;
    script.path = path;
    m_scripts.append(script);
    appendIrcStyledText(_("Script %1 was loaded").arg(path), 3, false);
    appendIrcStyledText(QString("%1: %2").arg(script.name).arg(script.description), 3, false);
    lua_getglobal(L, "dxirc_Init");
    if (lua_type(L, -1) != LUA_TFUNCTION) lua_pop(L, 1);
    else
    {
        if (lua_pcall(L, 0, 0, 0))
        {
            appendIrcStyledText(_("Lua plugin: error calling dxirc_Init() %1").arg(QString::fromUtf8(lua_tostring(L, -1))), 4, false);
            lua_pop(L, 1);
        }
    }
    return 1;
#else
    return 0;
#endif //HAVE_LUA
}

int dxirc::unloadLuaScript(QString name)
{
#ifdef HAVE_LUA
    bool success = false;
    if(!m_scripts.size())
    {
        appendIrcStyledText(_("Script %1 isn't loaded").arg(name), 4, false);
        return 0;
    }
    else
    {
        for(int i=m_scripts.size()-1; i>-1; i--)
        {
            if(QString::compare(name, m_scripts[i].name, Qt::CaseInsensitive)==0)
            {
                dxUtils.removeScriptCommands(m_scripts[i].name);
                lua_close(m_scripts[i].L);
                m_scripts.removeAt(i);
                success = true;
            }
        }
    }
    for(int i=m_scriptEvents.size()-1; i>-1; i--)
    {
        if(QString::compare(name, m_scriptEvents[i].script, Qt::CaseInsensitive)==0)
        {
            m_scriptEvents.removeAt(i);
            success = true;
        }
    }
    if(success) appendIrcStyledText(_("Script %1 was unloaded").arg(name), 4, false);
    else appendIrcStyledText(_("Script %1 isn't loaded").arg(name), 4, false);
    return 1;
#else
    return 0;
#endif //HAVE_LUA
}

bool dxirc::hasLuaAll(const QString &file)
{
    std::ifstream fin(file.toUtf8().constData());
    std::string line;
    while(fin.good())
    {
        std::getline(fin, line);
        if(line.find("dxirc.AddEvent") != std::string::npos)
        {
            QString qline = dxutils::fromStdString(line).toLower();
            if(qline.contains("\"all\""))
                return true;
        }
    }
    return false;
}

//check for mymsg in loaded script
bool dxirc::hasOwnMsg()
{
    if(!m_scripts.size() || !m_scriptEvents.size())
    {
        return false;
    }
    for(int i=0; i<m_scriptEvents.size(); i++)
    {
        if(QString::compare("ownmsg", m_scriptEvents[i].name, Qt::CaseInsensitive) == 0)
        {
            return true;
        }
    }
    return false;
}

//check for all in loaded script
bool dxirc::hasAllCommand()
{
    if(!m_scripts.size() || !m_scriptEvents.size())
    {
        return false;
    }
    for(int i=0; i<m_scriptEvents.size(); i++)
    {
        if(QString::compare("all", m_scriptEvents[i].name, Qt::CaseInsensitive) == 0)
        {
            return true;
        }
    }
    return false;
}

void dxirc::autoloadScripts()
{
#ifdef HAVE_LUA
    if(preferences.m_autoload && QFile::exists(preferences.m_autoloadPath))
    {
        QDir dir(preferences.m_autoloadPath);
        dir.setFilter(QDir::Files | QDir::Readable);
        dir.setSorting(QDir::Name);
        QStringList filters;
        filters << "*.lua";
        dir.setNameFilters(filters);
        QFileInfoList list = dir.entryInfoList();
        for(int i = 0; i < list.size(); ++i)
        {
            loadLuaScript(list.at(i).absoluteFilePath(), false);
        }
    }
#endif //HAVE_LUA
}

//check if exist .part file in dccpath
bool dxirc::isForResume(const QString& name)
{
    return QFile::exists(preferences.m_dccPath+"/"+name+".part");
}

//load Qt translation
void loadTranslation()
{
    QString langPath = "";
    QStringList paths;
    QString appdir = QCoreApplication::applicationDirPath();
    paths.append(appdir);
    paths.append(appdir + "/../share/" + QCoreApplication::applicationName().toLower());
    paths.append(appdir + "/../Resources");
    foreach(const QString& path, paths)
    {
        if(QFile::exists(path + "/translations/"))
        {
            langPath = path + "/translations/";
            break;
        }
    }
    QString lang = QLocale::system().name();
    static QTranslator qt_translator;
    if(!langPath.isEmpty())
    {
        if(!qt_translator.load("qt_"+lang, langPath))
        {
            qt_translator.load("qt_"+lang.left(2), langPath);
        }
    }
    else
    {
        if(qt_translator.load("qt_"+lang, QLibraryInfo::location(QLibraryInfo::TranslationsPath)))
        {
            qt_translator.load("qt_"+lang.left(2), QLibraryInfo::location(QLibraryInfo::TranslationsPath));
        }
    }
    QCoreApplication::installTranslator(&qt_translator);
}

const QString USAGE_MSG = _("\
\nUsage: dxirc [options] \n\
\n\
    [options] can be any of the following:\n\
\n\
        -h, --help         Print (this) help screen and exit.\n\
        -v, --version      Print version information and exit.\n\
        -l [FILE]          Load configuration from FILE.\n\
\n");

int main(int argc, char *argv[])
{
#ifdef Q_OS_WIN
    QCoreApplication::addLibraryPath(".");
#endif
    argn = argc;
    args = argv;
    QApplication a(argc, argv);
    QApplication::setOrganizationName("dxSolutions");
    QApplication::setApplicationName(PACKAGE);
    QApplication::setApplicationVersion(VERSION);
#ifdef Q_OS_DARWIN
    QCoreApplication::setAttribute(Qt::AA_DontShowIconsInMenus, true);
#endif
#ifdef ENABLE_NLS
    bindtextdomain("dxirc", dxutils::getLocalePath().toUtf8().constData());
    bind_textdomain_codeset("dxirc", "UTF-8");
    textdomain("dxirc");
#endif //ENABLE_NLS
    for(int i=0; i<argc; ++i)
    {
        if(QString::compare(argv[i],"-v")==0 || QString::compare(argv[i],"--version")==0)
        {
            fprintf(stdout, "%s %s\n", PACKAGE, VERSION);
            exit(0);
        }
        if(QString::compare(argv[i],"-h")==0 || QString::compare(argv[i],"--help")==0)
        {
            fprintf(stdout, "%s\n", USAGE_MSG.toLocal8Bit().constData());
            exit(0);
        }
        if(QString::compare(argv[i],"-l")==0)
        {
            preferences.setIniFile(argv[i+1]);
        }
    }
    loadTranslation(); //load qt library translation
    preferences.load();
    if(preferences.m_oneInstance && isInstanceRunningAlready(argc, argv))
        return 0;
    dxUtils.loadAvailableSpellLangs();
    dxirc w;
    w.show();
    dxUtils.loadAlias();
    return a.exec();
}
