/*
 *      preferences.cpp
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "preferences.h"
#include <QSettings>
#include <QApplication>
#include <QDesktopServices>
#include <QProcessEnvironment>
#include <QDir>
#include "dxutils.h"
#include "i18n.h"

Preferences::Preferences()
{
    //General settings tab in Configdialog
    m_oneInstance = false;
    m_disableSortTabs = false;
    m_reconnect = false;
    m_numberAttempt = 1;
    m_delayAttempt = 20;
    m_maxAway = 200;
    m_closeToTray = true;
    m_useSpell = false;
    m_ownServerWindow = true;
    m_logging = false;
#ifdef Q_OS_WIN
    m_logPath = QProcessEnvironment::systemEnvironment().value("APPDATA")+"/dxirc/logs";
#else
#if(QT_VERSION > 0x050000)
    m_logPath = QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0)+"/.dxirc/logs";
#else
    m_logPath = QDesktopServices::storageLocation(QDesktopServices::HomeLocation)+"/.dxirc/logs";
#endif //QT_VERSION
#endif
    m_autoload = false;
#if(QT_VERSION > 0x050000)
    m_autoloadPath = QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0);
#else
    m_autoloadPath = QDesktopServices::storageLocation(QDesktopServices::HomeLocation);
#endif

    //Look tab in ConfigDialog
    m_unreadColor = QColor("blue");
    m_highlightColor = QColor("red");
    m_tabPosition = 0;

    //Irc Text settings pane in Configdialog
    m_colors.text = QColor(255,255,255);
    m_colors.back = QColor(0,0,0);
    m_colors.user = QColor(43,57,35);
    m_colors.action = QColor(255,165,0);
    m_colors.notice = QColor(0,163,215);
    m_colors.error = QColor(255,0,0);
    m_colors.hilight = QColor(11,154,11);
    m_colors.link = QColor(142,196,210);
    m_colors.mymsg = QColor(130,150,253);
    m_colors.log = QColor(43,57,35);
    m_nickCompletionChar = ":";
    m_sameCmd = false;
    m_sameList = false;
    m_coloredNick = true;
    m_stripColors = true;
    m_loadLog = false;
    m_loadLines = 25;

    //NickList settings pane in Configdialog
    m_themePath = dxutils::getDataPath()+"/icons/default";
    m_themesList = m_themePath+";";
    m_sortUserMode = false;

    //DCC settings pane in Configdialog
#if(QT_VERSION > 0x050000)
    m_dccPath = QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0);
#else
    m_dccPath = QDesktopServices::storageLocation(QDesktopServices::HomeLocation);
#endif
    m_dccIP = "";
    m_dccPortD = 0;
    m_dccPortH = 0;
    m_dccTimeout = 66;
    m_autoDccChat = false;
    m_autoDccFile = false;

    //Notification settings pane in Configdialog
    m_sounds = false;
    m_soundConnect = false;
    m_pathConnect = dxutils::getDataPath()+"/sounds/connected.wav";
    m_soundDisconnect = false;
    m_pathDisconnect = dxutils::getDataPath()+"/sounds/disconnected.wav";
    m_soundMessage = false;
    m_pathMessage = dxutils::getDataPath()+"/sounds/message.wav";
    m_notify = false;
    m_notifyPosition = 3;
    m_notifyConnect = false;
    m_notifyDisconnect = false;
    m_notifyMessage = false;
    m_notifyTray = false;

    //Smileys settings pane in Configdialog
    m_useSmileys = false;

    //Other settings value
    m_usersShown = true;
    m_statusShown = true;
    m_x = 50;
    m_y = 50;
    m_w = 888;
    m_h = 666;
    m_maximized = false;
    m_showSpellCombo = true;

    m_iniFile = "";

    fillDefaultKeybindings();
}

Preferences &Preferences::instance()
{
    static Preferences p;
    return p;
}

void Preferences::setIniFile(const QString &file)
{
    m_iniFile = file;
    createIniPath();
}

QString Preferences::getIniFile() const
{
    return m_iniFile;
}

void Preferences::load()
{
    if(m_iniFile.isEmpty()) createIniPath();
    QSettings set(m_iniFile, QSettings::IniFormat);
    //General settings tab in Configdialog
    set.beginGroup("BASIC");
    m_oneInstance = set.value("oneInstance", m_oneInstance).toBool();
    m_disableSortTabs = set.value("disableSortTabs", m_disableSortTabs).toBool();
    m_reconnect = set.value("reconnect", m_reconnect).toBool();
    m_numberAttempt = set.value("numberAttempt", m_numberAttempt).toInt();
    m_delayAttempt = set.value("delayAttempt", m_delayAttempt).toInt();
    m_maxAway = set.value("maxAway", m_maxAway).toInt();
    m_closeToTray = set.value("closeToTray", m_closeToTray).toBool();
    m_useSpell = set.value("useSpell", m_useSpell).toBool();
    m_ownServerWindow = set.value("ownServerWindow", m_ownServerWindow).toBool();
    m_logging = set.value("logging", m_logging).toBool();
    m_logPath = set.value("logPath", m_logPath).toString();
    m_autoload = set.value("autoload", m_autoload).toBool();
    m_autoloadPath = set.value("autoloadPath", m_autoloadPath).toString();
    set.endGroup();
    //Look settings pane in Configdialog
    set.beginGroup("LOOK");
    m_unreadColor = set.value("unreadColor", m_unreadColor).value<QColor>();
    m_highlightColor = set.value("highlightColor", m_highlightColor).value<QColor>();
    m_tabPosition = set.value("tabPosition", m_tabPosition).toInt();
    set.endGroup();
    //Irc Text settings pane in Configdialog
    set.beginGroup("IRCTEXT");
    m_colors.action = set.value("actionColor", m_colors.action).value<QColor>();
    m_colors.back = set.value("backColor", m_colors.back).value<QColor>();
    m_colors.error = set.value("errorColor", m_colors.error).value<QColor>();
    m_colors.hilight = set.value("hilightColor", m_colors.hilight).value<QColor>();
    m_colors.link = set.value("linkColor", m_colors.link).value<QColor>();
    m_colors.log = set.value("logColor", m_colors.log).value<QColor>();
    m_colors.mymsg = set.value("mymsgColor", m_colors.mymsg).value<QColor>();
    m_colors.notice = set.value("noticeColor", m_colors.notice).value<QColor>();
    m_colors.text = set.value("textColor", m_colors.text).value<QColor>();
    m_colors.user = set.value("userColor", m_colors.user).value<QColor>();
    m_ircFont.fromString(set.value("ircFont", "Sans Serif,9,-1,5,50,0,0,0,0,0").toString());
    m_nickCompletionChar = set.value("nickCompletionChar", m_nickCompletionChar).toString();
    m_sameCmd = set.value("sameCmd", m_sameCmd).toBool();
    m_sameList = set.value("sameList", m_sameList).toBool();
    m_coloredNick = set.value("coloredNick", m_coloredNick).toBool();
    m_stripColors = set.value("stripColors", m_stripColors).toBool();
    m_loadLog = set.value("loadLog", m_loadLog).toBool();
    m_loadLines = set.value("loadLines", m_loadLines).toInt();
    set.endGroup();
    //NickList settings pane in Configdialog
    set.beginGroup("NICKLIST");
    m_themePath = dxutils::checkThemePath(set.value("themePath", m_themePath).toString());
    m_themesList = dxutils::checkThemesList(set.value("themesList", m_themesList).toString());
    m_sortUserMode = set.value("sortUserMode", m_sortUserMode).toBool();
    set.endGroup();
    //Ignore settings pane in Configdialog
    set.beginGroup("IGNORE");
    m_ignoreCommandsList = set.value("commandsList", "").toString().split(';',QString::SkipEmptyParts);
    int usersNum = set.value("usersNumber", 0).toInt();
    m_ignoreUsersList.clear();
    if(usersNum)
    {
        for(int i=0; i<usersNum; i++)
        {
            UserInfo user;
            user.nick = set.value(QString("nick%1").arg(i), QString("xxx%1").arg(i)).toString();
            user.channel = set.value(QString("channel%1").arg(i), "all").toString();
            user.network = set.value(QString("network%1").arg(i), "all").toString();
            m_ignoreUsersList.append(user);
        }
    }
    set.endGroup();
    //DCC settings pane in Configdialog
    set.beginGroup("DCC");
    m_dccPath = set.value("dccPath", m_dccPath).toString();
#if(QT_VERSION > 0x050000)
    if(!QDir(m_dccPath).exists()) m_dccPath = QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0);
#else
    if(!QDir(m_dccPath).exists()) m_dccPath = QDesktopServices::storageLocation(QDesktopServices::HomeLocation);
#endif
    m_dccIP = set.value("dccIP", m_dccIP).toString();
    QRegExp rex("\\l");
    if(m_dccIP.count('.')!=3 || rex.exactMatch(m_dccIP)) m_dccIP = "";
    m_dccPortD = set.value("dccPortD", m_dccPortD).toInt();
    m_dccPortH = set.value("dccPortH", m_dccPortH).toInt();
    m_dccTimeout = set.value("dccTimeout", m_dccTimeout).toInt();
    m_autoDccChat = set.value("autoDccChat", m_autoDccChat).toBool();
    m_autoDccFile = set.value("autoDccFile", m_autoDccFile).toBool();
    set.endGroup();
    //Notification settings pane in Configdialog
    set.beginGroup("NOTIFICATION");
    m_friendsList.clear();
    int friendsNum = set.value("friendsNumber", 0).toInt();
    if(friendsNum)
    {
        for(int i=0; i<friendsNum; i++)
        {
            UserInfo user;
            user.nick = set.value(QString("nick%1").arg(i), QString("xxx%1").arg(i)).toString();
            user.channel = set.value(QString("channel%1").arg(i), "all").toString();
            user.network = set.value(QString("network%1").arg(i), "all").toString();
            m_friendsList.append(user);
        }
    }
    m_sounds = set.value("sounds", m_sounds).toBool();
    m_soundConnect = set.value("soundConnect", m_soundConnect).toBool();
    m_pathConnect = set.value("pathConnect", m_pathConnect).toString();
    m_soundDisconnect = set.value("soundDisconnect", m_soundDisconnect).toBool();
    m_pathDisconnect = set.value("pathDisconnect", m_pathDisconnect).toString();
    m_soundMessage = set.value("soundMessage", m_soundMessage).toBool();
    m_pathMessage = set.value("pathMessage", m_pathMessage).toString();
    m_notify = set.value("notify", m_notify).toBool();
    m_notifyPosition = set.value("notifyPosition", m_notifyPosition).toInt();
    m_notifyConnect = set.value("notifyConnect", m_notifyConnect).toBool();
    m_notifyDisconnect = set.value("notifyDisconnect", m_notifyDisconnect).toBool();
    m_notifyMessage = set.value("notifyMessage", m_notifyMessage).toBool();
    m_notifyTray = set.value("notifyTray", m_notifyTray).toBool();
    set.endGroup();
    //Smileys settings tab in Configdialog
    set.beginGroup("SMILEYS");
    m_useSmileys = set.value("useSmileys", m_useSmileys).toBool();
    int smileysNum = set.value("number", 0).toInt();
    m_smileysMap.clear();
    if(smileysNum)
    {
        for(int i=0; i<smileysNum; i++)
        {
            QString key, value;
            key = set.value(QString("smiley%1").arg(i), "").toString();
            value = set.value(QString("path%1").arg(i), "").toString();
            if(!key.isEmpty())
                m_smileysMap.insert(key, value);
        }
    }
    set.endGroup();
    //Other settings value
    set.beginGroup("SERVERS");
    int serversNum = set.value("number", 0).toInt();
    if(serversNum)
    {
        for(int i=0; i<serversNum; i++)
        {
            ServerInfo server;
            server.hostname = set.value(QString("hostname%1").arg(i), QString("localhost%1").arg(i)).toString();
            server.name = set.value(QString("name%1").arg(i), server.hostname).toString();
            server.port = set.value(QString("port%1").arg(i), 6667).toInt();
            server.nick = set.value(QString("nick%1").arg(i), "xxx").toString();
            server.realname = set.value(QString("realname%1").arg(i), "xxx").toString();
            server.passwd = dxutils::decrypt(set.value(QString("hes%1").arg(i), "").toString());
            server.channels = set.value(QString("channels%1").arg(i), "").toString();
            server.commands = set.value(QString("commands%1").arg(i), "").toString();
            server.autoConnect = set.value(QString("autoconnect%1").arg(i), false).toBool();
            server.useSsl = set.value(QString("ssl%1").arg(i), false).toBool();
            m_serverList.append(server);
        }
    }
    set.endGroup();
    set.beginGroup("OTHER");
    m_usersShown = set.value("usersShown", m_usersShown).toBool();
    m_statusShown = set.value("statusShown", m_statusShown).toBool();
    m_x=set.value("x", m_x).toInt();
    m_y=set.value("y", m_y).toInt();
    m_w=set.value("w", m_w).toInt();
    m_h=set.value("h", m_h).toInt();
    m_maximized = set.value("maximized", m_maximized).toBool();
    m_showSpellCombo = set.value("showSpellCombo", m_showSpellCombo).toBool();
    set.endGroup();
    set.beginGroup("LANGS");
    int langsNum = set.value("number", 0).toInt();
    m_langs.clear();
    if(langsNum)
    {
        for(int i=0; i<langsNum; i++)
        {
            QString key, value;
            key = set.value(QString("tab%1").arg(i), "").toString();
            value = set.value(QString("lang%1").arg(i), "").toString();
            if(!key.isEmpty() && !value.isEmpty())
                m_langs.insert(key, value);
        }
    }
    set.endGroup();
    set.beginGroup("KEYBINDINGS");
    if(m_defaultkeybindings.size())
    {
        QMap<QString,QKeySequence>::iterator it;
        for(it=m_defaultkeybindings.begin(); it!=m_defaultkeybindings.end(); ++it)
            m_keybindings.insert(it.key(), set.value(it.key(), it.value()).toString());
    }
    set.endGroup();
}

void Preferences::save()
{
    QSettings set(m_iniFile, QSettings::IniFormat);
    foreach(QString key, set.allKeys())
        set.remove(key);
    //General settings tab in Configdialog
    set.beginGroup("BASIC");
    set.setValue("oneInstance", m_oneInstance);
    set.setValue("disableSortTabs", m_disableSortTabs);
    set.setValue("reconnect", m_reconnect);
    set.setValue("numberAttempt", m_numberAttempt);
    set.setValue("delayAttempt", m_delayAttempt);
    set.setValue("maxAway", m_maxAway);
    set.setValue("closeToTray", m_closeToTray);
    set.setValue("useSpell", m_useSpell);
    set.setValue("ownServerWindow", m_ownServerWindow);
    set.setValue("logging", m_logging);
    set.setValue("logPath", QDir::toNativeSeparators(m_logPath));
    set.setValue("autoload", m_autoload);
    set.setValue("autoloadPath", QDir::toNativeSeparators(m_autoloadPath));
    set.endGroup();
    //Look settings pane in Configdialog
    set.beginGroup("LOOK");
    set.setValue("unreadColor", m_unreadColor);
    set.setValue("highlightColor", m_highlightColor);
    set.setValue("tabPosition", m_tabPosition);
    set.endGroup();
    //Irc Text settings pane in Configdialog
    set.beginGroup("IRCTEXT");
    set.setValue("actionColor", m_colors.action);
    set.setValue("backColor", m_colors.back);
    set.setValue("errorColor", m_colors.error);
    set.setValue("hilightColor", m_colors.hilight);
    set.setValue("linkColor", m_colors.link);
    set.setValue("logColor", m_colors.log);
    set.setValue("mymsgColor", m_colors.mymsg);
    set.setValue("noticeColor", m_colors.notice);
    set.setValue("textColor", m_colors.text);
    set.setValue("userColor", m_colors.user);
    set.setValue("ircFont", m_ircFont.toString());
    set.setValue("nickCompletionChar", m_nickCompletionChar);
    set.setValue("sameCmd", m_sameCmd);
    set.setValue("sameList", m_sameList);
    set.setValue("coloredNick", m_coloredNick);
    set.setValue("stripColors", m_stripColors);
    set.setValue("loadLog", m_loadLog);
    set.setValue("loadLines", m_loadLines);
    set.endGroup();
    //NickList settings pane in Configdialog
    set.beginGroup("NICKLIST");
    set.setValue("themePath", QDir::toNativeSeparators(m_themePath));
    set.setValue("themesList", themeListToSeparators());
    set.setValue("sortUserMode", m_sortUserMode);
    set.endGroup();
    //Ignore settings pane in Configdialog
    set.beginGroup("IGNORE");
    set.setValue("commandsList", m_ignoreCommandsList.join(";"));
    set.setValue("usersNumber", m_ignoreUsersList.size());
    for(int i=0; i<m_ignoreUsersList.size(); i++)
    {
        set.setValue(QString("nick%1").arg(i), m_ignoreUsersList[i].nick);
        set.setValue(QString("channel%1").arg(i), m_ignoreUsersList[i].channel);
        set.setValue(QString("network%1").arg(i), m_ignoreUsersList[i].network);
    }
    set.endGroup();
    //DCC settings pane in Configdialog
    set.beginGroup("DCC");
    set.setValue("dccPath", QDir::toNativeSeparators(m_dccPath));
    set.setValue("dccIP", m_dccIP);
    set.setValue("dccPortD", m_dccPortD);
    set.setValue("dccPortH", m_dccPortH);
    set.setValue("dccTimeout", m_dccTimeout);
    set.setValue("autoDccChat", m_autoDccChat);
    set.setValue("autoDccFile", m_autoDccFile);
    set.endGroup();
    //Notification settings pane in Configdialog
    set.beginGroup("NOTIFICATION");
    set.setValue("friendsNumber", m_friendsList.size());
    for(int i=0; i<m_friendsList.size(); i++)
    {
        set.setValue(QString("nick%1").arg(i), m_friendsList[i].nick);
        set.setValue(QString("channel%1").arg(i), m_friendsList[i].channel);
        set.setValue(QString("network%1").arg(i), m_friendsList[i].network);
    }
    set.setValue("sounds", m_sounds);
    set.setValue("soundConnect", m_soundConnect);
    set.setValue("pathConnect", QDir::toNativeSeparators(m_pathConnect));
    set.setValue("soundDisconnect", m_soundDisconnect);
    set.setValue("pathDisconnect", QDir::toNativeSeparators(m_pathDisconnect));
    set.setValue("soundMessage", m_soundMessage);
    set.setValue("pathMessage", QDir::toNativeSeparators(m_pathMessage));
    set.setValue("notify", m_notify);
    set.setValue("notifyPosition", m_notifyPosition);
    set.setValue("notifyConnect", m_notifyConnect);
    set.setValue("notifyDisconnect", m_notifyDisconnect);
    set.setValue("notifyMessage", m_notifyMessage);
    set.setValue("notifyTray", m_notifyTray);
    set.endGroup();
    //Smileys settings tab in Configdialog
    set.beginGroup("SMILEYS");
    set.setValue("useSmileys", m_useSmileys);
    set.setValue("number", m_smileysMap.size());
    if(m_smileysMap.size())
    {
        QMap<QString,QString>::iterator it;
        int i;
        for(i=0, it=m_smileysMap.begin(); it!=m_smileysMap.end(); it++,i++)
        {
            set.setValue(QString("smiley%1").arg(i), it.key());
            set.setValue(QString("path%1").arg(i), QDir::toNativeSeparators(it.value()));
        }
    }
    set.endGroup();
    //Other settings value
    set.beginGroup("SERVERS");
    set.setValue("number", m_serverList.size());
    for(int i=0; i<m_serverList.size(); i++)
    {
        set.setValue(QString("hostname%1").arg(i), m_serverList[i].hostname);
        set.setValue(QString("name%1").arg(i), m_serverList[i].name);
        set.setValue(QString("port%1").arg(i), m_serverList[i].port);
        set.setValue(QString("nick%1").arg(i), m_serverList[i].nick);
        set.setValue(QString("realname%1").arg(i), m_serverList[i].realname);
        set.setValue(QString("hes%1").arg(i), dxutils::encrypt(m_serverList[i].passwd));
        set.setValue(QString("channels%1").arg(i), m_serverList[i].channels);
        set.setValue(QString("commands%1").arg(i), m_serverList[i].commands);
        set.setValue(QString("autoconnect%1").arg(i), m_serverList[i].autoConnect);
        set.setValue(QString("ssl%1").arg(i), m_serverList[i].useSsl);
    }
    set.endGroup();
    set.beginGroup("OTHER");
    set.setValue("usersShown", m_usersShown);
    set.setValue("statusShown", m_statusShown);
    set.setValue("x", m_x);
    set.setValue("y", m_y);
    set.setValue("w", m_w);
    set.setValue("h", m_h);
    set.setValue("maximized", m_maximized);
    set.setValue("showSpellCombo", m_showSpellCombo);
    set.endGroup();
    set.beginGroup("LANGS");
    set.setValue("number", m_langs.size());
    if(m_langs.size())
    {
        QMap<QString,QString>::iterator it;
        int i;
        for(i=0, it=m_langs.begin(); it!=m_langs.end(); it++,i++)
        {
            set.setValue(QString("tab%1").arg(i), it.key());
            set.setValue(QString("lang%1").arg(i), it.value());
        }
    }
    set.endGroup();
    set.beginGroup("ALIASES");
    QMap<QString,QString> aliases = dxutils::instance().getAliases();
    set.setValue("number", aliases.size());
    if(aliases.size())
    {
        QMap<QString,QString>::iterator it;
        int i;
        for(i=0,it=aliases.begin(); it!=aliases.end(); it++,i++)
        {
            set.setValue(QString("key%1").arg(i), it.key());
            set.setValue(QString("value%1").arg(i), it.value());
        }
    }
    set.endGroup();
    set.beginGroup("KEYBINDINGS");
    if(m_keybindings.size())
    {
        QMap<QString,QKeySequence>::iterator it;
        for(it=m_keybindings.begin(); it!=m_keybindings.end(); ++it)
            set.setValue(it.key(), it.value());
    }
    set.endGroup();
    set.sync();
}

QKeySequence Preferences::getBinding(const QString &key)
{
    if(m_keybindings.find(key)==m_keybindings.end())
        return QKeySequence();
    else
        return m_keybindings.find(key).value();
}

QString Preferences::getBindingName(const QString &key)
{
    if(m_namekeybindings.find(key)==m_namekeybindings.end())
        return "";
    else
        return m_namekeybindings.find(key).value();
}

void Preferences::fillDefaultKeybindings()
{
    m_defaultkeybindings.clear();
    m_namekeybindings.clear();
    m_defaultkeybindings.insert("serverList", QKeySequence(Qt::Key_F2));
    m_namekeybindings.insert("serverList", _("Server list"));
    m_defaultkeybindings.insert("quickConnect", QKeySequence(Qt::Key_F7));
    m_namekeybindings.insert("quickConnect", _("Quick connect"));
    m_defaultkeybindings.insert("disconnect", QKeySequence(Qt::CTRL+Qt::Key_D));
    m_namekeybindings.insert("disconnect", _("Disconnect"));
    m_defaultkeybindings.insert("dccTransfers", QKeySequence(Qt::CTRL+Qt::Key_T));
    m_namekeybindings.insert("dccTransfers", _("DCC transfers"));
    m_defaultkeybindings.insert("scripts", QKeySequence(Qt::CTRL+Qt::Key_S));
    m_namekeybindings.insert("scripts", _("Scripts"));
    m_defaultkeybindings.insert("logViewer", QKeySequence(Qt::CTRL+Qt::Key_G));
    m_namekeybindings.insert("logViewer", _("Log viewer"));
#ifdef Q_OS_WIN32
    m_defaultkeybindings.insert("quit", QKeySequence(Qt::ALT+Qt::Key_F4));
#else
    m_defaultkeybindings.insert("quit", QKeySequence(QKeySequence::Quit));
#endif //WIN32
    m_namekeybindings.insert("quit", _("Quit"));
    m_defaultkeybindings.insert("closeCurrentTab", QKeySequence(Qt::CTRL+Qt::Key_W));
    m_namekeybindings.insert("closeCurrentTab", _("Close current tab"));
    m_defaultkeybindings.insert("clearWindow", QKeySequence(Qt::CTRL+Qt::Key_L));
    m_namekeybindings.insert("clearWindow", _("Clear window"));
    m_defaultkeybindings.insert("clearAllWindows", QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_L));
    m_namekeybindings.insert("clearAllWindows", _("Clear all windows"));
    m_defaultkeybindings.insert("usersList", QKeySequence(Qt::CTRL+Qt::Key_U));
    m_namekeybindings.insert("usersList", _("Users list"));
    m_defaultkeybindings.insert("spellList", QKeySequence(Qt::CTRL+Qt::Key_P));
    m_namekeybindings.insert("spellList", _("Spellchecking language list"));
    m_defaultkeybindings.insert("help", QKeySequence(Qt::Key_F1));
    m_namekeybindings.insert("help", _("Help"));
#ifdef Q_OS_DARWIN
    m_defaultkeybindings.insert("switchTab", QKeySequence(Qt::META+Qt::Key_N).toString(QKeySequence::NativeText));
#else
    m_defaultkeybindings.insert("switchTab", QKeySequence(Qt::CTRL+Qt::Key_Tab));
#endif
    m_namekeybindings.insert("switchTab", _("Switch between tabs"));
    m_defaultkeybindings.insert("unreadTab", QKeySequence(Qt::CTRL+Qt::Key_N));
    m_namekeybindings.insert("unreadTab", _("Switch between unreaded tabs"));
    m_defaultkeybindings.insert("moveTabUp", QKeySequence(Qt::CTRL+Qt::Key_Up));
    m_namekeybindings.insert("moveTabUp", _("Move tab up"));
    m_defaultkeybindings.insert("moveTabDown", QKeySequence(Qt::CTRL+Qt::Key_Down));
    m_namekeybindings.insert("moveTabDown", _("Move tab down"));
    m_defaultkeybindings.insert("forceFocus", QKeySequence(Qt::CTRL+Qt::Key_F));
    m_namekeybindings.insert("forceFocus", _("Force commandline focus in current tab"));
    m_defaultkeybindings.insert("boldControlChar", QKeySequence(Qt::CTRL+Qt::Key_B));
    m_namekeybindings.insert("boldControlChar", _("Insert bold control char"));
    m_defaultkeybindings.insert("colorControlChar", QKeySequence(Qt::CTRL+Qt::Key_K));
    m_namekeybindings.insert("colorControlChar", _("Insert color control char"));
    m_defaultkeybindings.insert("underlineControlChar", QKeySequence(Qt::CTRL+Qt::Key_I));
    m_namekeybindings.insert("underlineControlChar", _("Insert underline control char"));
    m_defaultkeybindings.insert("reverseControlChar", QKeySequence(Qt::CTRL+Qt::Key_R));
    m_namekeybindings.insert("reverseControlChar", _("Insert reverse control char"));
    m_defaultkeybindings.insert("resetControlChar", QKeySequence(Qt::CTRL+Qt::Key_O));
    m_namekeybindings.insert("resetControlChar", _("Insert reset control char"));
    m_defaultkeybindings.insert("firstTab", QKeySequence(Qt::ALT+Qt::Key_1));
    m_namekeybindings.insert("firstTab", _("Go to 1. tab"));
    m_defaultkeybindings.insert("secondTab", QKeySequence(Qt::ALT+Qt::Key_2));
    m_namekeybindings.insert("secondTab", _("Go to 2. tab"));
    m_defaultkeybindings.insert("thirdTab", QKeySequence(Qt::ALT+Qt::Key_3));
    m_namekeybindings.insert("thirdTab", _("Go to 3. tab"));
    m_defaultkeybindings.insert("fourthTab", QKeySequence(Qt::ALT+Qt::Key_4));
    m_namekeybindings.insert("fourthTab", _("Go to 4. tab"));
    m_defaultkeybindings.insert("fifthTab", QKeySequence(Qt::ALT+Qt::Key_5));
    m_namekeybindings.insert("fifthTab", _("Go to 5. tab"));
    m_defaultkeybindings.insert("sixthTab", QKeySequence(Qt::ALT+Qt::Key_6));
    m_namekeybindings.insert("sixthTab", _("Go to 6. tab"));
    m_defaultkeybindings.insert("seventhTab", QKeySequence(Qt::ALT+Qt::Key_7));
    m_namekeybindings.insert("seventhTab", _("Go to 7. tab"));
    m_defaultkeybindings.insert("eighthTab", QKeySequence(Qt::ALT+Qt::Key_8));
    m_namekeybindings.insert("eighthTab", _("Go to 8. tab"));
    m_defaultkeybindings.insert("ninethTab", QKeySequence(Qt::ALT+Qt::Key_9));
    m_namekeybindings.insert("ninethTab", _("Go to 9. tab"));
    m_defaultkeybindings.insert("listCommand", QKeySequence(Qt::ALT+Qt::Key_L));
    m_namekeybindings.insert("listCommand", _("/LIST command"));
}

void Preferences::createIniPath()
{
    if(m_iniFile.isEmpty())
    {
        QString iniPath;
#ifdef Q_OS_WIN
        iniPath = QProcessEnvironment::systemEnvironment().value("APPDATA")+"/dxirc";
#else
#if(QT_VERSION > 0x050000)
        iniPath = QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0)+"/.config/dxirc";
#else
        iniPath = QDesktopServices::storageLocation(QDesktopServices::HomeLocation)+"/.config/dxirc";
#endif //QT_VERSION
#endif
        QDir dir;
        if(!dir.exists(iniPath))
            dir.mkpath(iniPath);
        m_iniFile = iniPath+"/dxirc-qt.ini";
    }
}

QString Preferences::themeListToSeparators()
{
    QString themesList = "";
    QStringList themes = m_themesList.split(';', QString::SkipEmptyParts);
    for(int i=0; i<themes.size(); ++i)
    {
        themesList.append(QDir::toNativeSeparators(themes.at(i)));
        themesList.append(';');
    }
    return themesList;
}
