/*
 *      boatstabitem.h
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */


#ifndef BOATSTABITEM_H
#define BOATSTABITEM_H

#include "engine/ircengine.h"
#include "dxtabitem.h"

static const int bcolumns = 10;
static const int brows = 10;
static const int boats = 10;

enum CELLTYPE {
    CLEAR = 0,
    BOAT,
    FIRE,
    WATER,
    HIT,
    PLACED
};

struct BoatsCell
{
    BoatsCell(int x1 = -1, int y1 = -1, CELLTYPE type1 = CLEAR)
    :   x(x1),
        y(y1),
        type(type1)
    {
    }

    bool operator==(const BoatsCell& cell) const
    {
        return x == cell.x && y == cell.y && type == cell.type;
    }

    int x;
    int y;
    CELLTYPE type;
};

struct Boat
{
    Boat(int x1, int y1, int cells1, bool rotated1)
    :   x(x1),
        y(y1),
        cells(cells1),
        rotated(rotated1)
    {
    }

    Boat()
    : x(-1),
      y(-1),
      cells(0),
      rotated(false)
    {
    }

    bool operator ==(const Boat& boat) const
    {
        return x == boat.x && y == boat.y && cells == boat.cells
                && rotated == boat.rotated;
    }

    bool isValid() const
    {
        return x != -1 && y != -1;
    }

    bool constainsCell(int x1, int y1)
    {
        if(!isValid())
            return false;
        if(rotated)
        {
            return x1 == x && y1 >= y && y1 < y+cells;
        }
        else
        {
            return y1 == y && x1 >= x && x1 < x+cells;
        }
    }

    int x;
    int y;
    int cells;
    bool rotated;
};

class Boats : public QWidget
{
    Q_OBJECT
public:
    Boats(bool first, QWidget *parent = 0);

    void setCellType(int x, int y, bool my, CELLTYPE type);
    CELLTYPE getCellType(int x, int y, bool my);
    bool first() const;
    void setFirst(bool first);
    bool onFire() const;
    void setOnFire(bool onFire);
    bool readyToPlay() const;
    void setReadyToPlay(bool readyToPlay);
    bool placeBoats() const;
    void setPlaceBoats(bool placeBoats);
    bool boatDestroyed(int x, int y);
    void clearCellsAndBoats();
    bool gameOver();
signals:
    void ready();
    void statusText(const QString &text);
    void fire(int x, int y);
protected:
    void mousePressEvent(QMouseEvent* event);
    void mouseMoveEvent(QMouseEvent* event);
    void paintEvent(QPaintEvent*);
    void resizeEvent(QResizeEvent* event);
private:
    int m_cell;
    BoatsCell m_mycells[bcolumns][brows];
    BoatsCell m_hiscells[bcolumns][brows];
    Boat m_boats[boats];
    bool m_first; //I fire first
    bool m_placeBoats;
    bool m_rotated; //Place rotated boat
    int m_previousX; //Placed boat
    int m_previousY; //Placed boat
    int m_cells; //Placed boat
    bool m_onFire;
    bool m_readyToPlay;

    void placeMyBoats();
    QColor getCellColor(CELLTYPE type);
    bool isRightForPlaceBoat(int x, int y);
};

class BoatsTabItem: public dxTabItem
{
    Q_OBJECT
    friend class dxirc;
public:
    BoatsTabItem(const QString &name, bool first, TabWidget *tabbook, int id=0, IrcEngine *engine=0, QWidget *parent = 0);
    virtual ~BoatsTabItem();

    QString getServerName() { return m_engine ? m_engine->getServerName() : ""; }
    int getServerPort() { return m_engine ? m_engine->getServerPort() : 0; }
    QString getNickName() { return m_engine ? m_engine->getNickName() : ""; }
    QString getRealServerName() { return m_engine ? m_engine->getNetworkName() : ""; }
    TYPE getType() { return m_type; }
    IrcEngine* getEngine() { return m_engine; }
    QString getTabName() { return m_name; }
    void leftGame();
    bool isSameServer(IrcEngine *engine);
signals:
    void newname(QString);
private:
    IrcEngine *m_engine;
    TYPE m_type;
    QString m_name;
    QLabel *m_status;
    Boats *m_gamecanvas;
    QPushButton *m_newGame;

    void sendCtcp(const QString &msg);
private slots:
    //IrcEngine slots
    void slotIrcNick(QString nick, QString newnick);
    void slotIrcQuit(QString nick);
    void slotIrc401(QString nick);
    void slotIrcBoats(QString nick, QString position, QString status);
    void slotIrcBoatsReady(QString nick);
    void slotIrcBoatsLeft(QString nick);
    void slotIrcBoatsDestoyed(QString nick);
    void slotIrcBoatsNewGame(QString nick);
    void slotIrcBoatsWin(QString nick);
    //Boats slots
    void slotFire(int x, int y);
    void slotStatusText(QString text);
    void slotReady();
    //Other slots
    void slotNewGame();
};

#endif // BOATSTABITEM_H
