/*
 *      irctabitem.cpp
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "irctabitem.h"
#include "config.h"
#include "i18n.h"
#include "dxutils.h"
#include "preferences.h"
#include "dialogs/userdialog.h"
#include "dialogs/dccsenddialog.h"

static bool caseInsensitiveLessThan(const QString &s1, const QString &s2)
{
    return s1.toLower() < s2.toLower();
}

IrcTabItem::IrcTabItem(const QString &name, IrcEngine *engine, TabWidget *tabbook, TYPE type, int id, QWidget *parent)
    : dxTabItem(tabbook, id, parent), m_engine(engine), m_type(type), m_logstream(NULL), m_model(this), m_sortModel(this)
{
    m_currentPosition = 0;
    m_historyMax = 25;
    m_numberUsers = 0;
    m_maxLen = 460;
    m_iamOp = false;
    m_topic = _("No topic is set");
    m_editableTopic = true;
    m_pipe = NULL;
    m_sendPipe = false;
    m_scriptHasAll = false;
    m_scriptHasOwnMsg = false;
    m_loadLog = false;
    if(name[0] == '&') m_name = name.mid(1);
    else m_name = name;

    if(m_type == CHANNEL && m_engine->getConnected())
    {
        m_engine->sendMode(m_name);
    }

    m_pipeTimer = new QTimer(this);
    m_eggTimer = new QTimer(this);
    m_topicline = new QLineEdit;
    m_topicline->setStyleSheet(QString("QLineEdit { color:%1; background:%2; }").arg(preferences.m_colors.text.name()).arg(preferences.m_colors.back.name()));
    m_topicline->setFont(preferences.m_ircFont);
    m_topicline->setText(m_topic);
    m_topicline->setToolTip(m_topic);
    if(m_type != CHANNEL)
    {
        m_topicline->hide();
    }
    m_splitter = new QSplitter(this);
    m_splitter->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    m_text = new QTextBrowser;
    m_text->setOpenExternalLinks(true);
    m_text->document()->setDefaultStyleSheet(QString("a { color:%1 }").arg(preferences.m_colors.link.name()));
    m_text->setStyleSheet(QString("QTextEdit { color:%1; background:%2; }").arg(preferences.m_colors.text.name()).arg(preferences.m_colors.back.name()));
    m_text->setFont(preferences.m_ircFont);
    m_splitter->addWidget(m_text);
    m_users = new QListView(this);
    m_sortModel.setSourceModel(&m_model);
    m_users->setModel(&m_sortModel);
    m_users->setStyleSheet(QString("QListView { \
                    background-color: %1; \
                    color: %2; \
                    } \
                    ").arg(preferences.m_colors.back.name())
                    .arg(preferences.m_colors.text.name())
                    );
    if(preferences.m_sameList) m_users->setFont(preferences.m_ircFont);
    m_users->setContextMenuPolicy(Qt::CustomContextMenu);
    m_users->setEditTriggers(QAbstractItemView::NoEditTriggers);
    IrcItemDelegate *delegate = new IrcItemDelegate();
    m_users->setItemDelegate(delegate);
    m_sortModel.sort(0);
    m_splitter->addWidget(m_users);
    m_commandline = new LineSpell;
    m_commandline->installEventFilter(this);
    m_commandline->setStyleSheet(QString("QTextEdit { color:%1; background:%2; }").arg(preferences.m_colors.text.name()).arg(preferences.m_colors.back.name()));
    if(preferences.m_sameCmd) m_commandline->setFont(preferences.m_ircFont);
    m_spellLangs = new QComboBox;
    m_spellLangs->setToolTip(_("Spellchecking language list"));
    m_spellLangs->hide();
    if(preferences.m_useSpell && (m_type==CHANNEL || m_type==QUERY) && dxUtils.getAvailableSpellLangsNum())
    {
        QStringList langs = dxUtils.getAvailableSpellLangs();
        QString lang = dxUtils.getChannelLang(m_name);
        for(int i=0; i<langs.size(); ++i)
        {
            m_spellLangs->addItem(langs[i]);
            if(langs[i]==lang) m_spellLangs->setCurrentIndex(i);;
        }
        if(preferences.m_showSpellCombo) m_spellLangs->show();
        m_commandline->setUseSpell(true);
        m_commandline->setLanguage(lang);
        m_commandline->setToolTip(_("Current spellchecking language: %1").arg(lang));
    }
    QHBoxLayout *command = new QHBoxLayout;
    command->setSpacing(0);
    command->addWidget(m_commandline);
    command->addWidget(m_spellLangs);
    QVBoxLayout *mainLayout = new QVBoxLayout(this);
    mainLayout->addWidget(m_topicline);
    mainLayout->addWidget(m_splitter);
    mainLayout->addLayout(command);
    setLayout(mainLayout);
    QList<int> list;
    list << width()-120 << 120;
    m_splitter->setSizes(list);
    if(m_type != CHANNEL) m_users->hide();

    connect(m_commandline, SIGNAL(returnPressed()), this, SLOT(slotProcessCommandline()));
    connect(m_topicline, SIGNAL(returnPressed()), this, SLOT(slotProcessTopicline()));
    connect(m_users, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(slotPopup(QPoint)));
    connect(m_users, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(slotDoubleClickUsers(QModelIndex)));
    connect(m_pipeTimer, SIGNAL(timeout()), this, SLOT(slotPipeTimeout()));
    connect(m_eggTimer, SIGNAL(timeout()), this, SLOT(slotEggTimeout()));
    if(m_engine)
    {
        connect(m_engine, SIGNAL(ircConnect(QString)), this, SLOT(slotIrcConnect(QString)));
        connect(m_engine, SIGNAL(ircDisconnect(QString)), this, SLOT(slotIrcDisconnect(QString)));
        connect(m_engine, SIGNAL(ircServerReply(QString)), this, SLOT(slotIrcServerReply(QString)));
        connect(m_engine, SIGNAL(ircServerError(QString)), this, SLOT(slotIrcServerError(QString)));
        connect(m_engine, SIGNAL(irc372(QString)), this, SLOT(slotIrc372(QString)));
        connect(m_engine, SIGNAL(ircNotice(QString,QString)), this, SLOT(slotIrcNotice(QString,QString)));
        connect(m_engine, SIGNAL(ircChnotice(QString,QString,QString)), this, SLOT(slotIrcChnotice(QString,QString,QString)));
        connect(m_engine, SIGNAL(ircPrivmsg(QString,QString,QString)), this, SLOT(slotIrcPrivmsg(QString,QString,QString)));
        connect(m_engine, SIGNAL(ircAction(QString,QString,QString)), this, SLOT(slotIrcAction(QString,QString,QString)));
        connect(m_engine, SIGNAL(ircError(QString)), this, SLOT(slotIrcError(QString)));
        connect(m_engine, SIGNAL(ircReconnect(QString)), this, SLOT(slotIrcReconnect(QString)));
        connect(m_engine, SIGNAL(ircReconnectAll(QString)), this, SLOT(slotIrcReconnectAll(QString)));
        connect(m_engine, SIGNAL(ircCtcpReply(QString,QString)), this, SLOT(slotIrcCtcpReply(QString,QString)));
        connect(m_engine, SIGNAL(ircCtcpRequest(QString,QString)), this, SLOT(slotIrcCtcpRequest(QString,QString)));
        connect(m_engine, SIGNAL(ircNickInfo(NickInfo)), this, SLOT(slotIrcNickInfo(NickInfo)));
        connect(m_engine, SIGNAL(ircJoin(QString,QString)), this, SLOT(slotIrcJoin(QString,QString)));
        connect(m_engine, SIGNAL(irc353(QString,QString,bool)), this, SLOT(slotIrc353(QString,QString,bool)));
        connect(m_engine, SIGNAL(irc366(QString,QString,bool)), this, SLOT(slotIrc366(QString,QString,bool)));
        connect(m_engine, SIGNAL(ircPart(QString,QString,QString)), this, SLOT(slotIrcPart(QString,QString,QString)));
        connect(m_engine, SIGNAL(ircQuit(QString,QString)), this, SLOT(slotIrcQuit(QString,QString)));
        connect(m_engine, SIGNAL(ircTopic(QString,QString,QString)), this, SLOT(slotIrcTopic(QString,QString,QString)));
        connect(m_engine, SIGNAL(ircNick(QString,QString)), this, SLOT(slotIrcNick(QString,QString)));
        connect(m_engine, SIGNAL(ircMode(QString,QString)), this, SLOT(slotIrcMode(QString,QString)));
        connect(m_engine, SIGNAL(ircUmode(QString,QString,QString,QString)), this, SLOT(slotIrcUmode(QString,QString,QString,QString)));
        connect(m_engine, SIGNAL(ircChmode(QString,QString)), this, SLOT(slotIrcChmode(QString,QString)));
        connect(m_engine, SIGNAL(irc301(QString,QString)), this, SLOT(slotIrc301(QString,QString)));
        connect(m_engine, SIGNAL(irc305(QString)), this, SLOT(slotIrc305(QString)));
        connect(m_engine, SIGNAL(irc306(QString)), this, SLOT(slotIrc306(QString)));
        connect(m_engine, SIGNAL(irc331(QString,QString)), this, SLOT(slotIrc331(QString,QString)));
        connect(m_engine, SIGNAL(irc332(QString,QString)), this, SLOT(slotIrc332(QString,QString)));
        connect(m_engine, SIGNAL(irc333(QString,QString)), this, SLOT(slotIrc333(QString,QString)));
        connect(m_engine, SIGNAL(ircKick(QString,QString,QString,QString)), this, SLOT(slotIrcKick(QString,QString,QString,QString)));
        connect(m_engine, SIGNAL(ircAway(QString)), this, SLOT(slotIrcAway(QString)));
        connect(m_engine, SIGNAL(ircEndMotd()), this, SLOT(slotIrcEndMotd()));
    }

    loadLogLines();
}

IrcTabItem::~IrcTabItem()
{
}

bool IrcTabItem::isSameServer(IrcEngine* engine)
{
    return engine == m_engine;
}

void IrcTabItem::clearChat()
{
    m_text->clear();
}

void IrcTabItem::hideUsers()
{
    if(m_type == CHANNEL)
        m_users->hide();
}

void IrcTabItem::showUsers()
{
    if(m_type == CHANNEL)
        m_users->show();
}

void IrcTabItem::setType(const TYPE& typ, const QString &name)
{
    if(typ == CHANNEL)
    {
        if(preferences.m_usersShown) m_users->show();
        m_topicline->show();
        m_topicline->setText(m_topic);
        m_topicline->setToolTip(dxutils::wrapText(m_topic,400));
        m_topicline->setCursorPosition(0);
        m_name = name;
        if(m_engine->getConnected()) m_engine->sendMode(m_name);
        m_type = typ;
    }
    else if(typ == SERVER || typ == QUERY)
    {
        m_users->hide();
        m_topicline->setText("");
        m_topicline->hide();
        m_topic = _("No topic is set");
        m_name = name;
        if(m_type == CHANNEL)
        {
            m_model.clear();
            m_numberUsers = 0;
        }
        m_type = typ;
    }
    this->stopLogging();
    if(m_type==CHANNEL || m_type==QUERY)
        loadLogLines();
    if(preferences.m_useSpell && (m_type==CHANNEL || m_type==QUERY) && dxUtils.getAvailableSpellLangsNum())
    {
        QStringList langs = dxUtils.getAvailableSpellLangs();
        QString lang = dxUtils.getChannelLang(m_name);
        for(int i=0; i<langs.size(); ++i)
        {
            m_spellLangs->addItem(langs[i]);
            if(langs[i]==lang) m_spellLangs->setCurrentIndex(i);
        }
        if(preferences.m_showSpellCombo) m_spellLangs->show();
        m_commandline->setUseSpell(true);
        m_commandline->setLanguage(lang);
        m_commandline->setToolTip(_("Current spellchecking language: %1").arg(lang));
    }
    else
    {
        m_commandline->setUseSpell(false);
        m_commandline->setToolTip("");
        m_spellLangs->hide();
    }
}

void IrcTabItem::preferencesUpdated()
{
    m_topicline->setStyleSheet(QString("QLineEdit { color:%1; background:%2; }").arg(preferences.m_colors.text.name()).arg(preferences.m_colors.back.name()));
    m_text->document()->setDefaultStyleSheet(QString("a { color:%1 }").arg(preferences.m_colors.link.name()));
    m_text->setStyleSheet(QString("QTextEdit { color:%1; background:%2; }").arg(preferences.m_colors.text.name()).arg(preferences.m_colors.back.name()));
    m_users->setStyleSheet(QString("QListView { \
                    background-color: %1; \
                    color: %2; \
                    } \
                    ").arg(preferences.m_colors.back.name())
                    .arg(preferences.m_colors.text.name())
                    );
    m_commandline->setStyleSheet(QString("QTextEdit { color:%1; background:%2; }").arg(preferences.m_colors.text.name()).arg(preferences.m_colors.back.name()));
    m_topicline->setFont(preferences.m_ircFont);
    m_text->setFont(preferences.m_ircFont);
    if(preferences.m_sameCmd) m_commandline->setFont(preferences.m_ircFont);
    else m_commandline->setFont(QFont());
    if(preferences.m_sameList) m_users->setFont(preferences.m_ircFont);
    else m_users->setFont(QFont());
    m_model.updateIconTheme();
    stopLogging();
    if(preferences.m_useSpell && (m_type==CHANNEL || m_type==QUERY) && dxUtils.getAvailableSpellLangsNum())
    {
        QStringList langs = dxUtils.getAvailableSpellLangs();
        QString lang = dxUtils.getChannelLang(m_name);
        for(int i=0; i<langs.size(); ++i)
        {
            m_spellLangs->addItem(langs[i]);
            if(langs[i]==lang) m_spellLangs->setCurrentIndex(i);
        }
        if(preferences.m_showSpellCombo) m_spellLangs->show();
        m_commandline->setUseSpell(true);
        m_commandline->setLanguage(lang);
        m_commandline->setToolTip(_("Current spellchecking language: %1").arg(lang));
    }
    else
    {
        m_commandline->setUseSpell(false);
        m_commandline->setToolTip("");
        m_spellLangs->hide();
    }
}

void IrcTabItem::setCommandFocus()
{
    m_commandline->setFocus();
}

void IrcTabItem::showSpellComboUpdated()
{
    if(m_type == CHANNEL || m_type == QUERY)
    {
        if(preferences.m_showSpellCombo) m_spellLangs->show();
        else m_spellLangs->hide();
    }
}

void IrcTabItem::makeLastRowVisible()
{
    QTextCursor c =  m_text->textCursor();
    c.movePosition(QTextCursor::End);
    m_text->setTextCursor(c);
}

QString IrcTabItem::getSpellLang()
{
#ifdef HAVE_ENCHANT
    if(m_spellLangs->count()) return m_spellLangs->currentText();
#endif
    return "";
}

//if highlight==true, highlight tab
void IrcTabItem::appendText(QString msg, bool bhighlight, bool logLine)
{
    appendIrcText(msg, 0, false, logLine);
    if(bhighlight)
    {
        if(highlightNeeded(msg))
        {
            emit highlight(m_type);
        }
        else emit newmsg(m_type);
    }
}

/* if highlight==true, highlight tab
 * disableStrip is for dxirc.Print
*/
void IrcTabItem::appendStyledText(QString text, int style, bool bhighlight, bool disableStrip, bool logLine)
{
    if(style) appendIrcStyledText(text, style, "", disableStrip, logLine);
    else appendIrcText(text, "", disableStrip, logLine);
    if(bhighlight)
    {
        if(m_type != OTHER && highlightNeeded(text))
        {
            emit highlight(m_type);
        }
        else emit newmsg(m_type);
    }
}

bool IrcTabItem::processLine(const QString &commandtext)
{
    QString command = (commandtext[0] == '/' ? commandtext.section(' ',0,0) : "");
    if(!dxUtils.getAlias(command).isEmpty())
    {
        QString acommand = dxUtils.getAlias(command);
        if(acommand.contains("%s"))
            acommand.replace("%s", commandtext.section(' ',1));
        else
            acommand += commandtext.section(' ',1);
        int num = acommand.count("&&");
        if(num)
        {
            bool result = false;
            QStringList commands = acommand.split("&&", QString::SkipEmptyParts);
            for(int i=0; i<=commands.size(); i++)
            {
                if(!commands[i].trimmed().isEmpty()) result = processCommand(commands[i].trimmed());
            }
            return result;
        }
        else
        {
            return processCommand(acommand);
        }
    }
    return processCommand(commandtext);
}

//for "handle" checking, if script contains "all". Send from dxirc.
void IrcTabItem::setHasAllCommand(bool hasAll)
{
    m_scriptHasAll = hasAll;
}

//for "handle" checking, if script contains "ownmsg". Send from dxirc.
void IrcTabItem::setHasOwnMsg(bool hasOwn)
{
    m_scriptHasOwnMsg = hasOwn;
}

bool IrcTabItem::eventFilter(QObject *obj, QEvent *ev)
{
    if(obj == m_commandline && ev->type() == QEvent::KeyPress)
    {
        QKeyEvent *e = (QKeyEvent *)ev;
        if(e->key() == Qt::Key_Tab)
        {
            processCompletion();
            return true;
        }
        else if(e->key() == Qt::Key_Up)
        {
            QString line = m_commandline->text();
            if(m_currentPosition!=-1 && m_currentPosition<m_commandsHistory.size())
            {
                if(!line.isEmpty() && line!=m_commandsHistory[m_currentPosition])
                {
                    m_commandsHistory.append(line);
                    if(m_commandsHistory.size() > m_historyMax)
                        m_commandsHistory.removeFirst();
                    m_currentPosition = m_commandsHistory.size()-1;
                }
                if(m_currentPosition > 0 && !line.isEmpty())
                    --m_currentPosition;
                m_commandline->setText(m_commandsHistory[m_currentPosition]);
            }
            return true;
        }
        else if(e->key() == Qt::Key_Down)
        {
            QString line = m_commandline->text();
            if(m_currentPosition!=-1 && m_currentPosition<m_commandsHistory.size())
            {
                if(!line.isEmpty() && line!=m_commandsHistory[m_currentPosition])
                {
                    m_commandsHistory.append(line);
                    if(m_commandsHistory.size() > m_historyMax)
                        m_commandsHistory.removeFirst();
                    m_currentPosition = m_commandsHistory.size()-1;
                }
                if(m_currentPosition < m_commandsHistory.size()-1)
                {
                    ++m_currentPosition;
                    m_commandline->setText(m_commandsHistory[m_currentPosition]);
                }
                else
                    m_commandline->setText("");
            }
            return true;
        }
        else if(dxutils::getKeybinding(e) == preferences.getBinding("colorControlChar"))
        {
            int curpos = m_commandline->cursorPosition();
            QString line = m_commandline->text();
            line.insert(curpos, "^C");
            m_commandline->setText(line);
            m_commandline->setCursorPosition(curpos+2);
            return true;
        }
        else if(dxutils::getKeybinding(e) == preferences.getBinding("boldControlChar"))
        {
            int curpos = m_commandline->cursorPosition();
            QString line = m_commandline->text();
            line.insert(curpos, "^B");
            m_commandline->setText(line);
            m_commandline->setCursorPosition(curpos+2);
            return true;
        }
        else if(dxutils::getKeybinding(e) == preferences.getBinding("underlineControlChar"))
        {
            int curpos = m_commandline->cursorPosition();
            QString line = m_commandline->text();
            line.insert(curpos, "^_");
            m_commandline->setText(line);
            m_commandline->setCursorPosition(curpos+2);
            return true;
        }
        else if(dxutils::getKeybinding(e) == preferences.getBinding("reverseControlChar"))
        {
            int curpos = m_commandline->cursorPosition();
            QString line = m_commandline->text();
            line.insert(curpos, "^V");
            m_commandline->setText(line);
            m_commandline->setCursorPosition(curpos+2);
            return true;
        }
        else if(dxutils::getKeybinding(e) == preferences.getBinding("resetControlChar"))
        {
            int curpos = m_commandline->cursorPosition();
            QString line = m_commandline->text();
            line.insert(curpos, "^O");
            m_commandline->setText(line);
            m_commandline->setCursorPosition(curpos+2);
            return true;
        }
    }
    return QWidget::eventFilter(obj, ev);
}

void IrcTabItem::processCompletion()
{
    QString line = m_commandline->text();
    if(line.section(' ',0,0).compare("/help", Qt::CaseInsensitive) == 0 && !line.section(' ',1).isEmpty())
    {
        completeHelpCommand();
        return;
    }
    if(line[0] == '/' && line.section(' ', 1).isEmpty())
    {
        line = line.section(' ',0,0).section('/',1);
        for(int i = 0; i < dxUtils.commandsNo(); i++)
        {
            if(line.compare(dxUtils.commandsAt(i), Qt::CaseInsensitive) == 0)
            {
                if((i+1) < dxUtils.commandsNo()) m_commandline->setText("/"+dxUtils.commandsAt(++i)+" ");
                else m_commandline->setText("/"+dxUtils.commandsAt(0)+" ");
                break;
            }
            else if(line.compare(dxUtils.commandsAt(i).left(line.length()), Qt::CaseInsensitive) == 0)
            {
                m_commandline->setText("/"+dxUtils.commandsAt(i)+" ");
                break;
            }
        }
        return;
    }
    if(line[0] != '/' && line.section(' ',1).isEmpty() && m_type == CHANNEL)
    {
        m_commandline->setText(m_sortModel.getCompletion(line));
        return;
    }
    if(line.contains(' ') && m_type == CHANNEL)
    {
        int curpos = m_commandline->cursorPosition();
        m_commandline->setText(m_sortModel.getCompletion(line, curpos));
        m_commandline->setCursorPosition(curpos);
        return;
    }
}

bool IrcTabItem::commandScript(const QString &commandtext)
{
    LuaRequest lua;
    lua.type = LUA_COMMAND;
    lua.text = commandtext.section('/',1);
    emit luaRequest(lua);
    return true;
}

bool IrcTabItem::commandAdmin(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        return m_engine->sendAdmin(commandtext.section(' ',1));
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandAway(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        if(commandtext.section(' ',1).length() > m_engine->getAwayLen())
        {
            appendIrcStyledText(_("Warning: Away message is too long. Max. away message length is %1.").arg(m_engine->getAwayLen()), 4, "", false, false);
        }
        return m_engine->sendAway(commandtext.section(' ',1));
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandBanlist(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString channel = commandtext.section(' ',1);
        if(channel.isEmpty() && m_type == CHANNEL)
        {
            return m_engine->sendBanlist(m_name);
        }
        else if(!isChannel(channel) && m_type != CHANNEL)
        {
            appendIrcStyledText(_("/banlist <channel>, shows banlist for channel."), 4, "", false, false);
            return false;
        }
        else
        {
            return m_engine->sendBanlist(channel);
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandBoats(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString to = commandtext.section(' ',1);
        if(to.isEmpty())
        {
            appendIrcStyledText(_("/boats <nick>, invites someone to a boats game."), 4, "", false, false);
            return false;
        }
        else
        {
            return m_engine->sendCtcp(to, "BOATS INVITE");
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandConnect(const QString &commandtext)
{
    if(commandtext.section(' ',1).isEmpty())
    {
        appendIrcStyledText(_("/connect <server> [port] [nick] [password] [realname] [channels], connects for given server."), 4, "", false, false);
        return false;
    }
    else
    {
        ServerInfo srv;
        srv.hostname = commandtext.section(' ',1,1);
        srv.port = commandtext.section(' ',2,2).isEmpty() ? 6667 : commandtext.section(' ',2,2).toInt();
        srv.nick = commandtext.section(' ',3,3).isEmpty() ? "xxx" : commandtext.section(' ',3,3);
        srv.passwd = commandtext.section(' ',4,4).isEmpty() ? "" : commandtext.section(' ',4,4);
        srv.realname = commandtext.section(' ',5,5).isEmpty() ? "xxx" : commandtext.section(' ',5,5);
        srv.channels = commandtext.section(' ',6).isEmpty() ? "" : commandtext.section(' ',6);
        srv.useSsl = false;
        srv.autoConnect = false;
        emit connectServer(srv);
        return true;
    }
}

bool IrcTabItem::commandCommands()
{
    appendIrcStyledText(dxUtils.availableCommands(), 3, "", false, false);
    return true;
}

bool IrcTabItem::commandCtcp(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString to = commandtext.section(' ',1).section(' ',0,0);
        QString msg = commandtext.section(' ',2);
        if(to.isEmpty() || msg.isEmpty())
        {
            appendIrcStyledText(_("/ctcp <nick> <message>, sends a CTCP message to a user."), 4, "", false, false);
            return false;
        }
        if(msg.length() > m_maxLen-12-to.length())
        {
            appendIrcStyledText(_("Warning: ctcp message is too long. Max. ctcp message length is %1.").arg(m_maxLen-12-to.length()), 4, "", false, false);
        }
        return m_engine->sendCtcp(to, msg);
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandCycle(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        if(m_type == CHANNEL)
        {
            if(isChannel(commandtext.section(' ',1)))
            {
                QString channel = commandtext.section(' ',1).section(' ',0,0);
                QString reason = commandtext.section(' ',2);
                reason.isEmpty() ? m_engine->sendPart(channel) : m_engine->sendPart(channel, reason);
                return m_engine->sendJoin(channel);
            }
            else
            {
                commandtext.section(' ',1).isEmpty() ? m_engine->sendPart(m_name) : m_engine->sendPart(m_name, commandtext.section(' ',1));
                return m_engine->sendJoin(m_name);
            }
        }
        else
        {
            if(isChannel(commandtext.section(' ',1)))
            {
                QString channel = commandtext.section(' ',1).section(' ',0,0);
                QString reason = commandtext.section(' ', 2);
                reason.isEmpty() ? m_engine->sendPart(channel) : m_engine->sendPart(channel, reason);
                return m_engine->sendJoin(channel);
            }
            else
            {
                appendIrcStyledText(_("/cycle <channel> [message], leaves and join channel."), 4, "", false, false);
                return false;
            }
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandDcc(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString dccCommand = dxutils::getParam(commandtext, 2, false).toLower();
        if(dccCommand == "chat")
        {
            QString nick = dxutils::getParam(commandtext, 3, false);
            if(!QString::compare(nick, "chat", Qt::CaseInsensitive))
            {
                appendIrcStyledText(_("Nick for chat wasn't entered."), 4, "", false, false);
                return false;
            }
            if(!QString::compare(nick, getNickName(), Qt::CaseInsensitive))
            {
                appendIrcStyledText(_("Chat with yourself isn't good idea."), 4, "", false, false);
                return false;
            }
            emit ircDccServer(m_engine, nick);
            return true;
        }
        else if(dccCommand == "send")
        {
            QString nick = dxutils::getParam(commandtext, 3, false);
            QString file = dxutils::getParam(commandtext, 4, true);
            if(!QString::compare(nick, "send", Qt::CaseInsensitive))
            {
                appendIrcStyledText(_("Nick for sending file wasn't entered."), 4, "", false, false);
                return false;
            }
            if(!QString::compare(nick, getNickName(), Qt::CaseInsensitive))
            {
                appendIrcStyledText(_("Sending to yourself isn't good idea."), 4, "", false, false);
                return false;
            }
            if(!QString::compare(nick, file, Qt::CaseInsensitive))
            {
                appendIrcStyledText(_("Filename wasn't entered"), 4, "", false, false);
                return false;
            }
            if(!QFile::exists(file))
            {
                appendIrcStyledText(_("File '%1' doesn't exist").arg(file), 4, "", false, false);
                return false;
            }
            emit ircDccOut(m_engine, nick, file);
            return true;
        }
        else if(dccCommand == "psend")
        {
            QString nick = dxutils::getParam(commandtext, 3, false);
            QString file = dxutils::getParam(commandtext, 4, true);
            if(!QString::compare(nick, "psend", Qt::CaseInsensitive))
            {
                appendIrcStyledText(_("Nick for sending file wasn't entered."), 4, "", false, false);
                return false;
            }
            if(!QString::compare(nick, getNickName(), Qt::CaseInsensitive))
            {
                appendIrcStyledText(_("Sending to yourself isn't good idea."), 4, "", false, false);
                return false;
            }
            if(!QString::compare(nick, file, Qt::CaseInsensitive))
            {
                appendIrcStyledText(_("Filename wasn't entered"), 4, "", false, false);
                return false;
            }
            if(!QFile::exists(file))
            {
                appendIrcStyledText(_("File '%1' doesn't exist").arg(file), 4, "", false, false);
                return false;
            }
            emit ircDccPout(m_engine, nick, file);
            return true;
        }
        else
        {
            appendIrcStyledText(_("'%1' isn't dcc command <chat|send|psend>").arg(dccCommand), 4, "", false, false);
            return false;
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandDeop(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString params = commandtext.section(' ',1);
        if(m_type == CHANNEL)
        {
            if(params.isEmpty())
            {
                appendIrcStyledText(_("/deop <nicks>, removes operator status from one or more nicks."), 4, "", false, false);
                return false;
            }
            else if(isChannel(params) && params.section(' ',1).isEmpty())
            {
                appendIrcStyledText(_("/deop <channel> <nicks>, removes operator status from one or more nicks."), 4, "", false, false);
                return false;
            }
            else if(isChannel(params) && !params.section(' ',1).isEmpty())
            {
                QString channel = params.section(' ',0,0);
                QString nicks = params.section(' ',1);
                QString modeparams = dxutils::createModes('-', 'o', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
            else
            {
                QString channel = m_name;
                QString nicks = params;
                QString modeparams = dxutils::createModes('-', 'o', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
        }
        else
        {
            if(isChannel(params) && !params.section(' ',1).isEmpty())
            {
                QString channel = params.section(' ',0,0);
                QString nicks = params.section(' ',1);
                QString modeparams = dxutils::createModes('-', 'o', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
            else
            {
                appendIrcStyledText(_("/deop <channel> <nicks>, removes operator status from one or more nicks."), 4, "", false, false);
                return false;
            }
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandDevoice(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString params = commandtext.section(' ',1);
        if(m_type == CHANNEL)
        {
            if(params.isEmpty())
            {
                appendIrcStyledText(_("/devoice <nicks>, removes voice from one or more nicks."), 4, "", false, false);
                return false;
            }
            else if(isChannel(params) && params.section(' ',1).isEmpty())
            {
                appendIrcStyledText(_("/devoice <channel> <nicks>, removes voice from one or more nicks."), 4, "", false, false);
                return false;
            }
            else if(isChannel(params) && !params.section(' ',1).isEmpty())
            {
                QString channel = params.section(' ',0,0);
                QString nicks = params.section(' ',1);
                QString modeparams = dxutils::createModes('-', 'v', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
            else
            {
                QString channel = m_name;
                QString nicks = params;
                QString modeparams = dxutils::createModes('-', 'v', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
        }
        else
        {
            if(isChannel(params) && !params.section(' ',1).isEmpty())
            {
                QString channel = params.section(' ',0,0);
                QString nicks = params.section(' ',1);
                QString modeparams = dxutils::createModes('-', 'v', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
            else
            {
                appendIrcStyledText(_("/devoice <channel> <nicks>, removes voice from one or more nicks."), 4, "", false, false);
                return false;
            }
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandDisconnect(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        if(commandtext.section(' ',1).isEmpty()) m_engine->disconnectIrc();
        else m_engine->disconnectIrc(commandtext.section(' ',1));
        return true;
    }
    else
    {
        m_engine->closeConnection(true);
        return true;
    }
}

bool IrcTabItem::commandDxirc()
{
    int max = 5+rand()%5;
    for(int i=0; i<max; i++)
    {
        appendIrcText("", "", false, false);
    }
    appendIrcText("     __  __         _", "", false, false);
    appendIrcText("  _/__//__/|_      | |       _", "", false, false);
    appendIrcText(" /_| |_| |/_/|   __| |__  __|_| _ _  ___", "", false, false);
    appendIrcText(" |_   _   _|/   / _  |\\ \\/ /| || '_)/ __)", "", false, false);
    appendIrcText(" /_| |_| |/_/| | (_| | |  | | || | | (__", "", false, false);
    appendIrcText(" |_   _   _|/   \\____|/_/\\_\\|_||_|  \\___)", "", false, false);
    appendIrcText("   |_|/|_|/     (c) 2008~ David Vachulka", "", false, false);
    appendIrcText("   http://dxirc.org", "", false, false);
    max = 5+rand()%5;
    for(int i=0; i<max; i++)
    {
        appendIrcText("", "", false, false);
    }
    return true;
}

bool IrcTabItem::commandEgg()
{
    clearChat();
    appendStyledText("ahoj sem pan Vajíčko.\n", 3, false, false, false);
    m_eggTimer->start(1000);
    m_pics = 0;
    return true;
}

bool IrcTabItem::commandExec(const QString &commandtext)
{
    QString params = commandtext.section(' ',1);
    if(params.isEmpty())
    {
        appendIrcStyledText(_("/exec [-o|-c] <command>, executes command, -o sends output to channel/query, -c closes running command."), 4, "", false, false);
        return false;
    }
    else
    {
        if(!m_pipe)
        {
            m_pipe = new QProcess(this);
            connect(m_pipe, SIGNAL(readyReadStandardOutput()), this, SLOT(slotOnPipe()));
        }
        m_pipeStrings.clear();
        if(params.section(' ',0,0).contains("-o"))
        {
            m_sendPipe = true;
            m_pipe->start(params.section(' ',1));
        }
        else if(params.section(' ',0,0).contains("-c"))
        {
            m_sendPipe = false;
            m_pipeStrings.clear();
            m_pipe->close();
        }
        else
        {
            m_sendPipe = false;
            m_pipe->start(params);
        }
        return true;
    }
}

bool IrcTabItem::commandHelp(const QString &commandtext)
{
    return showHelp(commandtext.section(' ',1).toLower().trimmed());
}

bool IrcTabItem::commandIgnore(const QString &commandtext)
{
    QString ignorecommand = commandtext.section(' ',1).section(' ',0,0);
    QString ignoretext = commandtext.section(' ',1).section(' ',1);
    if(ignorecommand.isEmpty())
    {
        appendIrcStyledText(_("/ignore <list|addcmd|rmcmd|addusr|rmusr> [command|user] [channel] [server]"), 4, "", false, false);
        return false;
    }
    if(QString::compare(ignorecommand, "list", Qt::CaseInsensitive)==0)
    {
        appendIrcStyledText(_("Ignored commands:"), 7, "", false, false);
        if(preferences.m_ignoreCommandsList.isEmpty())
        {
            appendIrcText(_("No ignored commands"), "", false, false);
        }
        else
        {
            appendIrcText(preferences.m_ignoreCommandsList.join(";"), "", false, false);
        }
        appendIrcStyledText(_("Ignored users:"), 7, "", false, false);
        if(!preferences.m_ignoreUsersList.size())
        {
            appendIrcText(_("No ignored users"), "", false, false);
        }
        else
        {
            for(int i=0; i<preferences.m_ignoreUsersList.size(); i++)
            {
                appendIrcText(_("%1 on channel(s): %2 and network: %3").arg(preferences.m_ignoreUsersList[i].nick).arg(preferences.m_ignoreUsersList[i].channel).arg(preferences.m_ignoreUsersList[i].network), "", false, false);
            }
        }
        return true;
    }
    else if(QString::compare(ignorecommand, "addcmd", Qt::CaseInsensitive)==0)
    {
        if(ignoretext.isEmpty())
        {
            appendIrcStyledText(_("/ignore addcmd <command>, adds command to ignored commands."), 4, "", false, false);
            return false;
        }
        emit addIgnoreCommand(ignoretext);
        return true;
    }
    else if(QString::compare(ignorecommand, "rmcmd", Qt::CaseInsensitive)==0)
    {
        if(ignoretext.isEmpty())
        {
            appendIrcStyledText(_("/ignore rmcmd <command>, removes command from ignored commands."), 4, "", false, false);
            return false;
        }
        emit removeIgnoreCommand(ignoretext);
        return true;
    }
    else if(QString::compare(ignorecommand, "addusr", Qt::CaseInsensitive)==0)
    {
        if(ignoretext.isEmpty())
        {
            appendIrcStyledText(_("/ignore addusr <user> [channel] [server], adds user to ignored users."), 4, "", false, false);
            return false;
        }
        emit addIgnoreUser(ignoretext);
        return true;
    }
    else if(QString::compare(ignorecommand, "rmusr", Qt::CaseInsensitive)==0)
    {
        if(ignoretext.isEmpty())
        {
            appendIrcStyledText(_("/ignore rmusr <user>, removes user from ignored users."), 4, "", false, false);
            return false;
        }
        emit removeIgnoreUser(ignoretext);
        return true;
    }
    else
    {
        appendIrcStyledText(_("'%1' isn't <list|addcmd|rmcmd|addusr|rmusr>").arg(ignorecommand), 4, "", false, false);
        return false;
    }
}

bool IrcTabItem::commandInvite(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString params = commandtext.section(' ',1);
        if(params.isEmpty())
        {
            appendIrcStyledText(_("/invite <nick> <channel>, invites someone to a channel."), 4, "", false, false);
            return false;
        }
        else if(isChannel(params) && params.section(' ',1).isEmpty())
        {
            appendIrcStyledText(_("/invite <nick> <channel>, invites someone to a channel."), 4, "", false, false);
            return false;
        }
        else
        {
            QString nick = params.section(' ',0,0);
            QString channel = params.section(' ',1);
            return m_engine->sendInvite(nick, channel);
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandJoin(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString channel = commandtext.section(' ',1);
        if(!isChannel(channel))
        {
            appendIrcStyledText(_("/join <channel>, joins a channel."), 4, "", false, false);
            appendIrcStyledText(_("'%1' isn't valid char for channel.").arg(channel[0]), 4, "", false, false);
            return false;
        }
        else
        {
            return m_engine->sendJoin(channel);
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandKick(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString params = commandtext.section(' ',1);
        if(m_type == CHANNEL)
        {
            if(params.isEmpty())
            {
                appendIrcStyledText(_("/kick <nick>, kicks a user from a channel."), 4, "", false, false);
                return false;
            }
            else if(isChannel(params) && params.section(' ',1).isEmpty())
            {
                appendIrcStyledText(_("/kick <channel> <nick>, kicks a user from a channel."), 4, "", false, false);
                return false;
            }
            else if(isChannel(params) && !params.section(' ',1).isEmpty())
            {
                QString channel = params.section(' ',0,0);
                QString nick = params.section(' ',1,1);
                QString reason = params.section(' ', 2);
                if(reason.length() > m_engine->getKickLen())
                {
                    appendIrcStyledText(_("Warning: reason of kick is too long. Max. reason length is %1.").arg(m_engine->getKickLen()), 4, "", false, false);
                }
                return m_engine->sendKick(channel, nick, reason);
            }
            else
            {
                QString channel = m_name;
                QString nick = params.section(' ',0,0);
                QString reason = params.section(' ',1);
                if(reason.length() > m_engine->getKickLen())
                {
                    appendIrcStyledText(_("Warning: reason of kick is too long. Max. reason length is %1.").arg(m_engine->getKickLen()), 4, "", false, false);
                }
                return m_engine->sendKick(channel, nick, reason);
            }
        }
        else
        {
            if(isChannel(params) && !params.section(' ',1).isEmpty())
            {
                QString channel = params.section(' ',0,0);
                QString nick = params.section(' ',1,1);
                QString reason = params.section(' ', 2);
                if(reason.length() > m_engine->getKickLen())
                {
                    appendIrcStyledText(_("Warning: reason of kick is too long. Max. reason length is %1.").arg(m_engine->getKickLen()), 4, "", false, false);
                }
                return m_engine->sendKick(channel, nick, reason);
            }
            else
            {
                appendIrcStyledText(_("/kick <channel> <nick>, kicks a user from a channel."), 4, "", false, false);
                return false;
            }
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandKill(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString params = commandtext.section(' ',1);
        QString nick = params.section(' ',0,0);
        QString reason = params.section(' ',1);
        if(params.isEmpty())
        {
            appendIrcStyledText(_("/kill <user> [reason], kills a user from the network."), 4, "", false, false);
            return false;
        }
        if(reason.length() > m_maxLen-7-nick.length())
        {
            appendIrcStyledText(_("Warning: reason of kill is too long. Max. reason length is %1.").arg(m_maxLen-7-nick.length()), 4, "", false, false);
        }
        return m_engine->sendKill(nick, reason);
    }
    else
    {
        appendIrcStyledText(_("You aren't connected"), 4, "", false, false);
        emit connectDialog();
        return true;
    }
}

bool IrcTabItem::commandList(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        return m_engine->sendList(commandtext.section(' ',1));
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandLog(const QString &commandtext)
{
    if(m_type == SERVER)
    {
        appendIrcStyledText(_("Server tab hasn't log files"), 4, "", false, false);
        return true;
    }
    QString logcommand = commandtext.section(' ',1).section(' ',0,0);
    QString logfile = commandtext.section(' ',1).section(' ',1);
    if(QString::compare(logcommand, "list", Qt::CaseInsensitive)==0)
    {
        listLogFiles();
        return true;
    }
    else if(QString::compare(logcommand, "show", Qt::CaseInsensitive)==0)
    {
        if(logfile.isEmpty())
            logfile = QDateTime::currentDateTime().toString("yyyy-MM-dd");
        QString logpath = preferences.m_logPath+"/"+m_engine->getNetworkName()+"/"+m_name+"/"+logfile;
        if(!QFile::exists(logpath))
        {
            appendIrcStyledText(_("Log file %1 doesn't exist").arg(logfile), 4, "", false, false);
            return false;
        }
        emit newlog(logpath);
        return true;
    }
    else
    {
        appendIrcStyledText(_("'%1' isn't <list|show>").arg(logcommand), 4, "", false, false);
        return false;
    }
}

bool IrcTabItem::commandLua(const QString &commandtext)
{
#ifdef HAVE_LUA
    QString luacommand = commandtext.section(' ',1).section(' ',0,0);
    QString luatext = commandtext.section(' ',1).section(' ',1);
    LuaRequest lua;
    if(luacommand.isEmpty())
    {
        appendIrcStyledText(_("/lua <help|load|unload|list> [scriptpath|scriptname]"), 4, "", false, false);
        return false;
    }
    if(QString::compare(luacommand, "help", Qt::CaseInsensitive)==0)
    {
        appendIrcStyledText(_("For help about Lua scripting visit: %1").arg(LUA_HELP_PATH), 3, "", false, false);
        return true;
    }
    else if(QString::compare(luacommand, "load", Qt::CaseInsensitive)==0) lua.type = LUA_LOAD;
    else if(QString::compare(luacommand, "unload", Qt::CaseInsensitive)==0) lua.type = LUA_UNLOAD;
    else if(QString::compare(luacommand, "list", Qt::CaseInsensitive)==0) lua.type = LUA_LIST;
    else
    {
        appendIrcStyledText(_("'%1' isn't <help|load|unload|list>").arg(luacommand), 4, "", false, false);
        return false;
    }
    lua.text = luatext;
    emit luaRequest(lua);
    return true;
#else
    appendIrcStyledText(_("dxirc is compiled without support for Lua scripting"), 4, "", false, false);
    return false;
#endif
}

bool IrcTabItem::commandMe(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString params = commandtext.section(' ',1);
        if(m_type == CHANNEL)
        {
            if(params.isEmpty())
            {
                appendIrcStyledText(_("/me <message>, sends the action to the current channel."), 4, "", false, false);
                return false;
            }
            else if(isChannel(params) && !params.section(' ',1).isEmpty())
            {
                QString channel = params.section(' ',0,0);
                QString message = params.section(' ',1);
                if(message.length() > m_maxLen-19-channel.length())
                {
                    QStringList messages = cutText(message, m_maxLen-19-channel.length());
                    bool result = true;
                    for(int i=0; i<messages.size(); i++)
                    {
                        if(channel == m_name)
                        {
                            appendIrcStyledText(getNickName()+" "+messages[i], 2, "");
                            emit ircAction(m_engine, getNickName(), channel, messages[i]);
                        }
                        result = m_engine->sendMe(channel, messages[i]) &result;
                    }
                    return result;
                }
                else
                {
                    if(channel == m_name)
                    {
                        appendIrcStyledText(getNickName()+" "+message, 2, "");
                        emit ircAction(m_engine, getNickName(), channel, message);
                    }
                    return m_engine->sendMe(channel, message);
                }
            }
            else
            {
                if(params.length() > m_maxLen-19-m_name.length())
                {
                    QStringList messages = cutText(params, m_maxLen-19-m_name.length());
                    bool result = true;
                    for(int i=0; i<messages.size(); i++)
                    {
                        appendIrcStyledText(getNickName()+" "+messages[i], 2, "");
                        emit ircAction(m_engine, getNickName(), m_name, messages[i]);
                        result = m_engine->sendMe(m_name, messages[i]) &result;
                    }
                    return result;
                }
                else
                {
                    appendIrcStyledText(getNickName()+" "+params, 2, "");
                    emit ircAction(m_engine, getNickName(), m_name, params);
                    return m_engine->sendMe(m_name, params);
                }
            }
        }
        else if(m_type == QUERY)
        {
            if(params.isEmpty())
            {
                appendIrcStyledText(_("/me <message>, sends the action to the current query."), 4, "", false, false);
                return false;
            }
            else
            {
                if(params.length() > m_maxLen-19-m_name.length())
                {
                    QStringList messages = cutText(params, m_maxLen-19-m_name.length());
                    bool result = true;
                    for(int i=0; i<messages.size(); i++)
                    {
                        appendIrcStyledText(getNickName()+" "+messages[i], 2, "");
                        emit ircAction(m_engine, getNickName(), m_name, messages[i]);
                        result = m_engine->sendMe(m_name, messages[i]) &result;
                    }
                    return result;
                }
                else
                {
                    appendIrcStyledText(getNickName()+" "+params, 2, "");
                    emit ircAction(m_engine, getNickName(), m_name, params);
                    return m_engine->sendMe(m_name, params);
                }
            }
        }
        else
        {
            if(!params.section(' ',1).isEmpty())
            {
                QString to = params.section(' ',0,0);
                QString message = params.section(' ',1);
                if(message.length() > m_maxLen-19-to.length())
                {
                    QStringList messages = cutText(message, m_maxLen-19-to.length());
                    bool result = true;
                    for(int i=0; i<messages.size(); i++)
                    {
                        result = m_engine->sendMe(to, messages[i]) &result;
                    }
                    return result;
                }
                else
                {
                    return m_engine->sendMe(to, message);
                }
            }
            else
            {
                appendIrcStyledText(_("/me <to> <message>, sends the action."), 4, "", false, false);
                return false;
            }
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandMode(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString params = commandtext.section(' ',1);
        if(params.isEmpty())
        {
            appendIrcStyledText(_("/mode <channel> <modes>, sets modes for a channel."), 4, "", false, false);
            return false;
        }
        else
        {
            return m_engine->sendMode(params);
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandMsg(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString params = commandtext.section(' ',1);
        QString to = params.section(' ',0,0);
        QString message = params.section(' ',1);
        if(!to.isEmpty() && !message.isEmpty())
        {
            if(message.length() > m_maxLen-10-to.length())
            {
                QStringList messages = cutText(message, m_maxLen-10-to.length());
                bool result = true;
                for(int i=0; i<messages.size(); i++)
                {
                    if(to == m_name)
                    {
                        appendIrcNickText(getNickName(), messages[i], "");
                        emit ircPrivmsg(m_engine, getNickName(), to, messages[i]);
                    }
                    result = m_engine->sendMsg(to, messages[i]) &result;
                }
                return result;
            }
            else
            {
                if(to == m_name)
                {
                    appendIrcNickText(getNickName(), message, "");
                    emit ircPrivmsg(m_engine, getNickName(), to, message);
                }
                return m_engine->sendMsg(to, message);
            }
        }
        else
        {
            appendIrcStyledText(_("/msg <nick/channel> <message>, sends a normal message."), 4, "", false, false);
            return false;
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandNames(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString params = commandtext.section(' ',1);
        if(m_type == CHANNEL)
        {
            if(params.isEmpty())
            {
                return m_engine->sendNames(m_name, true);
            }
            else
            {
                return m_engine->sendNames(params, true);
            }
        }
        else
        {
            if(params.isEmpty())
            {
                appendIrcStyledText(_("/names <channel>, for nicks on a channel."), 4, "", false, false);
                return false;
            }
            else
            {
                return m_engine->sendNames(params, true);
            }
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandNick(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString nick = commandtext.section(' ',1);
        if(nick.isEmpty())
        {
            appendIrcStyledText(_("/nick <nick>, changes nick."), 4, "", false, false);
            return false;
        }
        if(nick.length() > m_engine->getNickLen())
        {
            appendIrcStyledText(_("Warning: nick is too long. Max. nick length is %1.").arg(m_engine->getNickLen()), 4, "", false, false);
        }
        return m_engine->sendNick(nick);
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandNotice(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString params = commandtext.section(' ',1);
        QString to = params.section(' ',0,0);
        QString message = params.section(' ',1);
        if(!to.isEmpty() && !message.isEmpty())
        {
            if(message.length() > m_maxLen-9-to.length())
            {
                QStringList messages = cutText(message, m_maxLen-9-to.length());
                bool result = true;
                for(int i=0; i<messages.size(); i++)
                {
                    appendIrcStyledText(_("NOTICE to %1: %2").arg(to).arg(messages[i]), 2, "");
                    result = m_engine->sendNotice(to, messages[i]) &result;
                }
                return result;
            }
            else
            {
                appendIrcStyledText(_("NOTICE to %1: %2").arg(to).arg(message), 2, "");
                return m_engine->sendNotice(to, message);
            }
        }
        else
        {
            appendIrcStyledText(_("/notice <nick/channel> <message>, sends a notice."), 4, "", false, false);
            return false;
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandOp(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString params = commandtext.section(' ',1);
        if(m_type == CHANNEL)
        {
            if(params.isEmpty())
            {
                appendIrcStyledText(_("/op <nicks>, gives operator status for one or more nicks."), 4, "", false, false);
                return false;
            }
            else if(isChannel(params) && params.section(' ',1).isEmpty())
            {
                appendIrcStyledText(_("/op <channel> <nicks>, gives operator status for one or more nicks."), 4, "", false, false);
                return false;
            }
            else if(isChannel(params) && !params.section(' ',1).isEmpty())
            {
                QString channel = params.section(' ',0,0);
                QString nicks = params.section(' ',1);
                QString modeparams = dxutils::createModes('+', 'o', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
            else
            {
                QString channel = m_name;
                QString nicks = params;
                QString modeparams = dxutils::createModes('+', 'o', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
        }
        else
        {
            if(isChannel(params) && !params.section(' ',1).isEmpty())
            {
                QString channel = params.section(' ',0,0);
                QString nicks = params.section(' ',1);
                QString modeparams = dxutils::createModes('+', 'o', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
            else
            {
                appendIrcStyledText(_("/op <channel> <nicks>, gives operator status for one or more nicks."), 4, "", false, false);
                return false;
            }
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandOper(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString params = commandtext.section(' ',1);
        QString login = params.section(' ',0,0);
        QString password = params.section(' ',1);
        if(!login.isEmpty() && !password.isEmpty())
        {
            return m_engine->sendOper(login, password);
        }
        else
        {
            appendIrcStyledText(_("/oper <login> <password>, oper up."), 4, "", false, false);
            return false;
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandPart(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        if(m_type == CHANNEL)
        {
            if(commandtext.section(' ',1).isEmpty())
            {
                return m_engine->sendPart(m_name);
            }
            else
            {
                return m_engine->sendPart(m_name, commandtext.section(' ',1));
            }
        }
        else
        {
            if(isChannel(commandtext.section(' ',1)))
            {
                QString channel = commandtext.section(' ',1,1);
                QString reason = commandtext.section(' ', 2);
                if(reason.isEmpty())
                {
                    return m_engine->sendPart(channel);
                }
                else
                {
                    return m_engine->sendPart(channel, reason);
                }
            }
            else
            {
                appendIrcStyledText(_("/part <channel> [reason], leaves channel."), 4, "", false, false);
                return false;
            }
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandQuery(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        if(commandtext.section(' ',1).isEmpty())
        {
            appendIrcStyledText(_("/query <nick>, opens query with nick."), 4, "", false, false);
            return false;
        }
        else
        {
            emit ircQuery(m_engine, commandtext.section(' ',1).section(' ',0,0));
            return true;
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandQuit()
{
    emit ircQuit();
    return true;
}

bool IrcTabItem::commandQuote(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        return m_engine->sendQuote(commandtext.section(' ',1));
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandSay(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        if (m_type != SERVER && !commandtext.section(' ',1).isEmpty())
        {
            if(commandtext.section(' ',1).length() > m_maxLen-10-m_name.length())
            {
                QStringList messages = cutText(commandtext.section(' ',1), m_maxLen-10-m_name.length());
                bool result = true;
                for(int i=0; i<messages.size(); i++)
                {
                    appendMyMsg(messages[i], "");
                    emit ircPrivmsg(m_engine, getNickName(), m_name, messages[i]);
                    result = m_engine->sendMsg(m_name, messages[i]) &result;
                }
                return result;
            }
            else
            {
                appendMyMsg(commandtext.section(' ',1), "");
                emit ircPrivmsg(m_engine, getNickName(), m_name, commandtext.section(' ',1));
                return m_engine->sendMsg(m_name, commandtext.section(' ',1));
            }
        }
        return false;
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandStats(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        return m_engine->sendStats(commandtext.section(' ',1));
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandTetris()
{
    emit newtetris();
    return true;
}

bool IrcTabItem::commandTime()
{
    if(m_engine->getConnected())
    {
        return m_engine->sendQuote("TIME");
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandTopic(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString params = commandtext.section(' ',1);
        if(m_type == CHANNEL)
        {
            if(params.isEmpty())
            {
                return m_engine->sendTopic(m_name);
            }
            else if(isChannel(params) && !params.section(' ',1).isEmpty())
            {
                QString channel = params.section(' ',0,0);
                QString topic = params.section(' ',1);
                if(topic.length() > m_engine->getTopicLen())
                {
                    appendIrcStyledText(_("Warning: topic is too long. Max. topic length is %1.").arg(m_engine->getTopicLen()), 4, "", false, false);
                }
                return m_engine->sendTopic(channel, topic);
            }
            else
            {
                if(params.length() > m_engine->getTopicLen())
                {
                    appendIrcStyledText(_("Warning: topic is too long. Max. topic length is %1.").arg(m_engine->getTopicLen()), 4, "", false, false);
                }
                return m_engine->sendTopic(m_name, params);
            }
        }
        else
        {
            if(isChannel(params))
            {
                QString channel = params.section(' ',0,0);
                QString topic = params.section(' ',1);
                if(topic.length() > m_engine->getTopicLen())
                {
                    appendIrcStyledText(_("Warning: topic is too long. Max. topic length is %1.").arg(m_engine->getTopicLen()), 4, "", false, false);
                }
                return m_engine->sendTopic(channel, topic);
            }
            else
            {
                appendIrcStyledText(_("/topic <channel> [topic], views or changes channel topic."), 4, "", false, false);
                return false;
            }
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandVoice(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString params = commandtext.section(' ',1);
        if(m_type == CHANNEL)
        {
            if(params.isEmpty())
            {
                appendIrcStyledText(_("/voice <nicks>, gives voice for one or more nicks."), 4, "", false, false);
                return false;
            }
            else if(isChannel(params) && params.section(' ',1).isEmpty())
            {
                appendIrcStyledText(_("/voice <channel> <nicks>, gives voice for one or more nicks."), 4, "", false, false);
                return false;
            }
            else if(isChannel(params) && !params.section(' ',1).isEmpty())
            {
                QString channel = params.section(' ',0,0);
                QString nicks = params.section(' ',1);
                QString modeparams = dxutils::createModes('+', 'v', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
            else
            {
                QString channel = m_name;
                QString nicks = params;
                QString modeparams = dxutils::createModes('+', 'v', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
        }
        else
        {
            if(isChannel(params) && !params.section(' ',1).isEmpty())
            {
                QString channel = params.section(' ',0,0);
                QString nicks = params.section(' ',1);
                QString modeparams = dxutils::createModes('+', 'v', nicks);
                return m_engine->sendMode(channel+" "+modeparams);
            }
            else
            {
                appendIrcStyledText(_("/voice <channel> <nicks>, gives voice for one or more nicks."), 4, "", false, false);
                return false;
            }
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandWallops(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString params = commandtext.section(' ',1);
        if(params.isEmpty())
        {
            appendIrcStyledText(_("/wallops <message>, sends wallop message."), 4, "", false, false);
            return false;
        }
        else
        {
            if(params.length() > m_maxLen-9)
            {
                QStringList messages = cutText(params, m_maxLen-9);
                bool result = true;
                for(int i=0; i<messages.size(); i++)
                {
                    result = m_engine->sendWallops(messages[i]) &result;
                }
                return result;
            }
            else
            {
                return m_engine->sendWallops(params);
            }
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandWho(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString params = commandtext.section(' ',1);
        if(params.isEmpty())
        {
            appendIrcStyledText(_("/who <mask> [o], searchs for mask on network, if o is supplied, only search for opers."), 4, "", false, false);
            return false;
        }
        else
        {
            return m_engine->sendWho(params);
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandWhoami()
{
    if(m_engine->getConnected())
    {
        return m_engine->sendWhoami();
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandWhois(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString params = commandtext.section(' ',1);
        if(params.isEmpty())
        {
            if(m_type == QUERY)
            {
                return m_engine->sendWhois(m_name);
            }
            appendIrcStyledText(_("/whois <nick>, whois nick."), 4, "", false, false);
            return false;
        }
        else
        {
            return m_engine->sendWhois(params);
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::commandWhowas(const QString &commandtext)
{
    if(m_engine->getConnected())
    {
        QString params = commandtext.section(' ',1);
        if(params.isEmpty())
        {
            appendIrcStyledText(_("/whowas <nick>, whowas nick."), 4, "", false, false);
            return false;
        }
        else
        {
            return m_engine->sendWhowas(params);
        }
    }
    else
    {
        return notConnected();
    }
}

bool IrcTabItem::notConnected()
{
    appendIrcStyledText(_("You aren't connected"), 4, "", false, false);
    emit connectDialog();
    return true;
}

void IrcTabItem::slotProcessTopicline()
{
    if(m_editableTopic || m_iamOp)
    {
        if(m_topicline->text().length() > m_engine->getTopicLen())
        {
            appendIrcStyledText(_("Warning: topic is too long. Max. topic length is %1.").arg(m_engine->getTopicLen()), 4, "", false, false);
            m_engine->sendTopic(m_name, m_topicline->text());
            return;
        }
        m_engine->sendTopic(m_name, m_topicline->text());
    }
    else
    {
        m_topicline->setText(m_topic);
        m_topicline->setToolTip(dxutils::wrapText(m_topic,400));
        m_topicline->setCursorPosition(0);
    }
}

void IrcTabItem::slotProcessCommandline()
{
    QString commandtext = m_commandline->text();
    if(commandtext.isEmpty())
        return;
    m_commandsHistory.append(commandtext);
    if (m_commandsHistory.size() > m_historyMax)
        m_commandsHistory.removeFirst();
    m_currentPosition = m_commandsHistory.size()-1;
    m_commandline->clear();
    if(QString::compare(commandtext.left(4),"/say", Qt::CaseInsensitive) != 0)
        commandtext.replace("^B", "\002").replace("^C", "\003").replace("^O", "\017").replace("^V", "\026").replace("^_", "\037");
    for(int i=0; i<=commandtext.count('\n'); i++)
    {
        QString text = commandtext.section('\n', i).section('\r',0,0);
        if(QString::compare(text.section('/',1).section(' ',0,0), "quit", Qt::CaseInsensitive) == 0 ||
                QString::compare(text.section('/',1).section(' ',0,0), "lua", Qt::CaseInsensitive) == 0)
        {
            processLine(text);
            return;
        }
#ifdef HAVE_LUA
        if(text[0] != '/') emit ownMsg(text);
        emit command(text);
        if(text[0] == '/' && !m_scriptHasAll) processLine(text);
        else if(!m_scriptHasOwnMsg && !m_scriptHasAll) processLine(text);
#else
        processLine(text);
#endif
    }
}

void IrcTabItem::slotOnPipe()
{
    QByteArray byteArray = m_pipe->readAllStandardOutput();
    bool isUtf8 = dxutils::isUtf8(byteArray);
    QStringList lines = QString(isUtf8?QString::fromUtf8(byteArray):QString::fromLocal8Bit(byteArray)).split('\n', QString::SkipEmptyParts);
    foreach(QString line, lines)
    {
        if(m_sendPipe)
        {
            if(!m_pipeTimer->isActive()) m_pipeTimer->start(3000);
            m_pipeStrings.append(line);
        }
        else appendIrcText(line, "", true, false);
    }
}

void IrcTabItem::slotPipeTimeout()
{
    if(m_type == CHANNEL || m_type == QUERY)
    {
        if(m_pipeStrings.size() > 3)
        {
            if(m_pipeStrings[0].isEmpty()) m_pipeStrings[0] = " ";
            if(m_pipeStrings[0].length() > m_maxLen-10-m_name.length())
            {
                QStringList messages = cutText(m_pipeStrings[0], m_maxLen-10-m_name.length());
                for(int i=0; i<messages.size(); i++)
                {
#ifdef HAVE_LUA
                    emit ownMsg(messages[i]);
                    if(!m_scriptHasOwnMsg)
                    {
                        appendMyMsg(messages[i], "");
                        m_engine->sendMsg(m_name, messages[i]);
                    }
#else
                    appendMyMsg(messages[i], "");
                    m_engine->sendMsg(m_name, messages[i]);
#endif
                }
            }
            else
            {
#ifdef HAVE_LUA
                emit ownMsg(m_pipeStrings[0]);
                if(!m_scriptHasOwnMsg)
                {
                    appendMyMsg(m_pipeStrings[0], "");
                    m_engine->sendMsg(m_name, m_pipeStrings[0]);
                }
#else
                appendMyMsg(m_pipeStrings[0], "");
                m_engine->sendMsg(m_name, m_pipeStrings[0]);
#endif
            }
            m_pipeStrings.pop_front();
            m_pipeTimer->start(3000);
        }
        else
        {
            while(m_pipeStrings.size())
            {
                if(m_pipeStrings[0].isEmpty()) m_pipeStrings[0] = " ";
                if(m_pipeStrings[0].length() > m_maxLen-10-m_name.length())
                {
                    QStringList messages = cutText(m_pipeStrings[0], m_maxLen-10-m_name.length());
                    for(int i=0; i<messages.size(); i++)
                    {
#ifdef HAVE_LUA
                        emit ownMsg(messages[i]);
                        if(!m_scriptHasOwnMsg)
                        {
                            appendMyMsg(messages[i], "");
                            m_engine->sendMsg(m_name, messages[i]);
                        }
#else
                        appendMyMsg(messages[i], "");
                        m_engine->sendMsg(m_name, messages[i]);
#endif
                    }
                }
                else
                {
#ifdef HAVE_LUA
                    emit ownMsg(m_pipeStrings[0]);
                    if(!m_scriptHasOwnMsg)
                    {
                        appendMyMsg(m_pipeStrings[0], "");
                        m_engine->sendMsg(m_name, m_pipeStrings[0]);
                    }
#else
                    appendMyMsg(m_pipeStrings[0], "");
                    m_engine->sendMsg(m_name, m_pipeStrings[0]);
#endif
                }
                m_pipeStrings.pop_front();
            }
        }
    }
}

void IrcTabItem::slotEggTimeout()
{
    if(m_pics<24)
    {
        m_eggTimer->start(222);
        QString replace = "<br />";
        int max = rand()%30;
        for(int i=0; i<max; i++)
            replace.append("&nbsp;");
        QString pic = "<br />dxirc bezi i na Qt, svet shori!!!";
        pic.replace("<br />", replace);
        pic.append("<br />");
        m_text->clear();
        max = rand()%15;
        for(int i=0; i<max; i++)
            pic.prepend("<br />");
        pic.prepend(QString("<div><font color=\"%1\">").arg(QColor(rand()%255,rand()%255,rand()%255).name()));
        pic.append("</font></div>");
        dxutils::debugLine(pic);
        m_text->append(pic);
        m_pics++;
    }
}

void IrcTabItem::slotPopup(const QPoint& pos)
{
    QModelIndex index = m_users->indexAt(pos);
    if(index.isValid())
    {
        QPoint globalPos = m_users->mapToGlobal(pos);
        QMenu *popup = new QMenu(this);
        NickInfo nick = m_engine->getNickInfo(index.data().toString());
        QString flagpath = DXIRC_DATADIR "/icons/flags";
        if(QFile::exists(flagpath+"/"+nick.host.section('.',-1)+".png"))
            flagpath = flagpath+"/"+nick.host.section('.',-1)+".png";
        else
            flagpath = flagpath+"/unknown.png";
        popup->addAction(QIcon(flagpath), _("User: %1@%2").arg(nick.user).arg(nick.host));
        popup->addAction(_("Realname: %1").arg(nick.real));
        if(nick.nick != getNickName())
        {
            popup->addSeparator();
            connect(popup->addAction(_("Query")), SIGNAL(triggered()), this, SLOT(slotPopupQuery()));
            connect(popup->addAction(_("User information (WHOIS)")), SIGNAL(triggered()), this, SLOT(slotPopupWhois()));
            connect(popup->addAction(_("DCC chat")), SIGNAL(triggered()), this, SLOT(slotPopupDccChat()));
            connect(popup->addAction(_("Send file")), SIGNAL(triggered()), this, SLOT(slotPopupSendFile()));
            connect(popup->addAction(_("Ignore")), SIGNAL(triggered()), this, SLOT(slotPopupIgnore()));
            if(m_iamOp)
            {
                UserMode mode = static_cast<UserMode>(index.data(IrcItem::RMode).toInt());
                QMenu *opmenu = new QMenu(_("Operator actions"), this);
                if(mode != OP) connect(opmenu->addAction(_("Give op")), SIGNAL(triggered()), this, SLOT(slotPopupOp()));
                if(mode == OP) connect(opmenu->addAction(_("Remove op")), SIGNAL(triggered()), this, SLOT(slotPopupDeop()));
                opmenu->addSeparator();
                if(mode != VOICE) connect(opmenu->addAction(_("Give voice")), SIGNAL(triggered()), this, SLOT(slotPopupVoice()));
                if(mode == VOICE) connect(opmenu->addAction(_("Remove voice")), SIGNAL(triggered()), this, SLOT(slotPopupDevoice()));
                opmenu->addSeparator();
                connect(opmenu->addAction(_("Kick")), SIGNAL(triggered()), this, SLOT(slotPopupKick()));
                opmenu->addSeparator();
                connect(opmenu->addAction(_("Ban")), SIGNAL(triggered()), this, SLOT(slotPopupBan()));
                connect(opmenu->addAction(_("KickBan")), SIGNAL(triggered()), this, SLOT(slotPopupKickBan()));
                popup->addMenu(opmenu);
            }
        }
        else
        {
            popup->addSeparator();
            if(m_engine->isAway(getNickName()))
                connect(popup->addAction(_("Remove Away")), SIGNAL(triggered()), this, SLOT(slotPopupDeAway()));
            else
                connect(popup->addAction(_("Set Away")), SIGNAL(triggered()), this, SLOT(slotPopupAway()));
        }
        popup->exec(globalPos);
    }
}

bool IrcTabItem::isNoCurrent()
{
    IrcTabItem* currenttab = qobject_cast<IrcTabItem*>(m_tabbook->currentWidget());
    if(currenttab && currenttab->isSameServer(m_engine))
        return false;
    return true;
}

void IrcTabItem::startLogging()
{
    if(m_logstream && QFile::exists(preferences.m_logPath+"/"+m_engine->getNetworkName()+"/"+m_name+"/"+QDateTime::currentDateTime().toString("yyyy-MM-dd")))
        return;
    if(preferences.m_logging && m_type != SERVER)
    {
        QDir dir;
        if(!dir.exists(preferences.m_logPath+"/"+m_engine->getNetworkName()))
            dir.mkpath(preferences.m_logPath+"/"+m_engine->getNetworkName());
        if(!dir.exists(preferences.m_logPath+"/"+m_engine->getNetworkName()+"/"+m_name))
            dir.mkpath(preferences.m_logPath+"/"+m_engine->getNetworkName()+"/"+m_name);
        m_logstream = new std::ofstream(dxutils::toStdString(preferences.m_logPath+QDir::separator()+m_engine->getNetworkName()+QDir::separator()+m_name+QDir::separator()+QDateTime::currentDateTime().toString("yyyy-MM-dd")).c_str(), std::ios::out|std::ios::app);
    }
}

void IrcTabItem::stopLogging()
{
    if(m_logstream)
    {
        m_logstream->close();
        delete m_logstream;
        m_logstream = NULL;
    }
}

void IrcTabItem::logLine(const QString &line, const QString &time)
{
    if(preferences.m_logging && m_type != SERVER)
    {
        this->startLogging();
        *m_logstream << dxutils::toStdString(time) << dxutils::toStdString(line) << std::endl;
    }
}



void IrcTabItem::appendMyMsg(QString msg, QString time)
{
    if(time.isEmpty()) time = QDateTime::currentDateTime().toString("[hh:mm:ss] ");
    if(preferences.m_stripColors) msg = dxutils::stripColors(msg, false);
    QString nick = "";
    if(preferences.m_coloredNick) nick = QString("<font color=\"%1\">%2: </font>").arg(dxutils::getColorForStyle(10).name()).arg(getNickName());
    else nick = QString("<b>%1: </b>").arg(getNickName());
    m_text->append("<div>"+time+nick+dxutils::parseText(msg, 10)+"</div>");
    this->logLine("\01410\014<"+getNickName()+"> "+msg, time);
}

void IrcTabItem::appendIrcNickText(QString nick, QString msg, QString time, bool logLine)
{
    if(time.isEmpty()) time = QDateTime::currentDateTime().toString("[hh:mm:ss] ");
    if(preferences.m_stripColors) msg = dxutils::stripColors(msg, false);
    QString nickStyled = "";
    if(preferences.m_coloredNick) nickStyled = QString("<font color=\"%1\">%2: </font>").arg(dxutils::getNickColor(nick)).arg(nick);
    else nickStyled = QString("<b>%1: </b>").arg(nick);
    if(m_type != OTHER)
        m_text->append("<div>"+time+nickStyled+dxutils::parseText(msg, 0)+"</div>");
    else
        m_text->append("<div>"+nickStyled+dxutils::parseText(msg, 0)+"</div>");
    if(logLine) this->logLine("<"+nick+"> "+msg, time);
}

void IrcTabItem::appendIrcText(QString msg, QString time, bool disableStrip, bool logLine)
{
    if(time.isEmpty()) time = QDateTime::currentDateTime().toString("[hh:mm:ss] ");
    if(preferences.m_stripColors && !disableStrip) msg = dxutils::stripColors(msg, false);
    if(m_type != OTHER)
        m_text->append("<div>"+time+dxutils::parseText(msg, 0)+"</div>");
    else
        m_text->append("<div>"+dxutils::parseText(msg, 0)+"</div>");
    if(logLine) this->logLine(msg, time);
}

void IrcTabItem::appendIrcStyledText(QString styled, int stylenum, QString time, bool disableStrip, bool logLine)
{
    if(time.isEmpty()) time = QDateTime::currentDateTime().toString("[hh:mm:ss] ");
    if(preferences.m_stripColors && !disableStrip) styled = dxutils::stripColors(styled, false);
    if(m_type != OTHER)
        m_text->append("<div>"+time+dxutils::parseText(styled, stylenum)+"</div>");
    else
        m_text->append("<div>"+dxutils::parseText(styled, stylenum)+"</div>");
    if(logLine)
    {
        if(stylenum==8) //highlight msg
        {
            this->logLine("\0148\014<"+styled.section(':',0,0)+">"+styled.section(':',1), time);
        }
        else
        {
            this->logLine(QString("\014%1\014%2").arg(stylenum).arg(styled), time);
        }
    }
}

//check need of highlight in msg
bool IrcTabItem::highlightNeeded(const QString &msg)
{
    int pos = msg.indexOf(getNickName());
    if(pos==-1) return false;
    bool before = true;
    bool after = false;
    if(pos) before = dxutils::isDelimiter(msg[pos-1].toLatin1());
    if(pos+getNickName().length() == msg.length()) after = true;
    if(pos+getNickName().length() < msg.length()) after = dxutils::isDelimiter(msg[pos+getNickName().length()].toLatin1());
    return before && after;
}

//checking away in channel
void IrcTabItem::checkAway()
{
    if(m_type == CHANNEL && m_engine->getConnected() && m_numberUsers < preferences.m_maxAway)
    {
        m_engine->addIgnoreWho(m_name);
    }
}

//Apply new theme icon, usefull for nick icon theme change
void IrcTabItem::recreateNickIcons()
{
    m_model.updateIconTheme();
}

//return names
QStringList IrcTabItem::getNicks()
{
    QStringList names;
    for(int i=0; i<m_sortModel.rowCount(); ++i)
        names.append(m_sortModel.index(i,0).data().toString());
    return names;
}

bool IrcTabItem::hasNick(const QString& nick)
{
    return m_model.hasItem(nick);
}

void IrcTabItem::showLog(const QString& path)
{
    QFile file(path);
    if(file.exists())
    {
        if(!file.open(QIODevice::ReadOnly | QIODevice::Text))
        {
            appendStyledText(_("Unable to open file: %1").arg(path), 4, false, false, false);
            return;
        }
        clearChat();
        appendStyledText(_("Log from %1:").arg(path.section('/',-1)), 7, false, false, false);
        while(!file.atEnd())
        {
            m_text->append("<div>"+dxutils::parseLogline(QString::fromUtf8(file.readLine()))+"</div>");
        }
        file.close();
    }
    else
    {
        appendStyledText(_("'%1' path doesn't exist").arg(path), 4, false, false, false);
    }
}

//Check is this tab right for server message (e.g. not for IRC_SERVERREPLY)
bool IrcTabItem::isRightForServerMsg()
{
    int indexOfThis = m_tabbook->indexOf(this);
    //Check if current tab is in server targets
    IrcTabItem *current = qobject_cast<IrcTabItem*>(m_tabbook->currentWidget());
    if(current)
    {
        if(current->isSameServer(m_engine))
        {
            return indexOfThis == m_tabbook->currentIndex();
        }
    }
    else
    {
        for(int i=0; i<m_tabbook->count(); i++)
        {
            IrcTabItem *item = qobject_cast<IrcTabItem*>(m_tabbook->widget(i));
            if(item)
            {
                if(item->isSameServer(m_engine))
                {
                    if(i == indexOfThis) return true;
                    else return false;
                }
            }
        }
    }
    return false;
}

bool IrcTabItem::isCommandIgnored(const QString &command)
{
    if(preferences.m_ignoreCommandsList.contains(command)) return true;
    return false;
}

void IrcTabItem::addUser(const QString& user, UserMode mode)
{
    if(!m_model.hasItem(user))
    {
        m_model.addItem(user, mode);
        m_numberUsers++;
    }
}

void IrcTabItem::removeUser(const QString& user)
{
    if(m_model.hasItem(user))
    {
        m_model.removeItem(user);
        m_numberUsers--;
    }
}

void IrcTabItem::changeNickUser(const QString& nick, const QString& newnick)
{
    if(m_model.hasItem(nick))
    {
        m_model.updateItem(nick, newnick, m_engine->getNickInfo(newnick));
    }
}

void IrcTabItem::setUserMode(const QString& nick, UserMode mode)
{
    if(m_model.hasItem(nick))
    {
        m_model.updateItem(nick, mode);
    }
}

UserMode IrcTabItem::getUserMode(const char mode)
{
    if(mode == m_engine->getAdminPrefix())
        return ADMIN;
    if(mode == m_engine->getOwnerPrefix())
        return OWNER;
    if(mode == m_engine->getOpPrefix())
        return OP;
    if(mode == m_engine->getHalfopPrefix())
        return HALFOP;
    if(mode == m_engine->getVoicePrefix())
        return VOICE;
    return NONE;
}

void IrcTabItem::onAway()
{
    if(m_numberUsers < preferences.m_maxAway)
    {
        QString nick;
        for(int i = 0; i < m_model.rowCount(); ++i)
        {
            nick = m_model.getItemNick(i);
            m_model.updateAway(nick, m_engine->getNickInfo(nick).away);
        }
    }
    else
    {
        for(int i = 0; i < m_model.rowCount(); ++i)
        {
            m_model.updateAway(m_model.getItemNick(i), false);
        }
    }
}

void IrcTabItem::onBan(const QString &banmask, const bool &sign, const QString &sender, const QString &time)
{
    if(sign)
    {
        QStringList nicks = m_engine->getBannedNick(banmask);
        QString myNick = getNickName();
        foreach(QString nick, nicks)
        {
            for(int i = 0; i < m_model.rowCount(); ++i)
            {
                if(nick == m_model.getItemNick(i))
                {
                    if(m_model.getItemNick(i) == myNick) appendIrcStyledText(_("You was banned by %1").arg(sender), 1, time, false, false);
                    else
                    {
                        if(!isCommandIgnored("ban") && !m_engine->isUserIgnored(m_model.getItemNick(i), m_name)) appendIrcStyledText(_("%1 was banned by %2").arg(m_model.getItemNick(i)).arg(sender), 1, time, false, false);
                        //RemoveUser(users->getItemText(i));
                    }
                }
            }
        }
    }
}

bool IrcTabItem::isChannel(const QString &text)
{
    if(text.length()) return m_engine->getChanTypes().contains(text[0]);
    return false;
}

void IrcTabItem::completeHelpCommand()
{
    QString line = m_commandline->text();
    for(int i = 0; i < dxUtils.commandsNo(); i++)
    {
        if(line.section(' ',1).compare(dxUtils.commandsAt(i), Qt::CaseInsensitive) == 0)
        {
            if((i+1) < dxUtils.commandsNo()) m_commandline->setText("/help "+dxUtils.commandsAt(++i));
            else m_commandline->setText("/help "+dxUtils.commandsAt(0));
            break;
        }
        else if(line.section(' ',1).compare(dxUtils.commandsAt(i).left(line.section(' ',1).length()), Qt::CaseInsensitive) == 0)
        {
            m_commandline->setText("/help "+dxUtils.commandsAt(i));
            break;
        }
    }
}

//Load lines from logs
void IrcTabItem::loadLogLines()
{
    if(m_type != CHANNEL && m_type != QUERY)
        return;
    if(!m_engine)
        return;
    if(!preferences.m_loadLog)
        return;
    clearChat();
    appendIrcStyledText(_("Loading log..."), 11, "", false, false);
    QString logPath = preferences.m_logPath+"/"+m_engine->getNetworkName()+"/"+m_name;
    QApplication::setOverrideCursor(Qt::WaitCursor);
    QDir dir(logPath);
    dir.setFilter(QDir::Files | QDir::Readable);
    dir.setSorting(QDir::Name);
    QStringList files;
    QFileInfoList list = dir.entryInfoList();
    foreach(QFileInfo fileInfo, list)
    {
        if(dxutils::isRightLogFile(fileInfo.absoluteFilePath(), fileInfo.fileName()))
        {
            files << fileInfo.fileName();
        }
    }
    qSort(files.begin(), files.end(), caseInsensitiveLessThan);
    QStringList temp, result;
    for(int i=files.size()-1; i>-1; i--)
    {
        temp.clear();
        std::ifstream file(QString(logPath+"/"+files[i]).toUtf8().constData());
        std::string line;
        while(getline(file, line))
            temp.append(dxutils::normalizeLogLine(files[i], line));
        file.close();
        int nlines = preferences.m_loadLines-result.size();
        if(nlines>0)
        {
            for(int j=temp.size()-1; j>-1&&nlines>0; j--, nlines--)
                result.prepend(temp[j]);
        }
        else break;
    }
    clearChat();
    foreach(QString line, result)
    {
        m_text->append("<div>"+dxutils::parseLogline(line, true)+"</div>");
    }
    QApplication::restoreOverrideCursor();
}

//List log files for tab
void IrcTabItem::listLogFiles()
{
    QString logPath = preferences.m_logPath+"/"+m_engine->getNetworkName()+"/"+m_name;
    QApplication::setOverrideCursor(Qt::WaitCursor);
    QDir dir(logPath);
    dir.setFilter(QDir::Files | QDir::Readable);
    dir.setSorting(QDir::Name);
    QStringList files;
    QFileInfoList list = dir.entryInfoList();
    foreach(QFileInfo fileInfo, list)
    {
        if(dxutils::isRightLogFile(fileInfo.absoluteFilePath(), fileInfo.fileName()))
        {
            files << fileInfo.fileName();
        }
    }
    qSort(files.begin(), files.end(), caseInsensitiveLessThan);
    QApplication::restoreOverrideCursor();
    if(!files.size())
    {
        appendIrcStyledText(_("No log files for %1-%2.").arg(m_engine->getNetworkName()).arg(m_name), 4, "", false, false);
        return;
    }
    appendIrcStyledText(_("Log files:"), 7, "", false, false);
    QString lastYear = dxutils::getParam(files[0], 1, false, '-');
    int lastMonth = dxutils::getParam(files[0], 2, false, '-').toInt();
    appendIrcStyledText(QString("%1:").arg(lastYear), 5, "", false, false);
    appendIrcStyledText(QString("%1:").arg(dxutils::getMonth(lastMonth)), 5, "", false, false);
    QString items = "";
    for(int i=0; i<files.size(); i++)
    {
        if(lastYear!=dxutils::getParam(files[i], 1, false, '-'))
        {
            lastYear = dxutils::getParam(files[i], 1, false, '-');
            lastMonth = dxutils::getParam(files[i], 2, false, '-').toInt();
            appendIrcText(items.left(items.lastIndexOf(',')), "", false, false);
            appendIrcStyledText(QString("%1:").arg(lastYear), 5, "", false, false);
            appendIrcStyledText(QString("%1:").arg(dxutils::getMonth(lastMonth)), 5, "", false, false);
            items.clear();
            items.append(dxutils::getParam(files[i], 3, false, '-')+", ");
        }
        else if(lastMonth!=dxutils::getParam(files[i], 2, false, '-').toInt())
        {
            lastMonth = dxutils::getParam(files[i], 2, false, '-').toInt();
            appendIrcText(items.left(items.lastIndexOf(',')), "", false, false);
            appendIrcStyledText(QString("%1:").arg(dxutils::getMonth(lastMonth)), 5, "", false, false);
            items.clear();
            items.append(dxutils::getParam(files[i], 3, false, '-')+", ");
        }
        else items.append(dxutils::getParam(files[i], 3, false, '-')+", ");
    }
    if(!items.isEmpty()) appendIrcText(items.left(items.lastIndexOf(',')), "", false, false);
}

QStringList IrcTabItem::cutText(QString text, int len)
{
    register int previous = 0;
    register int s = -1;
    register int lw = 0;
    QStringList texts;
    for(int i = 0; i<text.length(); i++)
    {
        if(text[i] == ' ') s = i;
        if(lw>len)
        {
            if(s!=-1)
            {
                texts.append(text.mid(previous, s));
                i = s+1;
                s = -1;
                previous = i;
            }
            else
            {
                texts.append(text.mid(previous, lw));
                previous = previous+lw;
            }
            lw = 0;
        }
        lw++;
    }
    if(previous<text.length()) texts.append(text.right(text.length()-previous));
    return texts;
}

bool IrcTabItem::showHelp(QString command)
{
    if(dxUtils.isScriptCommand(command))
    {
        appendIrcStyledText(dxUtils.getHelpText(command), 3, "", false, false);
        return true;
    }
    if(command == "admin")
    {
        appendIrcStyledText(_("ADMIN [server], finds information about administrator for current server or [server]."), 3, "", false, false);
        return true;
    }
    if(command == "away")
    {
        appendIrcStyledText(_("AWAY [message], sets away status."), 3, "", false, false);
        return true;
    }
    if(command == "banlist")
    {
        appendIrcStyledText(_("BANLIST <channel>, shows banlist for channel."), 3, "", false, false);
        return true;
    }
    if(command == "boats")
    {
        appendIrcStyledText(_("BOATS <nick>, invites someone to a boats game."), 3, "", false, false);
        return true;
    }
    if(command == "commands")
    {
        appendIrcStyledText(_("COMMANDS, shows available commands"), 3, "", false, false);
        return true;
    }
    if(command == "connect")
    {
        appendIrcStyledText(_("CONNECT <server> [port] [nick] [password] [realname] [channels], connects for given server."), 3, "", false, false);
        return true;
    }
    if(command == "ctcp")
    {
        appendIrcStyledText(_("CTCP <nick> <message>, sends a CTCP message to a user."), 3, "", false, false);
        return true;
    }
    if(command == "cycle")
    {
        appendIrcStyledText(_("CYCLE <channel> [message], leaves and join channel."), 3, "", false, false);
        return true;
    }
    if(command == "dcc")
    {
        appendIrcStyledText(_("DCC chat <nick>, starts DCC chat."), 3, "", false, false);
        appendIrcStyledText(_("DCC send <nick> <filename>, sends file over DCC."), 3, "", false, false);
        appendIrcStyledText(_("DCC psend <nick> <filename>, sends file passive over DCC."), 3, "", false, false);
        appendIrcStyledText(_("More information about passive DCC on http://en.wikipedia.org/wiki/Direct_Client-to-Client#Passive_DCC"), 3, "", false, false);
        return true;
    }
    if(command == "deop")
    {
        appendIrcStyledText(_("DEOP <channel> <nicks>, removes operator status from one or more nicks."), 3, "", false, false);
        return true;
    }
    if(command == "devoice")
    {
        appendIrcStyledText(_("DEVOICE <channel> <nicks>, removes voice from one or more nicks."), 3, "", false, false);
        return true;
    }
    if(command == "disconnect")
    {
        appendIrcStyledText(_("DISCONNECT [reason], leaves server."), 3, "", false, false);
        return true;
    }
    if(command == "exec")
    {
        appendIrcStyledText(_("EXEC [-o|-c] <command>, executes command, -o sends output to channel/query, -c closes running command."), 3, "", false, false);
        return true;
    }
    if(command == "help")
    {
        appendIrcStyledText(_("HELP <command>, shows help for command."), 3, "", false, false);
        return true;
    }
    if(command == "ignore")
    {
        appendIrcStyledText(_("IGNORE list, shows list ignored commands and users."), 3, "", false, false);
        appendIrcStyledText(_("IGNORE addcmd <command>, adds command to ignored commands."), 3, "", false, false);
        appendIrcStyledText(_("IGNORE rmcmd <command>, removes command from ignored commands."), 3, "", false, false);
        appendIrcStyledText(_("IGNORE addusr <user> [channel] [server], adds user to ignored users."), 3, "", false, false);
        appendIrcStyledText(_("IGNORE rmusr <user>, removes user from ignored users."), 3, "", false, false);
        return true;
    }
    if(command == "invite")
    {
        appendIrcStyledText(_("INVITE <nick> <channel>, invites someone to a channel."), 3, "", false, false);
        return true;
    }
    if(command == "join")
    {
        appendIrcStyledText(_("JOIN <channel>, joins a channel."), 3, "", false, false);
        return true;
    }
    if(command == "kick")
    {
        appendIrcStyledText(_("KICK <channel> <nick>, kicks a user from a channel."), 3, "", false, false);
        return true;
    }
    if(command == "kill")
    {
        appendIrcStyledText(_("KILL <user> [reason], kills a user from the network."), 3, "", false, false);
        return true;
    }
    if(command == "list")
    {
        appendIrcStyledText(_("LIST [channel], lists channels and their topics."), 3, "", false, false);
        return true;
    }
    if(command == "log")
    {
        appendIrcStyledText(_("LOG list, shows list of log files for current tab"), 3, "", false, false);
        appendIrcStyledText(_("LOG show [date], shows last log file for current tab, if date is empty"), 3, "", false, false);
        appendIrcStyledText(_("else file for given date"), 3, "", false, false);
        appendIrcStyledText(_("date example: 2011-01-31"), 3, "", false, false);
        return true;
    }
#ifdef HAVE_LUA
    if(command == "lua")
    {
        appendIrcStyledText(_("LUA help, shows help for lua scripting."), 3, "", false, false);
        appendIrcStyledText(_("LUA load <path>, loads script."), 3, "", false, false);
        appendIrcStyledText(_("Example: /lua load /home/dvx/test.lua"), 3, "", false, false);
        appendIrcStyledText(_("LUA unload <name>, unloads script."), 3, "", false, false);
        appendIrcStyledText(_("Example: /lua unload test"), 3, "", false, false);
        appendIrcStyledText(_("LUA list, shows list of loaded scripts"), 3, "", false, false);
        return true;
    }
#else
    if(command == "lua")
    {
        appendIrcStyledText(_("dxirc is compiled without support for Lua scripting"), 4, "", false, false);
        return true;
    }
#endif
    if(command == "me")
    {
        appendIrcStyledText(_("ME <to> <message>, sends the action."), 3, "", false, false);
        return true;
    }
    if(command == "mode")
    {
        appendIrcStyledText(_("MODE <channel> <modes>, sets modes for a channel."), 3, "", false, false);
        return true;
    }
    if(command == "msg")
    {
        appendIrcStyledText(_("MSG <nick/channel> <message>, sends a normal message."), 3, "", false, false);
        return true;
    }
    if(command == "names")
    {
        appendIrcStyledText(_("NAMES <channel>, for nicks on a channel."), 3, "", false, false);
        return true;
    }
    if(command == "nick")
    {
        appendIrcStyledText(_("NICK <nick>, changes nick."), 3, "", false, false);
        return true;
    }
    if(command == "notice")
    {
        appendIrcStyledText(_("NOTICE <nick/channel> <message>, sends a notice."), 3, "", false, false);
        return true;
    }
    if(command == "op")
    {
        appendIrcStyledText(_("OP <channel> <nicks>, gives operator status for one or more nicks."), 3, "", false, false);
        return true;
    }
    if(command == "oper")
    {
        appendIrcStyledText(_("OPER <login> <password>, oper up."), 3, "", false, false);
        return true;
    }
    if(command == "part")
    {
        appendIrcStyledText(_("PART <channel> [reason], leaves channel."), 3, "", false, false);
        return true;
    }
    if(command == "query")
    {
        appendIrcStyledText(_("QUERY <nick>, opens query with nick."), 3, "", false, false);
        return true;
    }
    if(command == "quit")
    {
        appendIrcStyledText(_("QUIT, closes application."), 3, "", false, false);
        return true;
    }
    if(command == "quote")
    {
        appendIrcStyledText(_("QUOTE [text], sends text to server."), 3, "", false, false);
        return true;
    }
    if(command == "say")
    {
        appendIrcStyledText(_("SAY [text], sends text to current tab."), 3, "", false, false);
        return true;
    }
    if(command == "stats")
    {
        appendIrcStyledText(_("STATS <type>, shows some irc server usage statistics. Available types vary slightly per server; some common ones are:"), 3, "", false, false);
        appendIrcStyledText(_("c - shows C and N lines for a given server.  These are the names of the servers that are allowed to connect."), 3, "", false, false);
        appendIrcStyledText(_("h - shows H and L lines for a given server (Hubs and Leaves)."), 3, "", false, false);
        appendIrcStyledText(_("k - show K lines for a server.  This shows who is not allowed to connect and possibly at what time they are not allowed to connect."), 3, "", false, false);
        appendIrcStyledText(_("i - shows I lines. This is who CAN connect to a server."), 3, "", false, false);
        appendIrcStyledText(_("l - shows information about amount of information passed to servers and users."), 3, "", false, false);
        appendIrcStyledText(_("m - shows a count for the number of times the various commands have been used since the server was booted."), 3, "", false, false);
        appendIrcStyledText(_("o - shows the list of authorized operators on the server."), 3, "", false, false);
        appendIrcStyledText(_("p - shows online operators and their idle times."), 3, "", false, false);
        appendIrcStyledText(_("u - shows the uptime for a server."), 3, "", false, false);
        appendIrcStyledText(_("y - shows Y lines, which lists the various connection classes for a given server."), 3, "", false, false);

        return true;
    }
    if(command == "tetris")
    {
        appendIrcStyledText(_("TETRIS, start small easteregg."), 3, "", false, false);
        appendIrcStyledText(_("Keys for playing:"), 3, "", false, false);
        appendIrcStyledText(_("n .. new game"), 3, "", false, false);
        appendIrcStyledText(_("p .. pause game"), 3, "", false, false);
        appendIrcStyledText(_("i .. rotate piece"), 3, "", false, false);
        appendIrcStyledText(_("l .. move piece right"), 3, "", false, false);
        appendIrcStyledText(_("k .. drop piece"), 3, "", false, false);
        appendIrcStyledText(_("j .. move piece left"), 3, "", false, false);
        return true;
    }
    if(command == "time")
    {
        appendIrcStyledText(_("TIME, displays the time of day, local to server."), 3, "", false, false);
        return true;
    }
    if(command == "topic")
    {
        appendIrcStyledText(_("TOPIC [topic], sets or shows topic."), 3, "", false, false);
        return true;
    }
    if(command == "voice")
    {
        appendIrcStyledText(_("VOICE <channel> <nicks>, gives voice for one or more nicks."), 3, "", false, false);
        return true;
    }
    if(command == "wallops")
    {
        appendIrcStyledText(_("WALLOPS <message>, sends wallop message."), 3, "", false, false);
        return true;
    }
    if(command == "who")
    {
        appendIrcStyledText(_("WHO <mask> [o], searchs for mask on network, if o is supplied, only search for opers."), 3, "", false, false);
        return true;
    }
    if(command == "whoami")
    {
        appendIrcStyledText(_("WHOAMI, whois about you."), 3, "", false, false);
        return true;
    }
    if(command == "whois")
    {
        appendIrcStyledText(_("WHOIS <nick>, whois nick."), 3, "", false, false);
        return true;
    }
    if(command == "whowas")
    {
        appendIrcStyledText(_("WHOWAS <nick>, whowas nick."), 3, "", false, false);
        return true;
    }
    if(!dxUtils.getAlias(command[0] == '/' ? command:"/"+command).isEmpty())
    {
        appendIrcStyledText(QString("%1: %2").arg(command.toUpper()).arg(dxUtils.getAlias(command[0] == '/' ? command:"/"+command)), 3, "", false, false);
        return true;
    }
    if(command.isEmpty()) appendIrcStyledText(_("Command is empty, type /commands for available commands"), 4, "", false, false);
    else appendIrcStyledText(_("Unknown command '%1', type /commands for available commands").arg(command), 4, "", false, false);
    return false;
}

bool IrcTabItem::processCommand(const QString& commandtext)
{
    QString command = (commandtext[0] == '/' ? commandtext.section('/',1).section(' ',0,0).toLower() : "");
    if(m_type == OTHER)
    {
        if(dxUtils.isScriptCommand(command))
        {
            return commandScript(commandtext);
        }
        if(command == "commands")
        {
            return commandCommands();
        }
        if(command == "egg")
        {
            return commandEgg();
        }
        if(command == "help")
        {
            return commandHelp(commandtext);
        }
        if(command == "tetris")
        {
            return commandTetris();
        }
        return true;
    }
    if(commandtext[0] == '/')
    {
        if(dxUtils.isScriptCommand(command))
        {
            return commandScript(commandtext);
        }
        if(command == "admin")
        {
            return commandAdmin(commandtext);
        }
        if(command == "away")
        {
            return commandAway(commandtext);
        }
        if(command == "banlist")
        {
            return commandBanlist(commandtext);
        }
        if(command == "boats")
        {
            return commandBoats(commandtext);
        }
        if(command == "connect")
        {
            return commandConnect(commandtext);
        }
        if(command == "commands")
        {
            return commandCommands();
        }
        if(command == "ctcp")
        {
            return commandCtcp(commandtext);
        }
        if(command == "cycle")
        {
            return commandCycle(commandtext);
        }
        if(command == "dcc")
        {
            return commandDcc(commandtext);
        }
        if(command == "deop")
        {
            return commandDeop(commandtext);
        }
        if(command == "devoice")
        {
            return commandDevoice(commandtext);
        }
        if(command == "disconnect")
        {
            return commandDisconnect(commandtext);
        }
        if(command == "dxirc")
        {
            return commandDxirc();
        }
        if(command == "egg")
        {
            return commandEgg();
        }
        if(command == "exec")
        {
            return commandExec(commandtext);
        }
        if(command == "help")
        {
            return commandHelp(commandtext);
        }
        if(command == "ignore")
        {
            return commandIgnore(commandtext);
        }
        if(command == "invite")
        {
            return commandInvite(commandtext);
        }
        if(command == "join")
        {
            return commandJoin(commandtext);
        }
        if(command == "kick")
        {
            return commandKick(commandtext);
        }
        if(command == "kill")
        {
            return commandKill(commandtext);
        }
        if(command == "list")
        {
            return commandList(commandtext);
        }
        if(command == "log")
        {
            return commandLog(commandtext);
        }
        if(command == "lua")
        {
            return commandLua(commandtext);
        }
        if(command == "me")
        {
            return commandMe(commandtext);
        }
        if(command == "mode")
        {
            return commandMode(commandtext);
        }
        if(command == "msg")
        {
            return commandMsg(commandtext);
        }
        if(command == "names")
        {
            return commandNames(commandtext);
        }
        if(command == "nick")
        {
            return commandNick(commandtext);
        }
        if(command == "notice")
        {
            return commandNotice(commandtext);
        }
        if(command == "op")
        {
            return commandOp(commandtext);
        }
        if(command == "oper")
        {
            return commandOper(commandtext);
        }
        if(command == "part")
        {
            return commandPart(commandtext);
        }
        if(command == "query")
        {
            return commandQuery(commandtext);
        }
        if(command == "quit")
        {
            return commandQuit();
        }
        if(command == "quote")
        {
            return commandQuote(commandtext);
        }
        if(command == "say")
        {
            return commandSay(commandtext);
        }
        if(command == "stats")
        {
            return commandStats(commandtext);
        }
        if(command == "tetris")
        {
            return commandTetris();
        }
        if(command == "time")
        {
            return commandTime();
        }
        if(command == "topic")
        {
            return commandTopic(commandtext);
        }
        if(command == "voice")
        {
            return commandVoice(commandtext);
        }
        if(command == "wallops")
        {
            return commandWallops(commandtext);
        }
        if(command == "who")
        {
            return commandWho(commandtext);
        }
        if(command == "whoami")
        {
            return commandWhoami();
        }
        if(command == "whois")
        {
            return commandWhois(commandtext);
        }
        if(command == "whowas")
        {
            return commandWhowas(commandtext);
        }
        appendIrcStyledText(_("Unknown command '%1', type /commands for available commands").arg(command), 4, "", false, false);
        return false;
    }
    else
    {
        if(command.isEmpty() && m_type != SERVER && !commandtext.isEmpty() && m_engine->getConnected())
        {
            if(commandtext.length() > m_maxLen-10-m_name.length())
            {
                QStringList messages = cutText(commandtext, m_maxLen-10-m_name.length());
                bool result = true;
                for(int i=0; i<messages.size(); i++)
                {
                    appendMyMsg(messages[i], "");
                    emit ircPrivmsg(m_engine, getNickName(), m_name, messages[i]);
                    result = m_engine->sendMsg(m_name, messages[i]) &result;
                }
                return result;
            }
            else
            {
                appendMyMsg(commandtext, "");
                emit ircPrivmsg(m_engine, getNickName(), m_name, commandtext);
                return m_engine->sendMsg(m_name, commandtext);
            }
        }
        if(!m_engine->getConnected())
        {
            appendIrcStyledText(_("You aren't connected"), 4, "", false, false);
            emit connectDialog();
            return true;
        }
        return false;
    }
    return false;
}

void IrcTabItem::slotIrcConnect(QString msg)
{
    appendIrcStyledText(msg, 3, "", false, false);
}

void IrcTabItem::slotIrcDisconnect(QString msg)
{
    appendIrcStyledText(msg, 4, "", false, false);
    if(isRightForServerMsg())
    {
        emit highlight(m_type);
    }
    if(m_type == CHANNEL)
    {
        m_model.clear();
        m_iamOp = false;
    }
}

void IrcTabItem::slotIrcServerReply(QString msg)
{
    if(m_type == SERVER)
    {
        appendIrcText(msg, "");
        emit newmsg(m_type);
    }
    else
    {
        if(isRightForServerMsg())
        {
            appendIrcText(msg, "", false, false);
            emit newmsg(m_type);
        }
    }
}

void IrcTabItem::slotIrcServerError(QString msg)
{
    if(isRightForServerMsg())
    {
        appendIrcStyledText(msg, 4, "", false, false);
        emit newmsg(m_type);
    }
}

void IrcTabItem::slotIrc372(QString msg)
{
    if(m_type == SERVER)
    {
        appendIrcText(msg, "");
        emit newmsg(m_type);
    }
    else
    {
        if(isRightForServerMsg())
        {
            appendIrcText(msg, "", false, false);
            emit newmsg(m_type);
        }
    }
}

void IrcTabItem::slotIrcNotice(QString to, QString msg)
{
    if(isRightForServerMsg())
    {
        if(to == getNickName() && !isCommandIgnored("notice"))
        {
            appendIrcStyledText(_("NOTICE for you: %1").arg(msg), 3, "");
            emit newmsg(m_type);
        }
        else if(!isCommandIgnored("notice"))
        {
            appendIrcStyledText(_("%1's NOTICE: %2").arg(to).arg(msg), 3, "");
            emit newmsg(m_type);
        }
    }
}

void IrcTabItem::slotIrcChnotice(QString nick, QString to, QString msg)
{
    if(!isCommandIgnored("notice"))
    {
        bool tabExist = false;
        IrcTabItem *item = NULL;
        for(int i = 0; i<m_tabbook->count(); i++)
        {
            item = qobject_cast<IrcTabItem*>(m_tabbook->widget(i));
            if(item && item->isSameServer(m_engine))
            {
                if((to.compare(m_tabbook->tabText(i), Qt::CaseInsensitive) == 0 && item->getType() == CHANNEL)
                    || (nick == m_tabbook->tabText(i) && item->getType() == QUERY && to == item->getNickName()))
                {
                    tabExist = true;
                    break;
                }
            }
        }
        if(tabExist)
        {
            if((QString::compare(to, m_name, Qt::CaseInsensitive) == 0 && m_type == CHANNEL) || (nick == m_name && m_type == QUERY && to == getNickName()))
            {
                bool needHighlight = false;
                if(msg.contains(getNickName()))
                    needHighlight = highlightNeeded(msg);
                appendIrcStyledText(_("%1's NOTICE: %2").arg(nick).arg(msg), 2, "");
                if(needHighlight) emit highlight(m_type);
                else emit newmsg(m_type);
            }
        }
        else
        {
            if(isRightForServerMsg())
            {
                appendIrcStyledText(_("%1's NOTICE: %2").arg(nick).arg(msg), 3, "");
            }
        }
    }
}

void IrcTabItem::slotIrcPrivmsg(QString nick, QString to, QString msg)
{
    if((QString::compare(to, m_name, Qt::CaseInsensitive) == 0 && m_type == CHANNEL) || (nick == m_name && m_type == QUERY && to == getNickName()))
    {
        bool needHighlight = false;
        if(msg.contains(getNickName()))
            needHighlight = highlightNeeded(msg);
        if(needHighlight) appendIrcStyledText(nick+": "+msg, 8, "");
        else appendIrcNickText(nick, msg, "");
        if(needHighlight) emit highlight(m_type);
        else emit newmsg(m_type);
    }
}

void IrcTabItem::slotIrcAction(QString nick, QString to, QString msg)
{
    if((QString::compare(to, m_name, Qt::CaseInsensitive) == 0 && m_type == CHANNEL) || (nick == m_name && m_type == QUERY && to == getNickName()))
    {
        if(!isCommandIgnored("me"))
        {
            bool needHighlight = false;
            if(msg.contains(getNickName()))
                needHighlight = highlightNeeded(msg);
            appendIrcStyledText(nick+" "+msg, 2, "");
            if(needHighlight) emit highlight(m_type);
            else emit newmsg(m_type);
        }
    }
}

void IrcTabItem::slotIrcError(QString msg)
{
    appendIrcStyledText(msg, 4, "", false, false);
}

void IrcTabItem::slotIrcReconnect(QString msg)
{
    appendIrcStyledText(msg, 4, "", false, false);
    if(isRightForServerMsg())
    {
        emit highlight(m_type);
    }
    if(m_type == CHANNEL)
    {
        m_model.clear();
        m_iamOp = false;
    }
}

void IrcTabItem::slotIrcReconnectAll(QString msg)
{
    appendIrcStyledText(msg, 3, "", false, false);
    if(m_type == CHANNEL)
    {
        m_model.clear();
        m_iamOp = false;
    }
}

void IrcTabItem::slotIrcCtcpReply(QString nick, QString msg)
{
    if(isRightForServerMsg())
    {
        if(!isCommandIgnored("ctcp"))
        {
            appendIrcStyledText(_("CTCP %1 reply from %2: %3").arg(dxutils::getParam(msg, 1, false)).arg(nick).arg(dxutils::getParam(msg, 2, true)), 2, "", false, false);
            emit newmsg(m_type);
        }
    }
}

void IrcTabItem::slotIrcCtcpRequest(QString nick, QString request)
{
    if(isRightForServerMsg())
    {
        if(!isCommandIgnored("ctcp"))
        {
            appendIrcStyledText(_("CTCP %1 request from %2").arg(request).arg(nick), 2, "", false, false);
            emit newmsg(m_type);
        }
    }
}

void IrcTabItem::slotIrcNickInfo(NickInfo info)
{
    m_model.updateItem(info.nick, info);
}

void IrcTabItem::slotIrcJoin(QString nick, QString channel)
{
    if(!QString::compare(channel, m_name, Qt::CaseInsensitive) && nick != getNickName())
    {
        if(!isCommandIgnored("join") && !m_engine->isUserIgnored(nick, m_name)) appendIrcStyledText(_("%1 has joined to %2").arg(nick).arg(channel), 1, "");
        addUser(nick, NONE);
    }
}

void IrcTabItem::slotIrc353(QString channel, QString nicks, bool list)
{
    QString myNick = getNickName();
    if(nicks.right(1) != " ") nicks.append(" ");
    if(QString::compare(channel, m_name, Qt::CaseInsensitive) == 0)
    {
        if(list && !isCommandIgnored("numeric")) appendIrcText(_("Users on %1: %2").arg(channel).arg(nicks), "", false, false);
        QStringList nicklist = nicks.split(' ', QString::SkipEmptyParts);
        foreach(QString nick, nicklist)
        {
            UserMode mode = getUserMode(nick[0].toLatin1());
            if(mode != NONE) nick = nick.section(nick[0],1);
            addUser(nick, mode);
            if(mode == OP && nick == myNick) m_iamOp = true;
        }
    }
    else
    {
        if(list && !isCommandIgnored("numeric"))
        {
            bool channelOn = false;
            IrcTabItem *item = NULL;
            for(int i = 0; i<m_tabbook->count(); i++)
            {
                item = qobject_cast<IrcTabItem*>(m_tabbook->widget(i));
                if(item)
                {
                    if(item->isSameServer(m_engine)
                            && !QString::compare(channel, m_name, Qt::CaseInsensitive))
                    {
                        channelOn = true;
                        break;
                    }
                }
            }
            if(!channelOn)
            {
                if(m_type == SERVER)
                {
                    appendIrcText(_("Users on %1: %2").arg(channel).arg(nicks), "", false, false);
                    emit newmsg(m_type);
                }
                else
                {
                    if(isRightForServerMsg())
                    {
                        appendIrcText(_("Users on %1: %2").arg(channel).arg(nicks), "", false, false);
                        emit newmsg(m_type);
                    }
                }
            }
        }
    }
}

void IrcTabItem::slotIrc366(QString channel, QString msg, bool list)
{
    if(QString::compare(channel, m_name, Qt::CaseInsensitive) == 0)
    {
        if(list && !isCommandIgnored("numeric")) appendIrcText(QString("%1: %2").arg(channel).arg(msg), "", false, false);
        else m_engine->addIgnoreWho(m_name);
    }
    else
    {
        if(list && !isCommandIgnored("numeric"))
        {
            bool channelOn = false;
            IrcTabItem *item = NULL;
            for(int i = 0; i<m_tabbook->count(); i++)
            {
                item = qobject_cast<IrcTabItem*>(m_tabbook->widget(i));
                if(item)
                {
                    if(item->isSameServer(m_engine)
                            && !QString::compare(channel, m_name, Qt::CaseInsensitive))
                    {
                        channelOn = true;
                        break;
                    }
                }
            }
            if(!channelOn)
            {
                if(m_type == SERVER)
                {
                    appendIrcText(QString("%1: %2").arg(channel).arg(msg), "", false, false);
                    emit newmsg(m_type);
                }
                else
                {
                    if(isRightForServerMsg())
                    {
                        appendIrcText(QString("%1: %2").arg(channel).arg(msg), "", false, false);
                        emit newmsg(m_type);
                    }
                }
            }
        }
    }
}

void IrcTabItem::slotIrcPart(QString nick, QString channel, QString reason)
{
    if(QString::compare(channel, m_name, Qt::CaseInsensitive) == 0)
    {
        if(reason.isEmpty() && !isCommandIgnored("part") && !m_engine->isUserIgnored(nick, m_name)) appendIrcStyledText(_("%1 has parted %2").arg(nick).arg(channel), 1, "");
        else if(!isCommandIgnored("part") && !m_engine->isUserIgnored(nick, m_name)) appendIrcStyledText(_("%1 has parted %2 (%3)").arg(nick).arg(channel).arg(reason), 1, "");
        removeUser(nick);
    }
}

void IrcTabItem::slotIrcQuit(QString nick, QString reason)
{
    if(m_type == CHANNEL && m_model.hasItem(nick))
    {
        if(reason.isEmpty())
        {
            if(!isCommandIgnored("quit") && !m_engine->isUserIgnored(nick, m_name)) appendIrcStyledText(_("%1 has quit").arg(nick), 1, "");
        }
        else
        {
            if(!isCommandIgnored("quit") && !m_engine->isUserIgnored(nick, m_name)) appendIrcStyledText(_("%1 has quit (%2)").arg(nick).arg(reason), 1, "");
        }
        removeUser(nick);
    }
    else if(m_type == QUERY && m_name == nick)
    {
        appendIrcStyledText(_("%1 has quit").arg(nick), 1, "", false, false);
    }
}

void IrcTabItem::slotIrcTopic(QString nick, QString channel, QString topic)
{
    if(QString::compare(channel, m_name, Qt::CaseInsensitive) == 0)
    {
        appendIrcText(_("%1 set new topic for %2: %3").arg(nick).arg(channel).arg(topic), "");
        m_topic = dxutils::stripColors(topic, true);
        m_topicline->setText(m_topic);
        m_topicline->setToolTip(dxutils::wrapText(m_topic,400));
        m_topicline->setCursorPosition(0);
    }
}

void IrcTabItem::slotIrcNick(QString nick, QString newnick)
{
    if(m_model.hasItem(nick))
    {
        if(newnick == getNickName() && !isCommandIgnored("nick")) appendIrcStyledText(_("You're now known as %1").arg(newnick), 1, "");
        else if(!isCommandIgnored("nick") && !m_engine->isUserIgnored(nick, m_name)) appendIrcStyledText(_("%1 is now known as %2").arg(nick).arg(newnick), 1, "");
        m_model.updateItem(nick, newnick, m_engine->getNickInfo(newnick));
    }
    if(m_type == QUERY && nick == m_name)
    {
        m_name = newnick;
        emit newname(m_name);
        stopLogging();
    }
}

void IrcTabItem::slotIrcMode(QString mode, QString who)
{
    if(isRightForServerMsg())
    {
        appendIrcStyledText(_("Mode change [%1] for %2").arg(mode).arg(who), 1, "", false, false);
        emit newmsg(m_type);
    }
}

void IrcTabItem::slotIrcUmode(QString moderator, QString channel, QString modes, QString args)
{
    if(QString::compare(channel, m_name, Qt::CaseInsensitive) == 0)
    {
        bool sign = false;
        int argsiter = 1;
        for(int i =0; i < modes.count(); i++) {
            switch(modes[i].toLatin1()) {
                case '+':
                    sign = true;
                    break;
                case '-':
                    sign = false;
                    break;
                case 'a': //admin
                {
                    QString nick = dxutils::getParam(args, argsiter, false);
                    setUserMode(nick, sign?ADMIN:NONE);
                    if(sign)
                    {
                        if(!isCommandIgnored("mode") && !m_engine->isUserIgnored(nick, m_name))
                        {
                            if(nick == getNickName()) appendIrcStyledText(_("%1 gave you admin").arg(moderator), 1, "", false, false);
                            else appendIrcStyledText(_("%1 gave %2 admin").arg(moderator).arg(nick), 1, "", false, false);
                        }
                    }
                    else
                    {
                        if(!isCommandIgnored("mode") && !m_engine->isUserIgnored(nick, m_name))
                        {
                            if(nick == getNickName()) appendIrcStyledText(_("%1 removed you admin").arg(moderator), 1, "", false, false);
                            else appendIrcStyledText(_("%1 removed %2 admin").arg(moderator).arg(nick), 1, "", false, false);
                        }
                    }
                    argsiter++;
                }break;
                case 'o': //op
                {
                    QString nick = dxutils::getParam(args, argsiter, false);
                    setUserMode(nick, sign?OP:NONE);
                    if(sign)
                    {
                        if(!isCommandIgnored("mode") && !m_engine->isUserIgnored(nick, m_name))
                        {
                            if(nick == getNickName()) appendIrcStyledText(_("%1 gave you op").arg(moderator), 1, "", false, false);
                            else appendIrcStyledText(_("%1 gave %2 op").arg(moderator).arg(nick), 1, "", false, false);
                        }
                    }
                    else
                    {
                        if(!isCommandIgnored("mode") && !m_engine->isUserIgnored(nick, m_name))
                        {
                            if(nick == getNickName()) appendIrcStyledText(_("%1 removed you op").arg(moderator), 1, "", false, false);
                            else appendIrcStyledText(_("%1 removed %2 op").arg(moderator).arg(nick), 1, "", false, false);
                        }
                    }
                    if(getNickName() == nick) sign ? m_iamOp = true : m_iamOp = false;
                    argsiter++;
                }break;
                case 'v': //voice
                {
                    QString nick = dxutils::getParam(args, argsiter, false);
                    setUserMode(nick, sign?VOICE:NONE);
                    if(sign)
                    {
                        if(!isCommandIgnored("mode") && !m_engine->isUserIgnored(nick, m_name))
                        {
                            if(nick == getNickName()) appendIrcStyledText(_("%1 gave you voice").arg(moderator), 1, "", false, false);
                            else appendIrcStyledText(_("%1 gave %2 voice").arg(moderator).arg(nick), 1, "", false, false);
                        }
                    }
                    else
                    {
                        if(!isCommandIgnored("mode") && !m_engine->isUserIgnored(nick, m_name))
                        {
                            if(nick == getNickName()) appendIrcStyledText(_("%1 removed you voice").arg(moderator), 1, "", false, false);
                            else appendIrcStyledText(_("%1 removed %2 voice").arg(moderator).arg(nick), 1, "", false, false);
                        }
                    }
                    argsiter++;
                }break;
                case 'h': //halfop
                {
                    QString nick = dxutils::getParam(args, argsiter, false);
                    setUserMode(nick, sign?HALFOP:NONE);
                    if(sign)
                    {
                        if(!isCommandIgnored("mode") && !m_engine->isUserIgnored(nick, m_name))
                        {
                            if(nick == getNickName()) appendIrcStyledText(_("%1 gave you halfop").arg(moderator), 1, "", false, false);
                            else appendIrcStyledText(_("%1 gave %2 halfop").arg(moderator).arg(nick), 1, "", false, false);
                        }
                    }
                    else
                    {
                        if(!isCommandIgnored("mode") && !m_engine->isUserIgnored(nick, m_name))
                        {
                            if(nick == getNickName()) appendIrcStyledText(_("%1 removed you halfop").arg(moderator), 1, "", false, false);
                            else appendIrcStyledText(_("%1 removed %2 halfop").arg(moderator).arg(nick), 1, "", false, false);
                        }
                    }
                    argsiter++;
                }break;
                case 'b': //ban
                {
                    QString banmask = dxutils::getParam(args, argsiter, false);
                    onBan(banmask, sign, moderator, "");
                    argsiter++;
                }break;
                case 't': //topic settable by channel operator
                {
                    sign ? m_editableTopic = false : m_editableTopic = true;
                }break;
                default:
                {
                    appendIrcStyledText(_("%1 set Mode: %2").arg(moderator).arg(modes+" "+args), 1, "", false, false);
                }
            }
        }
    }
}

void IrcTabItem::slotIrcChmode(QString channel, QString modes)
{
    if(QString::compare(channel, m_name, Qt::CaseInsensitive) == 0)
    {
        if(modes.contains('t')) m_editableTopic = false;
        appendIrcStyledText(_("Mode for %1: %2").arg(channel).arg(modes), 1, "", false, false);
    }
}

void IrcTabItem::slotIrc301(QString nick, QString msg)
{
    if(isCurrent() || m_name == nick)
    {
        if(!isCommandIgnored("away") && !m_engine->isUserIgnored(nick, m_name)) appendIrcStyledText(_("%1 is away: %2").arg(nick).arg(msg), 1, "");
    }
}

void IrcTabItem::slotIrc305(QString msg)
{
    if(m_model.hasItem(getNickName()))
    {
        appendIrcStyledText(msg, 1, "", false, false);
        m_model.updateAway(getNickName(), false);
    }
}

void IrcTabItem::slotIrc306(QString msg)
{
    if(m_model.hasItem(getNickName()))
    {
        appendIrcStyledText(msg, 1, "", false, false);
        m_model.updateAway(getNickName(), true);
    }
}

void IrcTabItem::slotIrc331(QString channel, QString msg)
{
    if(QString::compare(channel, m_name, Qt::CaseInsensitive) == 0)
    {
        appendIrcText(msg, "");
        m_topic = dxutils::stripColors(msg, true);
        m_topicline->setText(m_topic);
        m_topicline->setToolTip(m_topic);
        m_topicline->setCursorPosition(0);
    }
}

void IrcTabItem::slotIrc332(QString channel, QString msg)
{
    if(QString::compare(channel, m_name, Qt::CaseInsensitive) == 0)
    {
        appendIrcText(msg, "");
        m_topic = dxutils::stripColors(dxutils::getParam(msg, 2, true, ':').section(' ',1), true);
        m_topicline->setText(m_topic);
        m_topicline->setToolTip(dxutils::wrapText(m_topic,400));
        m_topicline->setCursorPosition(0);
    }
}

void IrcTabItem::slotIrc333(QString channel, QString msg)
{
    if(QString::compare(channel, m_name, Qt::CaseInsensitive) == 0)
    {
        appendIrcText(msg, "");
    }
}

void IrcTabItem::slotIrcKick(QString nick, QString to, QString channel, QString reason)
{
    if(QString::compare(channel, m_name, Qt::CaseInsensitive) == 0)
    {
        if(to != getNickName())
        {
            if(reason.isEmpty()) appendIrcStyledText(_("%1 was kicked from %2 by %3").arg(to).arg(channel).arg(nick), 1, "", false, false);
            else appendIrcStyledText(_("%1 was kicked from %2 by %3 (%4)").arg(to).arg(channel).arg(nick).arg(reason), 1, "", false, false);
            removeUser(to);
        }
    }
    if(to == getNickName() && isRightForServerMsg())
    {
        if(reason.isEmpty()) appendIrcStyledText(_("You were kicked from %1 by %2").arg(channel).arg(nick), 1, "", false, false);
        else appendIrcStyledText(_("You were kicked from %1 by %2 (%3)").arg(channel).arg(nick).arg(reason), 1, "", false, false);
        emit newmsg(m_type);
    }
}

void IrcTabItem::slotIrcAway(QString who)
{
    if(QString::compare(who, m_name, Qt::CaseInsensitive) == 0)
    {
        onAway();
        return;
    }
    //hack for &bitlbee
    if(QString::compare("&bitlbee", m_name, Qt::CaseInsensitive) == 0 && m_model.hasItem(who))
    {
        onAway();
        return;
    }
}

void IrcTabItem::slotIrcEndMotd()
{
    if(m_type == SERVER && !m_engine->getNetworkName().isEmpty())
    {
        m_name = m_engine->getNetworkName();
        emit newname(m_name);
    }
}

void IrcTabItem::slotPopupQuery()
{
    QModelIndex index = m_users->currentIndex();
    if(index.isValid())
    {
        emit ircQuery(m_engine, index.data().toString());
    }
}

void IrcTabItem::slotPopupWhois()
{
    QModelIndex index = m_users->currentIndex();
    if(index.isValid())
    {
        m_engine->sendWhois(index.data().toString());
    }
}

void IrcTabItem::slotPopupDccChat()
{
    QModelIndex index = m_users->currentIndex();
    if(index.isValid())
    {
        emit ircDccServer(m_engine, index.data().toString());
    }
}

void IrcTabItem::slotPopupSendFile()
{
    QModelIndex index = m_users->currentIndex();
    if(index.isValid())
    {
        DccSendDialog *dialog = new DccSendDialog(index.data().toString(), this);
        if(dialog->exec() == QDialog::Accepted)
        {
            if(QFile::exists(dialog->file()))
            {
                dialog->passive() ? emit ircDccPout(m_engine, index.data().toString(), dialog->file())
                    : emit ircDccOut(m_engine, index.data().toString(), dialog->file());
            }
        }
    }
}

void IrcTabItem::slotPopupIgnore()
{
    QModelIndex index = m_users->currentIndex();
    if(index.isValid())
    {
        NickInfo info = m_engine->getNickInfo(index.data().toString());
        UserDialog *dialog = new UserDialog(false, info.nick+"!"+info.user+"@"+info.host, m_name, getServerName(), true, this);
        if(dialog->exec() == QDialog::Accepted)
        {
            emit addIgnoreUser(dialog->nick()+" "+dialog->channels()+" "+dialog->server());
        }
    }
}

void IrcTabItem::slotPopupOp()
{
    QModelIndex index = m_users->currentIndex();
    if(index.isValid())
    {
        m_engine->sendMode(m_name+" +o "+index.data().toString());
    }
}

void IrcTabItem::slotPopupDeop()
{
    QModelIndex index = m_users->currentIndex();
    if(index.isValid())
    {
        m_engine->sendMode(m_name+" -o "+index.data().toString());
    }
}

void IrcTabItem::slotPopupVoice()
{
    QModelIndex index = m_users->currentIndex();
    if(index.isValid())
    {
        m_engine->sendMode(m_name+" +v "+index.data().toString());
    }
}

void IrcTabItem::slotPopupDevoice()
{
    QModelIndex index = m_users->currentIndex();
    if(index.isValid())
    {
        m_engine->sendMode(m_name+" -v "+index.data().toString());
    }
}

void IrcTabItem::slotPopupKick()
{
    QModelIndex index = m_users->currentIndex();
    if(index.isValid())
    {
        QDialog dialog(this);
        QVBoxLayout *mainLayout = new QVBoxLayout;
        QFormLayout *form = new QFormLayout;
        QLineEdit *reason = new QLineEdit;
        form->addRow(_("Kick reason:"), reason);
        mainLayout->addLayout(form);
        QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Close);
        connect(buttons, SIGNAL(accepted()), &dialog, SLOT(accept()));
        connect(buttons, SIGNAL(rejected()), &dialog, SLOT(reject()));
        mainLayout->addWidget(buttons);
        dialog.setLayout(mainLayout);
        dialog.setWindowTitle(_("Kick dialog"));
        if(dialog.exec() == QDialog::Accepted)
        {
            m_engine->sendKick(m_name, index.data().toString(), reason->text());
        }
    }
}

void IrcTabItem::slotPopupBan()
{
    QModelIndex index = m_users->currentIndex();
    if(index.isValid())
    {
        QDialog dialog(this);
        QVBoxLayout *mainLayout = new QVBoxLayout;
        QFormLayout *form = new QFormLayout;
        QLineEdit *mask = new QLineEdit;
        NickInfo nick = m_engine->getNickInfo(index.data().toString());
        mask->setText(nick.nick+"!"+nick.user+"@"+nick.host);
        form->addRow(_("Banmask:"), mask);
        mainLayout->addLayout(form);
        QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Close);
        connect(buttons, SIGNAL(accepted()), &dialog, SLOT(accept()));
        connect(buttons, SIGNAL(rejected()), &dialog, SLOT(reject()));
        mainLayout->addWidget(buttons);
        dialog.setLayout(mainLayout);
        dialog.setWindowTitle(_("Ban dialog"));
        if(dialog.exec() == QDialog::Accepted)
        {
            m_engine->sendMode(m_name+" +b "+mask->text());
        }
    }
}

void IrcTabItem::slotPopupKickBan()
{
    QModelIndex index = m_users->currentIndex();
    if(index.isValid())
    {
        QDialog dialog(this);
        QVBoxLayout *mainLayout = new QVBoxLayout;
        QFormLayout *form = new QFormLayout;
        QLineEdit *reason = new QLineEdit;
        form->addRow(_("Kick reason:"), reason);
        QLineEdit *mask = new QLineEdit;
        NickInfo nick = m_engine->getNickInfo(index.data().toString());
        mask->setText(nick.nick+"!"+nick.user+"@"+nick.host);
        form->addRow(_("Banmask:"), mask);
        mainLayout->addLayout(form);
        QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Close);
        connect(buttons, SIGNAL(accepted()), &dialog, SLOT(accept()));
        connect(buttons, SIGNAL(rejected()), &dialog, SLOT(reject()));
        mainLayout->addWidget(buttons);
        dialog.setLayout(mainLayout);
        dialog.setWindowTitle(_("Kick/Ban dialog"));
        if(dialog.exec() == QDialog::Accepted)
        {
            m_engine->sendKick(m_name, index.data().toString(), reason->text());
            m_engine->sendMode(m_name+" +b "+mask->text());
        }
    }
}

void IrcTabItem::slotPopupDeAway()
{
    m_engine->sendAway("");
}

void IrcTabItem::slotPopupAway()
{
    QModelIndex index = m_users->currentIndex();
    if(index.isValid())
    {
        QDialog dialog(this);
        QVBoxLayout *mainLayout = new QVBoxLayout;
        QFormLayout *form = new QFormLayout;
        QLineEdit *away = new QLineEdit;
        away->setText(_("away"));
        form->addRow(_("Message:"), away);
        mainLayout->addLayout(form);
        QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Close);
        connect(buttons, SIGNAL(accepted()), &dialog, SLOT(accept()));
        connect(buttons, SIGNAL(rejected()), &dialog, SLOT(reject()));
        mainLayout->addWidget(buttons);
        dialog.setLayout(mainLayout);
        dialog.setWindowTitle(_("Away dialog"));
        if(dialog.exec() == QDialog::Accepted)
        {
            m_engine->sendAway(away->text().isEmpty() ? _("away") : away->text());
        }
    }
}

void IrcTabItem::slotDoubleClickUsers(QModelIndex index)
{
    if(index.isValid() && index.data().toString() != getNickName())
    {
        emit ircQuery(m_engine, index.data().toString());
    }
}
