/*
 *      linespell.cpp
 *
 *      Copyright 2014 David Vachulka <arch_dvx@users.sourceforge.net>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include <QAbstractTextDocumentLayout>
#include <QKeyEvent>
#include "linespell.h"
#include "config.h"
#include "preferences.h"
#include "dxutils.h"

SpellHighlighter::SpellHighlighter(QTextDocument *parent)
    : QSyntaxHighlighter(parent), m_spellLang("")
{}

void SpellHighlighter::highlightBlock(const QString &text)
{
    if(!m_spellLang.isEmpty())
    {
        QTextCharFormat tcf;
        tcf.setUnderlineColor(QColor(255,0,0));
        tcf.setUnderlineStyle(QTextCharFormat::SpellCheckUnderline);
        QRegExp expression("\\b\\w+\\b");
        // Iterate through all words
        int index = text.indexOf(expression);
        while(index >= 0)
        {
            int length = expression.matchedLength();
            if(!dxUtils.checkWord(expression.cap(), m_spellLang))
                setFormat(index, length, tcf);
            index = text.indexOf(expression, index + length);
        }
    }
}

QString SpellHighlighter::language() const
{
    return m_spellLang;
}

void SpellHighlighter::setLanguage(const QString &lang)
{
    m_spellLang = lang;
}

LineSpell::LineSpell(QWidget *parent) :
    QTextEdit(parent), m_useSpell(preferences.m_useSpell)
{
    setWordWrapMode(QTextOption::NoWrap);
    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setTabChangesFocus(true);
    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    m_spell = new SpellHighlighter(document());
}

QSize LineSpell::minimumSizeHint() const
{
    QSize sh = QTextEdit::minimumSizeHint();
    sh.setHeight(fontMetrics().height() + 1);
    sh += QSize(0, QFrame::lineWidth() * 2);
    return sh;
}

QSize LineSpell::sizeHint() const
{
    QSize sh = QTextEdit::sizeHint();
    sh.setHeight(int(document()->documentLayout()->documentSize().height()));
    sh += QSize(0, QFrame::lineWidth() * 2);
    ((QTextEdit*)this)->setMaximumHeight(sh.height());
    return sh;
}

QString LineSpell::text() const
{
    return toPlainText();
}

void LineSpell::setText(const QString &text)
{
    setPlainText(text);
    QTextCursor cursor(textCursor());
    cursor.movePosition(QTextCursor::End, QTextCursor::MoveAnchor);
    setTextCursor(cursor);
}

int LineSpell::cursorPosition() const
{
    QTextCursor cursor(textCursor());
    return cursor.position();
}

void LineSpell::setCursorPosition(int pos)
{
    QTextCursor cursor(textCursor());
    cursor.setPosition(pos);
    setTextCursor(cursor);
}

void LineSpell::setUseSpell(bool spell)
{
    m_useSpell = spell;
    if(!m_useSpell)
    {
        m_spell->setLanguage("");
        int cursor = cursorPosition();
        setText(text());
        setCursorPosition(cursor);
    }
}

QString LineSpell::language() const
{
    return m_spell->language();
}

void LineSpell::setLanguage(const QString &lang)
{
    m_spell->setLanguage(lang);
}

void LineSpell::keyPressEvent(QKeyEvent *e)
{
    if(e->key() == Qt::Key_Enter || e->key() == Qt::Key_Return)
    {
        emit returnPressed();
        e->ignore();
    }
    else
        QTextEdit::keyPressEvent(e);
}


